package NodeDatabaseInterface;


use strict;
use POSIX qw(setsid);
use POSIX ();
use LWP;
use HTTP::Request;
use HTTP::Response;
use DBI;
use DBD::mysql;
use Carp::Assert;
use XML::Writer;
use IO::File;
use List::Util qw(max);

require "RSUtils.pm";

# read the configuration 
my $conffileOK = &RSUtils::readConfiguration("/etc/TARWIS.conf");
if(!$conffileOK) {
    print "NodeDatabaseInterface: configuration file not found or non-readable, exiting now!\n";
    exit(1);
}


# class constructor
sub new {
    my ($class) = @_;
    my $self = {
       _dbh => undef,
       _owner => undef,
    };
    
    bless $self, $class;
 
    my $host     =  $ENV{"MYSQL_DATABASE_HOST"};
    my $database =  $ENV{"MYSQL_DATABASE_NAME"};
    my $user     =  $ENV{"MYSQL_DATABASE_USER"};
    my $pw       =  $ENV{"MYSQL_DATABASE_PASSWORD"};
    my $connectionInfo="dbi:mysql:$database;$host";
    
    $self->{_dbh} = newConnection($connectionInfo, $user, $pw);
    $self->{_dbh}->{'mysql_auto_reconnect'} = 1;
    
    return $self;
}

sub owner {
    my ( $self, $owner ) = @_;
    $self->{_owner} = $owner if defined($owner);
    return $self->{_owner};
}

sub getDatabase {
    my ($self) = shift;
    return $self->{_dbh};
}


sub newConnection {
    my $connectionInfo = $_[0];
    my $user = $_[1];
    my $pw = $_[2];
    my $db = DBI->connect($connectionInfo,$user,$pw) or die "Couldn't connect to database: " . DBI->errstr;   
    return $db;
} 



#   returns a sorted list of all node URNs registered
sub getAllRegisteredSensorNodes {
    my $self = shift;
    my @rows = $self->executeSelectQuery("SELECT node_urn from sensornodes_positions");
    my @nodes = ();
    for(my $i=0; $i<@rows; $i++){
       my @singlerow = @{$rows[$i]};
       push(@nodes,@singlerow[0]);
    }
    @nodes = &RSUtils::sortURNs(@nodes);
    return @nodes;   
}

sub isNodeURNRegistered {
	my $self = shift;
	my $urn = $_[0];
	my @nodes = $self->getAllRegisteredSensorNodes(); 
	return &RSUtils::containsElement(\@nodes, $urn);
}


#   Executes any select query and returns the resulting values in an array consisting in arrays (the rows);

sub executeSelectQuery {
    my $self = shift;
    my $query = $_[0];
    
    # make connection to database
    my $dbh = $self->getDatabase();    
    # prepare and execute query
    my $sth = $dbh->prepare($query) or die "Couldn't prepare statement: " . $dbh->errstr . "\n" . "query=$query\n";
    $sth->execute() or die "Couldn't execute statement: " . $sth->errstr;
    my @allRows = ();
    while (my @data = $sth->fetchrow_array()) {
        my @row = @data;
        push(@allRows, \@row);
    }
    $sth->finish();
    return @allRows;
}

1;