package RSService;
use SOAP::Lite; #  + trace => all;
use strict;
use IPC::ShareLite;
use HTTP::Request;
use HTTP::Response;
use Env;
use Pod::WSDL;
use Data::Dumper;
use Date::Parse;
use Date::Manip qw(ParseDate UnixDate);
use SOAP::Data::ComplexType;
use SOAP::DateTime;
use Time::HiRes qw(gettimeofday usleep);
use XML::Simple;

require "ReservationDatabase.pm";
require "RSUtils.pm";
require "NodeDatabaseInterface.pm";


readConfiguration("/etc/RS.conf");
readConfiguration("/etc/TARWIS.conf");
readConfiguration("/etc/SNA.conf");
push(@INC, $ENV{"RESERVATION_SYSTEM_DIRECTORY"});


our $MAX_LINES = 1000;
our $KEY_LENGTH_CHARACTERS = 10;
our $outfile = "$ENV{'RESERVATION_SYSTEM_DIRECTORY'}"."RSServiceSOAPServer.out";
our $errfile = "$ENV{'RESERVATION_SYSTEM_DIRECTORY'}"."RSServiceSOAPServer.err";
our $raw_xml_soap_request;
our $raw_xml_soap_response;

#******************************************************************************

sub setRawXML_SOAPRequest
{
  $raw_xml_soap_request = $_[0];
}

#******************************************************************************

sub setRawXML_SOAPResponse
{
  $raw_xml_soap_response = $_[0];
}

#******************************************************************************

sub checkSOAPRequest
{
  if($_[0] eq "RSService")
  {
    for(my $i=0; $i<(@_-1); $i++)
    {
      $_[$i] = $_[$i+1];
    }
  }
}


#******************************************************************************

sub checkUserAuthentication
{
  my $secAuthKey         = $_[0];
  my $uniqueId           = $_[1];
  my $webserviceName     = $_[2];
  print "checkUserAuthentication:\n";
  
  print "secAuthKey: $secAuthKey\n";
  print "uniqueId: $uniqueId\n";
  print "webserviceName: $webserviceName\n";
    
  my @values = split(/;/, $secAuthKey);
  foreach my $value (@values) {
	my @cookieValues = split(/=/, $value);
	if(@cookieValues[0] eq "_idp_session"){
	   print "Name  = ".@cookieValues[0]."\n";
	   print "Value = ".@cookieValues[1]."\n";
	   print "getUID called: getUID $ENV{'GET_UID_URL'} " . @cookieValues[1] . "\n";
	   my @args = ("getUID", $ENV{'GET_UID_URL'}, @cookieValues[1]);
	   my @uniqueId = qx(@args);
	   $uniqueId = @uniqueId[0];
	   print "uniqueId: $uniqueId\n";
    }
  }  
    
# if empty, we are inside TARWIS, hence we do not need to obtain the uniqueID (delivered by Shibboleth Session)
#  if( ($secAuthKey ne "") && ($secAuthKey =~ m/^\_shibsession.*/)  )
#  {
#  	print "getUID called: getUID $ENV{'GET_UID_URL'} $secAuthKey\n";
#    my @args = ("getUID", $ENV{'GET_UID_URL'}, $secAuthKey);
#    my @uniqueId = qx(@args);
#    $uniqueId = @uniqueId[0];
#    print "uniqueId: $uniqueId\n";
#  }
 
  if($uniqueId eq "" || $uniqueId == 0 )
  {
    userUnknownFault($webserviceName);
  }
  
  print "checkUserAuthentication: $uniqueId\n";
  $uniqueId = &RSUtils::trimchomp($uniqueId);
  
  return $uniqueId;
}

#******************************************************************************

sub isAuthorized
{
  my $uniqueId = $_[0];
  my $action   = $_[1];
  
  my $soap_response = SOAP::Lite
        -> proxy($ENV{'SNA_AUTHORIZATION_URL'})
        -> isAuthorized($uniqueId, $action);
  my $res = $soap_response->result;                               
  print "SNA_AUTHORIZATION_URL: " . $ENV{'SNA_AUTHORIZATION_URL'} . "\n";
  print $uniqueId . " isAuthorized for " . $action . "? " . $res . "\n";
  if($res eq "user is authorized")
  {
    return 1;
  }
  else
  {
    return 0;
  }  
}

#******************************************************************************
#******************************************************************************
# WEBSERVICES
#******************************************************************************
#******************************************************************************

#******************************************************************************
# makeReservation:
# - makes a reservation of a number of given node, in a given period and for a
#   given user 
# - returns the secretReservationKey
# - SOAP Parameter:
#     <?xml version="1.0" encoding="UTF-8"?>
#     <SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="urn:RSService">
#     <SOAP-ENV:Body>
#     <ns1:makeReservation>
#     	<authenticationData>
#     		<username>858046@gridlab21.unibe.ch</username>
#     		<secretAuthenticationKey>lalalal</secretAuthenticationKey>
#     		<urnPrefix>gridlab21.unibe.ch</urnPrefix>
#     	</authenticationData>
#     	<reservation>
#     		<from>2010-05-12 10:00:00</from>
#     		<nodeURNs>urn1</nodeURNs>
#     		<nodeURNs>urn2</nodeURNs>
#     		<nodeURNs>urn3</nodeURNs>
#     		<nodeURNs>urn4</nodeURNs>
#     		<nodeURNs>urn5</nodeURNs>
#     		<to>2010-05-12 11:00:00</to>
#     		<userData>reserved</userData>
#     		<users>
#     			<urnPrefix>gridlab21.unibe.ch</urnPrefix>
#     			<username>858046@gridlab21.unibe.ch</username>
#     		</users>
#     	</reservation>
#     </ns1:makeReservation>
#     </SOAP-ENV:Body>
#     </SOAP-ENV:Envelope>
# - SOAP Return:
#     <?xml version="1.0" encoding="UTF-8"?>
#     <soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
#     <soap:Body>
#     <getReservationsResponse xmlns="urn:RSService">
#       <secretReservationKey>
#         <secretReservationKey>defrertertgd234r43t544</secretReservationKey>
#         <urnPrefix>gridlab21.unibe.ch</urnPrefix>
#       </secretReservationKey>
#     </getReservationsResponse>
#     </soap:Body>
#     </soap:Envelope>
#******************************************************************************
sub makeReservation
{
  print "makeReservation called " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  checkSOAPRequest(@_);
  print "SOAP: $raw_xml_soap_request\n";
  
  my $methodName = getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "makeReservation");
    
  my $uniqueId           	= &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "authenticationData", "username"));
  my $secAuthKey            = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "authenticationData", "secretAuthenticationKey"));
  my $urnPrefix  			= &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "authenticationData", "urnPrefix"));
  my $reservationFrom    	= &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "reservation", "from"));
  my $reservationTo         = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "reservation", "to"));
  my $reservationUserData   = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "reservation", "userData"));
  my $usersUserName	        = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "reservation", "data", "username"));
  my $usersUrnPrefix	    = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "reservation", "data", "urnPrefix"));
  my @nodeUrns              = getItemsFromSOAPBody($raw_xml_soap_request, $methodName, "reservation", "nodeURNs");
  
  my $numOfNodes = @nodeUrns;
  # print "numOfNodes " . $numOfNodes . "\n";
  # print "uniqueId " . $uniqueId . "\n";
  # print "secAuthKey " . $secAuthKey . "\n";
  # print "urnPrefix " . $urnPrefix . "\n";
  # print "reservationFrom " . $reservationFrom . "\n";
  # print "reservationTo " . $reservationTo . "\n";
  # print "reservationUserData " . $reservationUserData . "\n";
  # print "usersUserName " . $usersUserName . "\n";
  # print "usersUrnPrefix " . $usersUrnPrefix . "\n";
  # print "nodeUrns: \n";
  # print @nodeUrns;
  # print "\n";
  
  $reservationFrom = &RSUtils::convertDateTimeToMySQLFormat($reservationFrom);
  $reservationTo   = &RSUtils::convertDateTimeToMySQLFormat($reservationTo);
  
  # print "reservationFrom: $reservationFrom\n";
  # print "reservationTo: $reservationTo\n";
  
  
  print "makeReservation calls checkUserAuthentication: $uniqueId \n";
  my $uniqueId = checkUserAuthentication($secAuthKey, $uniqueId, "makeReservation");
  
  my $dbObject = new ReservationDatabase();
  my $nodeDBObject = new NodeDatabaseInterface();
  
  my $reservationKey;
  
  if($reservationUserData eq "" ||  $reservationUserData =~ /.*\@.*/ )
  {
    $reservationUserData = "reserved";
  }

  my @errorneous_nodes = ();
  my @unregistered_nodes = ();

  my $allNodesAreAvailable = 1;
  my $allNodesAreRegistered = 1;
  foreach my $nodeUrn(@nodeUrns)
  {
    my $isNodeRegistered = $nodeDBObject->isNodeURNRegistered($nodeUrn);
    if($isNodeRegistered == 0) {
      $allNodesAreRegistered = 0;
      push(@unregistered_nodes, $nodeUrn);
    }
    my $isNodeAvail = $dbObject->dbCheckNodeAvailability($nodeUrn, $reservationFrom, $reservationTo);
    # print "node available: $isNodeAvail\n";
    if($isNodeAvail == 0)
    {
      $allNodesAreAvailable = 0;
      push(@errorneous_nodes, $nodeUrn);
    }
  }
  
  if($allNodesAreRegistered == 0)
  {
  	my $urn_string = "";
    print @unregistered_nodes;
    for(my $i=0; $i<@unregistered_nodes; $i++)
    {
        $urn_string = $urn_string . ($i==0 ? "":"|") . @unregistered_nodes[$i];
    }
    # print "urn_string: $urn_string\n";
    nodesNotRegisteredFault("makeReservation", $urn_string);
  }
  elsif($allNodesAreAvailable == 1)
  {
    #check authorization
    if(isAuthorized($uniqueId, "makeReservation"))
    {
      $reservationKey = &RSUtils::generate_random_string($KEY_LENGTH_CHARACTERS);
      $dbObject->dbSaveReservation(\@nodeUrns, $reservationFrom, $reservationTo, $uniqueId, $reservationUserData, $reservationKey); 
    }
    else
    {
      userNotAuthorizedFault("makeReservation", $uniqueId);
    }
  }
  else
  {
  	my $urn_string = "";
  	print @errorneous_nodes;
  	for(my $i=0; $i<@errorneous_nodes; $i++)
  	{
  		$urn_string = $urn_string . ($i==0 ? "":"|") . @errorneous_nodes[$i];
  	}
  	print "urn_string: $urn_string\n";
  	nodesNotAvailableFault("makeReservation", $urn_string);
  }
  
  
  
  my $returnSOAP = generateSecretReservationKeySOAP($reservationKey, $usersUrnPrefix);

  print "makeReservation end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs()  . "\n\n\n";
  &RSUtils::truncateFile($outfile, $MAX_LINES);
  &RSUtils::truncateFile($errfile, $MAX_LINES);
  return $returnSOAP;
}

#******************************************************************************
# deleteReservation:
# - deletes a reservation with the given reservationKey
# - returns the success of the deletion
# - SOAP Paramter:
#     <?xml version="1.0" encoding="UTF-8"?>
#     <SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="urn:RSService">
#     <SOAP-ENV:Body>
#     <ns1:deleteReservation>
#     	<authenticationData>
#     		<username>858046@gridlab21.unibe.ch</username>
#     		<secretAuthenticationKey>lalalal</secretAuthenticationKey>
#     		<urnPrefix>gridlab21.unibe.ch</urnPrefix>
#     	</authenticationData
#     	<secretReservationKey>
#     		<secretReservationKey>sdflkdfgjkdhjkh</secretReservationKey>
#     		<urnPrefix>gridlab21.unibe.ch</urnPrefix>
#       </secretReservationKey>
#     </ns1:deleteReservation>
#     </SOAP-ENV:Body>
#     </SOAP-ENV:Envelope>
#******************************************************************************
sub deleteReservation
{
  print "deleteReservation called " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  checkSOAPRequest(@_);
  
  # print "SOAP: " . $raw_xml_soap_request . "\n";
  my $methodName = getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "deleteReservation");
  # print "MethodName: " . $methodName . "\n";
   
  my $uniqueId           = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "authenticationData", "username"));
  my $secAuthKey         = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "authenticationData", "secretAuthenticationKey"));
  my $urnPrefix          = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "authenticationData", "urnPrefix"));
  my $secResKeySecResKey = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "secretReservationKey", "secretReservationKey"));
  my $secResKeyUrnPrefix = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "secretReservationKey", "urnPrefix"));
  # print "uniqueId " . $uniqueId . "\n";
  # print "secAuthKey " . $secAuthKey . "\n";
  # print "urnPrefix " . $urnPrefix . "\n";
  # print "secResKeySecResKey " . $secResKeySecResKey . "\n";
  # print "secResKeyUrnPrefix " . $secResKeyUrnPrefix . "\n";
  
  my $uniqueId = checkUserAuthentication($secAuthKey, $uniqueId, "deleteReservation");
  my $returnSOAP;

  my $dbObject = new ReservationDatabase();
  my @rows = $dbObject->dbGetReservationOfReservationKey($secResKeySecResKey);
  # if there's no reservation for this key, the key must be invalid!
  if(@rows == 0)
  {
    @rows = $dbObject->dbGetMaintenanceBlockOfReservationKey($secResKeySecResKey);
    if(@rows == 0)
    {
  	  secretReservationKeyUnknownFault("getReservation");
	}
  }
  # my @singlerow     = @{$rows[0]};
  # my $fromTimestamp = &RSUtils::parseDate(@singlerow[1]);
  # my $toTimestamp   = &RSUtils::parseDate(@singlerow[2]);
  # my $nowTimestamp  = &RSUtils::get_UTC_time();

  # check authorization
  if(isAuthorized($uniqueId, "deleteReservation"))
  {
    my $dbObject = new ReservationDatabase();
    my $success = $dbObject->dbDeleteReservation($secResKeySecResKey);
    if($success == 1)
    {
      $returnSOAP = SOAP::Data->new(name => "return", value => $secResKeySecResKey,  type => "string");
    }
    else
    {
      dbAccessFault("deleteReservation");
    }
  }
  else
  {
    userNotAuthorizedFault("deleteReservation", $uniqueId);
  }
    
  print "deleteReservation end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  &RSUtils::truncateFile($outfile, $MAX_LINES);
  &RSUtils::truncateFile($errfile, $MAX_LINES);
  return $returnSOAP;
}

#******************************************************************************
# getReservation:
# - returns the reservation data for a given reservationKey
# - SOAP Paramter:
# - SOAP Paramter:
#     <?xml version="1.0" encoding="UTF-8"?>
#     <SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="urn:RSService">
#     <SOAP-ENV:Body>
#     <ns1:getReservation>
#     	<secretReservationKey>
#     		<secretReservationKey>sdflkdfgjkdhjkh</secretReservationKey>
#     		<urnPrefix>gridlab21.unibe.ch</urnPrefix>
#       </secretReservationKey>
#     </ns1:getReservation>
#     </SOAP-ENV:Body>
#     </SOAP-ENV:Envelope>
# - SOAP Return:
#     <?xml version="1.0" encoding="UTF-8"?><soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/"  xmlns:xsd="http://www.w3.org/2001/XMLSchema"  soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"  xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
#     <soap:Body>
#     <getReservationResponse xmlns="urn:RSService">
#       <reservationData>
#       	<from xsi:type="xsd:dateTime">2010-05-12T21:45:00</from>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:1</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:2</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:3</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:4</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:5</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:6</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:7</nodeURNs>
#       	<to xsi:type="xsd:dateTime">2010-05-12T21:59:59</to>
#       	<userData xsi:type="xsd:string">pre-processing</userData>
#       	<users>
#       		<urnPrefix xsi:type="xsd:string">gridlab21.unibe.ch</urnPrefix>
#       		<username xsi:type="xsd:string">858046@gridlab21.unibe.ch</username>
#       	</users>
#       </reservationData>
#     </getReservationResponse>
#     </soap:Body>
#     </soap:Envelope>
#******************************************************************************
sub getReservation
{
  print "getReservation called " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  checkSOAPRequest(@_);
  
  my $methodName = getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "getReservation");
     
  my $secretReservationKey = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "secretReservationKey", "secretReservationKey"));
  my $urnPrefix  		   = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "secretReservationKey", "urnPrefix"));
  
  my @urns = ();
  my @users = ();
  my $userData;
  my $fromRes = "";
  my $toRes = "";
  my $uniqueID;
  
 
  my $dbObject = new ReservationDatabase();
  my @rows = $dbObject->dbGetReservationOfReservationKey($secretReservationKey);
  
  # if there's no reservation for this key, the key must be invalid!
  if(@rows == 0)
  {
  	secretReservationKeyUnknownFault("getReservation");
  }
  
  for(my $i=0; $i<@rows; $i++)
  {
    my @singlerow = @{$rows[$i]};
    my $urn       = @singlerow[0];
    $fromRes      = @singlerow[1];
    $toRes        = @singlerow[2];
    $uniqueID     = @singlerow[3];
    $userData     = @singlerow[4];
    $secretReservationKey     = @singlerow[6];
    # print "urn:      " . $urn . "\n";
    # print "fromRes   " . $fromRes . "\n";
    # print "toRes:    " . $toRes . "\n";
    # print "uniqueID: " . $uniqueID . "\n";
    # print "userData: " . $userData . "\n";
    # print "secretReservationKey: " . $secretReservationKey . "\n";
    # print "urn:      " . $urn . "\n";
    if(!&RSUtils::containsElement(\@users, $uniqueID))
    {
    	push(@users, $uniqueID);
    }
    if(!&RSUtils::containsElement(\@urns, $urn))
    {
    	push(@urns, $urn);
    }
  }  
  @urns = &RSUtils::sortURNs(@urns);
  my $returnSOAP = getConfidentialReservationsSOAP($fromRes, \@urns, $toRes, $userData, \@users, $urnPrefix, $secretReservationKey);

  print "getReservation end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  &RSUtils::truncateFile($outfile, $MAX_LINES);
  &RSUtils::truncateFile($errfile, $MAX_LINES);
  return $returnSOAP;
}

#******************************************************************************
# getReservations:
# - return all reservations for a given period
# - does not returns the user data 
# - SOAP Parameter:
#     <?xml version="1.0" encoding="UTF-8"?>
#     <SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="urn:RSService">
#     <SOAP-ENV:Body>
#     <ns1:getReservations>
#       <from>2010-05-10 14:16:30</from>
#       <to>2010-05-10 14:16:30</to>
#     </ns1:getReservations>
#     </SOAP-ENV:Body>
#     </SOAP-ENV:Envelope>
# - SOAP Return:
#     <?xml version="1.0" encoding="UTF-8"?>
#     <soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
#     <soap:Body>
#     <getReservationsResponse xmlns="urn:RSService">
#       <reservations>
#         <from xsi:type="xsd:dateTime">2010-05-10T14:15:00</from>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:3</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:4</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:5</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:1</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:2</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:6</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:7</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:8</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:9</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:10</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:11</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:12</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:13</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:14</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:15</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:16</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:17</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:18</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:19</nodeURNs>
#         <nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:20</nodeURNs>
#         <to xsi:type="xsd:dateTime">2010-05-10T14:44:59</to>
#         <userData xsi:type="xsd:string">3</userData>
#       </reservations>
#     </getReservationsResponse>
#     </soap:Body>
#     </soap:Envelope>
#******************************************************************************
sub getReservations
{
  print "getReservations called ". &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  checkSOAPRequest(@_);
  
  # print "SOAP: " . $raw_xml_soap_request . "\n";
  my $methodName = getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "getReservations");
  # print "MethodName: " . $methodName . "\n";
  
  my $from = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "from"));
  my $to   = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "to"));
  
  $from = &RSUtils::convertDateTimeToMySQLFormat($from);
  $to   = &RSUtils::convertDateTimeToMySQLFormat($to);
  
  print "from " . $from . "\n";
  print "to " . $to . "\n";
    
  my $dbObject = new ReservationDatabase();
  my @reservations = ();
  my @reservationIDs = $dbObject->dbGetReservationIDsFromTo($from, $to);
  foreach my $reservationID (@reservationIDs)
  {
    my @urns = ();
    my $userData;
    my $fromRes = "";
    my $toRes = "";
    
    # print "listing all reservations of ID=$reservationID\n";
    my @rows = $dbObject->dbGetReservationsOfReservationID($reservationID);
    for(my $i=0; $i<@rows; $i++)
    {
      my @singlerow = @{$rows[$i]};
      my $urn       = @singlerow[0];
      $fromRes      = @singlerow[1];
      $toRes        = @singlerow[2];
      $userData     = @singlerow[4];
       
      if(!&RSUtils::containsElement(\@urns, $urn)){
      	push(@urns, $urn);
      }
      # print "urn: " . $urn . "\n";
    }
    @urns = &RSUtils::sortURNs(@urns);
    my $soap_obj =  getPublicReservationDataSOAP($fromRes, \@urns, $toRes, $userData);
    push(@reservations, $soap_obj); 
  }
  print "getReservations end ". &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n\n";
  
  &RSUtils::truncateFile($outfile, $MAX_LINES);
  &RSUtils::truncateFile($errfile, $MAX_LINES);
  return @reservations;
}

#******************************************************************************
# getConfidentialReservations:
# - return all reservations for a given period including the user data for a
#   given user (authentication key or username)
# - a user with the administrator rights gets all reservations for all users
# - SOAP Parameter:
#     <?xml version="1.0" encoding="UTF-8"?>
#     <SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="urn:RSService">
#     <SOAP-ENV:Body>
#     <ns1:getConfidentialReservations>
#         <secretAuthenticationKey>
#             <username>858047@gridlab21.unibe.ch</username>
#             <secretAuthenticationKey>sdfdfgd</secretAuthenticationKey>
#             <urnPrefix>gridlab21.unibe.ch</urnPrefix>
#         </secretAuthenticationKey>
#         <period>
#             <from>2010-05-12 00:00:00</from>
#             <to>2010-05-12 23:59:59</to>
#         </period>
#     </ns1:getConfidentialReservations>
#     </SOAP-ENV:Body>
#     </SOAP-ENV:Envelope>
# - SOAP Return:
#     <?xml version="1.0" encoding="UTF-8"?>
#     <soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
#     <soap:Body>
#     <getConfidentialReservationsResponse xmlns="urn:RSService">
#       <reservationData>
#       	<from xsi:type="xsd:dateTime">2010-05-12T21:45:00</from>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:1</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:2</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:3</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:4</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:5</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:6</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:7</nodeURNs>
#       	<to xsi:type="xsd:dateTime">2010-05-12T21:59:59</to>
#       	<userData xsi:type="xsd:string">pre-processing</userData>
#       	<users>
#       		<urnPrefix xsi:type="xsd:string">gridlab21.unibe.ch</urnPrefix>
#       		<username xsi:type="xsd:string">858046@gridlab21.unibe.ch</username>
#       	</users>
#       </reservationData>
#       <reservationData>
#       	<from xsi:type="xsd:dateTime">2010-05-12T22:00:00</from>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:1</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:2</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:3</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:4</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:5</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:6</nodeURNs>
#       	<nodeURNs xsi:type="xsd:string">urn:wisebed:node:ubern:7</nodeURNs>
#       	<to xsi:type="xsd:dateTime">2010-05-12T22:59:59</to>
#       	<userData xsi:type="xsd:string">reserved</userData>
#       	<users>
#       		<urnPrefix xsi:type="xsd:string">gridlab21.unibe.ch</urnPrefix>
#       		<username xsi:type="xsd:string">858046@gridlab21.unibe.ch</username>
#       	</users>
#       </reservationData>
#     </getConfidentialReservationsResponse>
#     </soap:Body>
#     </soap:Envelope>
#******************************************************************************
sub getConfidentialReservations
{
  print "getConfidentialReservations called " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  checkSOAPRequest(@_);
  my $methodName = getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "getConfidentialReservations");
  
  my $uniqueId    = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "secretAuthenticationKey", "username"));
  my $secAuthKey  = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "secretAuthenticationKey", "secretAuthenticationKey"));
  my $urnPrefix   = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "secretAuthenticationKey", "urnPrefix"));
  my $from        = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "period", "from"));
  my $to          = &RSUtils::trimchomp(getItemFromSOAPBody($raw_xml_soap_request, $methodName, "period", "to"));
  
  
  print "uniqueId " . $uniqueId . "\n";
  print "secAuthKey " . $secAuthKey . "\n";
  print "urnPrefix " . $urnPrefix . "\n";
  print "from " . $from . "\n";
  print "to " . $to . "\n";
 
  my $uniqueId = checkUserAuthentication($secAuthKey, $uniqueId, "getConfidentialReservations");
  print "UNIQUE ID obtained: $uniqueId\n";
  
  my $databaseObject = new ReservationDatabase();
  my @reservations = ();
  my @reservationIDs = ();
  
  if(isAuthorized($uniqueId, "maintainReservation"))
  {
  	$from = &RSUtils::convertDateTimeToMySQLFormat($from);
    $to   = &RSUtils::convertDateTimeToMySQLFormat($to);
    @reservationIDs = $databaseObject->dbGetReservationIDsFromTo($from, $to);
    # print "retrieved reservationIDs: @reservationIDs\n";
  }
  elsif(isAuthorized($uniqueId, "getReservations"))
  {
  	$from = &RSUtils::convertDateTimeToMySQLFormat($from);
    $to   = &RSUtils::convertDateTimeToMySQLFormat($to);
    @reservationIDs = $databaseObject->dbGetReservationIDsFromToUser($from, $to, $uniqueId);
    # print "retrieved reservationIDs: @reservationIDs\n";
  }
  else
  {
    userNotAuthorizedFault("getConfidentialReservations", $uniqueId);
  }

  print "retrieved reservationIDs: @reservationIDs\n";

  foreach my $reservationID (@reservationIDs)
  {
    my @urns = ();
    my @users = ();
    my $userData;
    my $fromRes = "";
    my $toRes = "";
    my $userID;
    my $secretReservationKey;
    
    # print "listing all reservations of ID=$reservationID\n";
    my @rows = $databaseObject->dbGetReservationsOfReservationID($reservationID);
    
    # print "rows: " . @rows . "\n";
    for(my $i=0; $i<@rows; $i++)
    {
      my @singlerow             = @{$rows[$i]};
      my $urn                   = @singlerow[0];
      $fromRes                  = @singlerow[1];
      $toRes                    = @singlerow[2];
      $userID                   = @singlerow[3];
      $userData                 = @singlerow[4];
      $secretReservationKey     = @singlerow[6];
      # print "urn:      " . $urn . "\n";
      # print "fromRes   " . $fromRes . "\n";
      # print "toRes:    " . $toRes . "\n";
      # print "userID:   " . $userID . "\n";
      # print "userData: " . $userData . "\n";
      # print "secretReservationKey: " . $secretReservationKey . "\n";
      # print "urn:      " . $urn . "\n";
       
      if(!&RSUtils::containsElement(\@users, $userID)){
      	push(@users, $userID);
      }
      if(!&RSUtils::containsElement(\@urns, $urn)){
      	push(@urns, $urn);
      }
    }
    # print "users:";
    # print @users;
    # print "\n";
    @urns = &RSUtils::sortURNs(@urns);
    my $soap_obj =  getConfidentialReservationsSOAP($fromRes, \@urns, $toRes, $userData, \@users, $urnPrefix, $secretReservationKey);
    push(@reservations, $soap_obj); 
  }

  print "getConfidentialReservations end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n\n";
  &RSUtils::truncateFile($outfile, $MAX_LINES);
  &RSUtils::truncateFile($errfile, $MAX_LINES);
  return @reservations;
}


#******************************************************************************
# secretReservationKeyUnknownFault
# - generates a SOAP fault if the erservation kesy is unkown
# - used by getReservation
#******************************************************************************
sub secretReservationKeyUnknownFault
{
    my $method = $_[0];
    die SOAP::Fault->faultcode("203")
                   ->faultstring("Error in $method: supplied reservation key could not be authenticated on the testbed: " . $ENV{'RS_TESTBED'} . "\n")
                   ->faultactor("RSService");
}

#******************************************************************************
# userUnknownFault
# - generates a SOAP fault if the user is unkown
# - used by checkUserAuthentication
#******************************************************************************
sub userUnknownFault
{
	my $method = $_[0];
	die SOAP::Fault->faultcode("202")
                   ->faultstring("Error in $method: user could not be authenticated")
                   ->faultactor("RSService");
}

#******************************************************************************
# userNotAuthorizedFault
# - generates a SOAP fault if the user is not authorized to call this method
# - used by ...
#******************************************************************************
sub userNotAuthorizedFault
{
	my $method = $_[0];
	my $uniqueId = $_[1];
	die SOAP::Fault->faultcode("201")
                   ->faultstring("Error in $method: user $uniqueId has no permission to call this method on the testbed: " . $ENV{'RS_TESTBED'} . "\n")
                   ->faultactor("RSService");
}

#******************************************************************************
# nodesNotRegisteredFault
# - generates a SOAP fault if there are nodes are nov available for the given
#   time period
# - used by ...
#******************************************************************************
sub nodesNotRegisteredFault
{
    my $method = $_[0];
    my $node_urns = $_[1];
    die SOAP::Fault->faultcode("102")
                   ->faultstring("Error in $method: Node(s) $node_urns are not registered on the testbed: " . $ENV{'RS_TESTBED'} . "\n")
                   ->faultactor("RSService");
}


#******************************************************************************
# nodesNotAvailableFault
# - generates a SOAP fault if there are nodes are nov available for the given
#   time period
# - used by ...
#******************************************************************************
sub nodesNotAvailableFault
{
	my $method = $_[0];
	my $node_urns = $_[1];
	die SOAP::Fault->faultcode("101")
                   ->faultstring("Error in $method: Node(s) $node_urns are not available during the full reservation time on the testbed: " . $ENV{'RS_TESTBED'} . "\n")
                   ->faultactor("RSService");
}

#******************************************************************************
# dbAccessFault
# - generates a SOAP fault if there is an error with the access of the database
# - used by checkUserAuthentication
#******************************************************************************
sub dbAccessFault
{
	my $method = $_[0];
	die SOAP::Fault->faultcode("301")
                   ->faultstring("Error in $method: accessing the database was not successful on the testbed: " . $ENV{'RS_TESTBED'} . "\n")
                   ->faultactor("RSService");
}

#******************************************************************************
# delResFault
# - generates a SOAP fault if a reservation should be deleted for a running
#   experiment 
# - used by deleteReservation
#******************************************************************************
sub delResFault
{
	my $method = $_[0];
	die SOAP::Fault->faultcode("401")
                   ->faultstring("Error in $method: a reservation of a running experiment cannot be deleted on the testbed: " . $ENV{'RS_TESTBED'} . "\n")
                   ->faultactor("RSService");
}


#******************************************************************************
#******************************************************************************
# SOAP OBJECTS GENERATION FUNCTIONS
#******************************************************************************
#******************************************************************************

#******************************************************************************
# generateSecretReservationKeySOAP
# - returns a SOAP structure (secretReservationKey)
# - used by makeReservation
#******************************************************************************
sub generateSecretReservationKeySOAP
{
  my $secretReservationKey        = $_[0];
  my $urnPrefix                   = $_[1];
  my $soapObj =         SOAP::Data->name("secretReservationKey" => \SOAP::Data->value( 
                      	SOAP::Data->new(name => "secretReservationKey", value => $secretReservationKey, type => ""),
                        SOAP::Data->new(name => "urnPrefix",            value => $urnPrefix,            type => "")));
  return $soapObj;
}

#******************************************************************************
# getConfidentialReservationsSOAP
# - returns a SOAP structure (confidentialReservationData)
# - used by makeReservation
#******************************************************************************
sub getConfidentialReservationsSOAP
{
  my $from                  = $_[0];
  my $nodeURNs              = $_[1];	
  my $to                    = $_[2];
  my $userData              = $_[3];
  my $users                 = $_[4];
  my $urnPrefix             = $_[5];
  my $secretReservationKey  = $_[6];
  
  my @urnsTags  = generateURNsTags($nodeURNs);
#   my @usersTags = generateUsersTags($users, $urnPrefix);
  my @usersDataTags = generateUsersDataTags($users, $urnPrefix, $secretReservationKey);
#   my $soapObj = SOAP::Data->name("reservationData" => \SOAP::Data->value( 
#                         SOAP::Data->new(name => "from",            value => generateSOAPDateTimeObject($from), type => "dateTime"), 
#                         @urnsTags,
#                         SOAP::Data->new(name => "to",              value => generateSOAPDateTimeObject($to),   type => "dateTime"),
#                         SOAP::Data->new(name => "userData",        value => $userData,                         type => "string"),
#                         @usersTags,
#                         generateSecretReservationKeySOAP($secretReservationKey, $urnPrefix)));
  my $soapObj = SOAP::Data->name("reservationData" => \SOAP::Data->value( 
                        SOAP::Data->new(name => "from",            value => generateSOAPDateTimeObject($from), type => "dateTime"), 
                        @urnsTags,
                        SOAP::Data->new(name => "to",              value => generateSOAPDateTimeObject($to),   type => "dateTime"),
                        SOAP::Data->new(name => "userData",        value => $userData,                         type => "string"),
                        @usersDataTags));
  return $soapObj;
}

#******************************************************************************
# getPublicReservationDataSOAP
# - returns a SOAP structure (publicReservationData)
# - used by makeReservation
#******************************************************************************
sub getPublicReservationDataSOAP
{
  my $from     = $_[0];
  my $nodeURNs = $_[1];	
  my $to       = $_[2];
  my $userData = $_[3];
  
  my @urnsTags = generateURNsTags($nodeURNs);
  my $soapObj = SOAP::Data->name("reservations" => \SOAP::Data->value( 
                        SOAP::Data->new(name => "from",         value => generateSOAPDateTimeObject($from), type => "dateTime"), 
                        @urnsTags,
                        SOAP::Data->new(name => "to",           value => generateSOAPDateTimeObject($to),   type => "dateTime"),
                        SOAP::Data->new(name => "userData",     value => $userData,                         type => "string"))); 
  return $soapObj;
}

#******************************************************************************
# generateUsersTags
# - returns a SOAP structure (user)
# - used by ...
#******************************************************************************
sub generateUsersTags
{
	my @users = @{$_[0]};
	my $urnPrefix = $_[1];
	
	my @soapTags = ();
	foreach my $userName (@users)
	{
	   my $perUserTag = SOAP::Data->name("users" => \SOAP::Data->value( 
                  			SOAP::Data->new(name => "urnPrefix",   value => $urnPrefix, type => "string"),                                         
                  			SOAP::Data->new(name => "username",    value => $userName,  type => "string")));
	   push(@soapTags, $perUserTag);
	} 
	return @soapTags;
}

#******************************************************************************
# generateUsersDataTags
# - returns a SOAP structure (data)
# - used by ...
#******************************************************************************
sub generateUsersDataTags
{
	my @users = @{$_[0]};
	my $urnPrefix = $_[1];
	my $secretReservationKey = $_[2];
	
	my @soapTags = ();
	foreach my $userName (@users)
	{
	   my $perUserTag = SOAP::Data->name("data" => \SOAP::Data->value( 
                  			SOAP::Data->new(name => "urnPrefix",   value => $urnPrefix, type => "string"),                                         
                  			SOAP::Data->new(name => "username",    value => $userName,  type => "string"),
                  			SOAP::Data->new(name => "secretReservationKey",    value => $secretReservationKey,  type => "string")));
	   push(@soapTags, $perUserTag);
	} 
	return @soapTags;
}

#******************************************************************************
# generateURNsTags
# - returns a SOAP structure (nodeURNs)
# - used by ...
#******************************************************************************
sub generateURNsTags
{
	my @urns = @{$_[0]};
	my @soapTags = ();
	foreach my $urn (@urns)
	{
    my $perNodeTag = SOAP::Data->new(name => "nodeURNs", value => $urn, type => "string");
    push(@soapTags, $perNodeTag);
	} 
	return @soapTags;
}

#******************************************************************************
# generateSOAPDateTimeObject
# - returns a SOAP datetime object for a given datetime string
# - used by ...
#******************************************************************************
sub generateSOAPDateTimeObject
{
  my $myTime = $_[0];
  my $date = ParseDate($myTime);
  my $datestr = UnixDate($date, "%a %b %e %H:%M:%S %z %Y"); 
  my $soap_datetime = ConvertDate($datestr);
  return $soap_datetime;
}


#******************************************************************************
#******************************************************************************
# SOAP OBJECTS GETTER FUNCTIONS
#******************************************************************************
#******************************************************************************

#******************************************************************************
# getTag_IncludingNamespace_FromRawSOAP
#******************************************************************************
sub getTag_IncludingNamespace_FromRawSOAP
{
  my $raw_soap = $_[0];
  my $ns = $_[1];
  
  if($raw_soap =~ m/$ns/ )
  {
    my $retVal = $raw_soap;
    while($retVal =~ m/[\r\n]/)
    {
      $retVal =~ s/[\r\n]/ /;
    }
    while( $retVal =~ m/\>\</ )
    {
      $retVal =~ s/\>\</\>\ \</;
    }
    # print "getTag_IncludingNamespace_FromRawSOAP2: $retVal\n";
    my @tokens = split(" ", $retVal);
    $retVal = $ns;
    foreach my $token (@tokens)
    {
      # print "token: $token\n";
      if($token =~ m/.*<\/.*:$ns>/)
      {
        # print "token matches\n";
        $retVal = $token;
        $retVal =~ s/\<//;
        $retVal =~ s/\///;
        $retVal =~ s/\>//;
        $retVal = &RSUtils::trimchomp($retVal);
        return $retVal;
      }
    }
    return $retVal;     
  }
  else
  {
    return -1;
  }
}

#******************************************************************************
# getItemsFromSOAPBody
#******************************************************************************
sub getItemsFromSOAPBody
{
  my $raw_soap = $_[0];
  my $xml = new XML::Simple;
  my $data = $xml->XMLin($raw_soap);
  my $Envelope_ns = getTag_IncludingNamespace_FromRawSOAP($raw_soap, "Envelope");
  my $Body_ns     = getTag_IncludingNamespace_FromRawSOAP($raw_soap, "Body");
  
  my $item_name = @_[@_-1];
  my @items = ();
  my $reference;
  
  if(&RSUtils::checkTagExists($data->{$Envelope_ns}))
  {
    $reference = $data->{$Envelope_ns};     
  }
  else
  {
    $reference = $data;
  }
  $reference = $data->{$Body_ns};
  
  # start at 1 because $_[0] is the raw xml
  for(my $i=1; $i<@_-1; $i++)
  {
    my $tag_name = @_[$i];
    $reference = $reference->{$tag_name};
  }
  $reference = $reference->{$item_name};
  
  if( ref($reference) eq "" )
  {
    # print "adding string: $reference\n";
    if(!&RSUtils::trimchomp($reference) eq "")
    {
      push(@items, $reference);
    }
  }
  elsif(ref($reference) eq 'HASH' )
  {
    # print "adding hash: $reference\n";
    # print Dumper($reference);
    my $record = &RSUtils::getValue($reference);
    push(@items, $record);
  }
  elsif(ref($reference) eq 'SCALAR')
  {
    # print "adding scalar: $reference\n";
    if(!&RSUtils::trimchomp("$reference") eq "")
    {
      push(@items, $reference);
    }
  }
  elsif(ref($reference) eq 'ARRAY')
  {
    # print "adding array: $reference\n";
    # print Dumper($reference);
    foreach my $record (@{$reference})
    {
      $record = &RSUtils::getValue($record);
      push(@items, $record);
    }
  }
  # print "getItemsFromSOAPBody end\n";
  return @items;
}

#******************************************************************************
# getItemFromSOAPBody
#******************************************************************************
sub getItemFromSOAPBody
{
  my @items = getItemsFromSOAPBody(@_);
  return @items[0];
}


sub readConfiguration
{
     my $conf = $_[0];
     open(CONFIG, $conf)  or die "Can't read file $conf :$!";   
     while (<CONFIG>) {
         chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
     return 1;
}


1;