#!/usr/bin/perl

package ControllerService;


use SOAP::Lite;
use IPC::ShareLite;
use HTTP::Request;
use HTTP::Response;
use Pod::WSDL;
use SOAP::Transport::HTTP;
use Proc::Simple;
use IO::Socket;
use POSIX qw(setsid);
use DateTime::Format::MySQL;
use Date::Parse;
use Env;
use Data::Dumper;


sub checkSOAPRequest {
    if($_[0] eq "ControllerService") {
       for(my $i=0; $i<(@_-1); $i++) {
          $_[$i] = $_[$i+1];
       }
    }
}


# read the configuration
readConfiguration("/etc/TARWIS.conf");
 
our $daemon;
our $exit = 1;
$SIG{INT} = \&closeDeamon;
$SIG{KILL} = \&closeDeamon;
$SIG{TERM} = \&closeDeamon;

print "starting PerlServer...\n";
use SOAP::Transport::TCP;

my $ip   = &Utils::getHostFromURL($ENV{"CONTROLLER_SERVICE_URL"});
my $port = &Utils::getPortFromURL($ENV{"CONTROLLER_SERVICE_URL"});


while(1 && $exit) {
   $daemon = SOAP::Transport::HTTP::Daemon
                    ->new(LocalAddr => $ip, LocalPort => $port, Listen => 50, ReuseAddr => 1);
      $daemon->dispatch_to('ControllerService');
       
   print "SOAP HTTP server listening...\n";
   print "   Host: ", $daemon->sockhost, "\n";
   print "   Port: ", $daemon->sockport, "\n";
   $daemon->handle();
}
print "closing daemon";
$daemon->close();
undef($daemon);


sub closeDeamon {
   print "signal caught! exiting\n";
   $exit = 0;  
}

sub receive {
    checkSOAPRequest(@_);
    print Dumper(@_);
    
    print "receive called! arguments:\n";
    my $sourcenodeID     = removeStuff(Dumper($_[0]{"sourceNodeID"}));
    my $timestamp        = removeStuff(Dumper($_[0]{"timestamp"}));
    my $textMessage      = removeStuff(Dumper($_[0]{"textMessage"}{"msg"}));
    my $textMessageLevel = removeStuff(Dumper($_[0]{"textMessage"}{"messageLevel"}));
    my $binaryMessage    = removeStuff(Dumper($_[0]{"binaryMessage"}{"binaryData"}));
    my $binaryType       = removeStuff(Dumper($_[0]{"binaryMessage"}{"binaryType"}));
    
    print "sourcenodeID = $sourcenodeID\n";
    print "timestamp = $timestamp\n";
    print "textmessage = $textMessage\n";
    print "binaryMessage = $binaryMessage\n";
}   

sub stripeIDfromURN {
    my $urn   = $_[0];
    # stripe away the testbed name
    $urn =~ s/.*://;
    return int($urn);
}


sub removeStuff {
    my $string = $_[0];
    $string =~ s/\$VAR1\ =\ '//;
    $string =~ s/';$//;
    return $string;
}

sub extractTextMessage {
    
}

sub extractTimestamp {
    
}

sub helloWorld {
    my $name = $_[0];
    print "function HelloWorld called!\n";
    return "Hello: " . $name;
}



sub deliverOutputFromSensorNode {
    checkSOAPRequest(@_);
    
    my $node_urn = $_[0];
    my $line = $_[1];
    my $time = $_[2];
    my $usecs = $_[3];
    
    print "deliverOutputFromSensorNode called: $node_urn $line $time $usecs\n";
}

sub flashPrograms {
    checkSOAPRequest(@_);
 
    print "function flashPrograms called!\n";
 
    my $node_urns = $_[0];
    my $images    = $_[1];
    
    my @exit_states = (1,1,1);
  
    my @soapNames1 = map{SOAP::Data->type('boolean')->value($_)}@exit_states;
    my $exit_states = SOAP::Data->name("exitstates")->value(\@soapNames1);
    return $exit_states;
}

sub readConfiguration {
     # read the configuration
     my $conf = $_[0];
     
     open(CONFIG, $conf)  or die "Can't read file $conf :$!";   
     while (<CONFIG>) {
         chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
}


1;