package SOAPServer;

use strict;
use POSIX qw(setsid);
use DateTime::Format::MySQL;
use SOAP::Transport::HTTP;
require "Utils.pm";
require "ControllerService.pm";

our $MAX_LINES = 500;

sub ControllerServiceSOAPServer {

   my $outfile = $_[0]; 
   my $errfile = $_[1];
   
   $SIG{INT}  = \&exitProcess;
   $SIG{KILL} = \&exitProcess;
   $SIG{TERM} = \&exitProcess;

   my $controller_ip   = &Utils::getHostFromURL($ENV{"CONTROLLER_SERVICE_URL"});
   my $controller_port = &Utils::getPortFromURL($ENV{"CONTROLLER_SERVICE_URL"});
	
   print "controller_ip: $controller_ip\n";
   print "controller_port: $controller_port\n";
	
   if(&Utils::portChecker($controller_ip, $controller_port)) {
	    print "A service is already running on port: $controller_port! shut it down and restart TARWIS!\n";
	    exit(1);
   }
   
   &Utils::redirectOutput($outfile,$errfile);
   
   &ControllerService::setOutfile($outfile);
   &ControllerService::setErrfile($errfile);

   # read the configuration
   my $conffileOK = &Utils::readConfiguration("/etc/TARWIS.conf");
   if(!$conffileOK) {
      print "ControllerServiceSOAPServer: configuration file not found or non-readable, exiting now!\n";
      exit(1);
   }

   my $ip   = &Utils::getHostFromURL($ENV{"CONTROLLER_SERVICE_URL"});
   my $port = &Utils::getPortFromURL($ENV{"CONTROLLER_SERVICE_URL"});

   $0 = "ControllerServiceSOAPServer_" . $port;
   
   print "starting ControllerServiceSOAPServer...\n";
   
   use SOAP::Transport::TCP;
   my $daemon;
   
   while(1) {
       $daemon = SOAP::Transport::HTTP::Daemon->new(LocalAddr => $ip, LocalPort => $port, Listen => 50, ReuseAddr => 1);
       $daemon->dispatch_to('ControllerService');
       $daemon->myuri("urn:ControllerService");
       $daemon->deserializer(MyDeserializer->new); 
       
       print "SOAP HTTP server listening...\n";
       print "   Host: ", $daemon->sockhost, "\n";
       print "   Port: ", $daemon->sockport, "\n";
       $daemon->handle();
       
       BEGIN {
          package MyDeserializer; @MyDeserializer::ISA = 'SOAP::Deserializer';
          use SOAP::Lite;
          use vars qw(@ISA);
		  require "Utils.pm";
          
          sub deserialize {
            &ControllerService::setRawXML_SOAPRequest($_[1]);
            shift->SUPER::deserialize(@_);
          }
       } 
   }
   
}

sub exitProcess {
     print "ControllerServiceSOAPServer: signal caught! exiting " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
     exit(1);  
}

sub ControllerServiceSOAPServer_workaround {

   my $outfile = $_[0]; 
   my $errfile = $_[1];

   my $controller_ip   = &Utils::getHostFromURL($ENV{"CONTROLLER_SERVICE_URL"});
   my $controller_port = &Utils::getPortFromURL($ENV{"CONTROLLER_SERVICE_URL"});
    
   print "controller_ip: $controller_ip\n";
   print "controller_port: $controller_port\n";
    
   if(&Utils::portChecker($controller_ip, $controller_port)) {
        print "A service is already running on port: $controller_port! shut it down and restart TARWIS!\n";
        exit(1);
   }
   &Utils::redirectOutput($outfile,$errfile);
   my $path = &Utils::correctPath($ENV{"TARWIS_DIRECTORY"} . "/CTRL_WORKAROUND/");
   print "path=$path\n"; 
   
   system("cd $path; ruby main.rb");
   
}



1;