#!/usr/bin/perl

use strict;
use POSIX qw(setsid);
use POSIX;
use LWP;
use HTTP::Request;
use HTTP::Response;
use LWP::UserAgent;
use File::Slurp;

$SIG{INT}  = \&exitProcess;
$SIG{KILL} = \&exitProcess;
$SIG{TERM} = \&exitProcess;


readConfiguration("/etc/TARWIS.conf");
push(@INC, $ENV{"TARWIS_DIRECTORY"} . "UBERN_TESTBED_IMPLEMENTATION");
require "UBERNUtils.pm";
our $CURRENT_DIRECTORY              = &UBERNUtils::correctPath($ENV{'TARWIS_DIRECTORY'} . "UBERN_TESTBED_IMPLEMENTATION");  
our $MSB_COMMAND_PORT               = 4000;
our $MSB_FLASH_PORT                 = 4001;      

if($ARGV[0] eq "") {	
    print "no node specified!\n";
	print "usage: DeployMSBImage.pl <msbX> <filepath>";
	exit(1);
}
if($ARGV[1] eq "") {
	if( -e "file.hex"){
	   print "using file.hex\n";
	   $ARGV[1] = "file.hex";
	}
	else {
	   print "no file specified!\n";
	   print "usage: DeployMSBImage.pl <msbX> <filepath>";
	   exit(1);
	}
}


flashNode($ARGV[0], $ARGV[1]);

our $node = $ARGV[0];

sub flashNode {
	my $urn             = $_[0];
    my $filepath        = $_[1];
    
    print "flashing node $urn\n";
    
    my $id              = &UBERNUtils::stripeIDfromURN($urn);
    my $eepc            = &UBERNUtils::getSensorNodeOfEEPC($id);
    my $number = $id;
    $number =~ s/msb//;
    $number = int($number)-1;
    # determine the host
    my $hostname = "ee0" . $number  . ".unibe.ch";
    
    print "host: $hostname\n";
    
    print "sending file, then giving command FLASH_NODE ...\n";
    
    my $text             = &UBERNUtils::encodeFile($filepath);
    
    my $success          = &UBERNUtils::sendData_flush($hostname, $MSB_FLASH_PORT, $text);
    
    if($success) {
    	print "send image file succeeded! continuing to give FLASH_NODE command!\n";
    	# important: wait some seconds such that the receiving node has REALLY written the stuff to the file!
    	sleep(5);
        my $command_success = &UBERNUtils::sendData_flush($hostname, $MSB_COMMAND_PORT, "FLASH_NODE\n");
        if($command_success) {
        	 print "FLASH_NODE command succeeded! exiting with success!\n";
        	 sleep(30);
             exit(0);
        }
        else {
        	 print "FLASH_NODE command failed! exiting!\n";
        	 exit(1);
        }
    }
    else {
    	print "send image file failed! exiting!\n";
        exit(1);
    }
}

sub readConfiguration {
     my $conf = $_[0];
     if(!-e $conf || !-r $conf) {
        print "Configuration file: $conf does not exist!\n";
        return 0;
     }
     open(CONFIG, $conf)  or return 0;   
     while (<CONFIG>) {
         chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
     return 1;
}

sub exitProcess {
     print "DeployMSBImage.pl dedicated to $node: signal caught! exiting " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
     exit(1);  
}
