package SensorNodesOutputHandler;

use strict;
use SOAP::Transport::HTTP;
use Proc::Simple;
use IO::Socket;
use Env;
use SOAP::Lite;
use Date::Parse;
use Date::Manip qw(ParseDate UnixDate);
use SOAP::Data::ComplexType;
use SOAP::DateTime;
use IPC::ShareLite;
use Time::HiRes qw(gettimeofday usleep);

my $PORT_BASE 			= 3030;
my $MSB_PORT_BASE       = 4000;
my $MAX_LINES 			= 1000;

my $START_EEPC_MAINTENANCE_BOOTS = 0;
our $USE_WORKAROUND_WITHOUT_SERVICES = 1;


# read the configuration
require "UBERNUtils.pm";
require "DatabaseInterface.pm";

sub getControllerServiceDriver {
   my $file = &Utils::correctPath($ENV{'TARWIS_DIRECTORY'}. "/WSDL/ControllerService.wsdl");
   &Utils::replaceEndpointURLinWSDL($file, $ENV{'CONTROLLER_SERVICE_URL'});
   my $controllerServiceDriver = SOAP::Lite->service("file:" . $file); 
   return $controllerServiceDriver;
}

our @sensornodes_telosB = ( "urn:wisebed:node:ubern:1",
                               "urn:wisebed:node:ubern:2",
                               "urn:wisebed:node:ubern:3",
                               "urn:wisebed:node:ubern:4",
                               "urn:wisebed:node:ubern:5",
                               "urn:wisebed:node:ubern:6",
                               "urn:wisebed:node:ubern:7",
                               "urn:wisebed:node:ubern:8",
                               "urn:wisebed:node:ubern:9",
                               "urn:wisebed:node:ubern:10",
                               "urn:wisebed:node:ubern:11",
                               "urn:wisebed:node:ubern:12",
                               "urn:wisebed:node:ubern:13",
                               "urn:wisebed:node:ubern:14",
                               "urn:wisebed:node:ubern:15",
                               "urn:wisebed:node:ubern:16",
                               "urn:wisebed:node:ubern:17",
                               "urn:wisebed:node:ubern:18",
                               "urn:wisebed:node:ubern:19",
                               "urn:wisebed:node:ubern:20",
                               "urn:wisebed:node:ubern:21",
                               "urn:wisebed:node:ubern:22",
                               "urn:wisebed:node:ubern:23",
                               "urn:wisebed:node:ubern:24",
                               "urn:wisebed:node:ubern:25",
                               "urn:wisebed:node:ubern:26",
                               "urn:wisebed:node:ubern:27",
                               "urn:wisebed:node:ubern:28",
                               "urn:wisebed:node:ubern:29",
                               "urn:wisebed:node:ubern:30",
                               "urn:wisebed:node:ubern:31",
                               "urn:wisebed:node:ubern:32",
                               "urn:wisebed:node:ubern:33",
                               "urn:wisebed:node:ubern:34",
                               "urn:wisebed:node:ubern:35",
                               "urn:wisebed:node:ubern:36",
                               "urn:wisebed:node:ubern:37",
                               "urn:wisebed:node:ubern:38",
                               "urn:wisebed:node:ubern:39",
                               "urn:wisebed:node:ubern:40"
                               );
     
our  @sensornodes_msb    = ("urn:wisebed:node:ubern:msb1", 
                              "urn:wisebed:node:ubern:msb2",
                              "urn:wisebed:node:ubern:msb3",
                              "urn:wisebed:node:ubern:msb4",
                              "urn:wisebed:node:ubern:msb5", 
                              "urn:wisebed:node:ubern:msb6",
                              "urn:wisebed:node:ubern:msb7");        


sub startGridlabMaintenanceReboot {
    my $myproc = Proc::Simple->new(); 
    $myproc->start(\&gridlabMaintenanceReboot);  
}


my $BOOT_TIME_GRIDLAB = 60;

sub gridlabMaintenanceReboot {
 
   $0 = "gridlabMaintenanceReboot";
   my $serviceName = "gridlabMaintenanceReboot";
   my $CURRENT_DIRECTORY = $ENV{'TARWIS_DIRECTORY'} . "UBERN_TESTBED_IMPLEMENTATION/";
   my $outfile = $CURRENT_DIRECTORY . "$serviceName.out";
   my $errfile = $CURRENT_DIRECTORY . "$serviceName.err";
   
   chdir '/'                   or die "Can't chdir to /: $!";
   open STDIN,  "/dev/null"    or die "Can't read /dev/null: $!";
   open STDOUT, ">$outfile"    or die "Can't write to $outfile: $!"; 
   open STDERR, ">$errfile"    or die "Can't write to $errfile: $!";   
      
   print "gridlabMaintenanceReboot started - number of nodes telosB: " . @sensornodes_telosB .  " number of nodes msb: " . @sensornodes_msb . "\n";         
      
   while(1) {   
      
	   my $time_now = &UBERNUtils::getTimeInMySQLFormat();
	   my $secs = &UBERNUtils::getMidnightAsInt()-time();   
	   
	   print "seconds until midnight: $secs\n";
	   
	   
	   sleep($secs);
	   
	   
	   my $database = new DatabaseInterface();
	       
	   my $trigger_reboot = 1;    
	       
	   foreach my $node_urn (@sensornodes_telosB) {
	      my $isScheduledNow  = int($database->isExperimentRunningWithNode($node_urn)); 
	      my $isScheduledWithinBootTime = int($database->isExperimentStartingWithNode_WithinSeconds($node_urn, $BOOT_TIME_GRIDLAB));
	      if($isScheduledNow || $isScheduledWithinBootTime) {
	      	  print "node $node_urn is busy\n";
	          $trigger_reboot = 0;
	      }
	   }
	   foreach my $node_urn (@sensornodes_msb) {
	      my $isScheduledNow            = int($database->isExperimentRunningWithNode($node_urn)); 
	      my $isScheduledWithinBootTime = int($database->isExperimentStartingWithNode_WithinSeconds($node_urn, $BOOT_TIME_GRIDLAB));
	      if($isScheduledNow || $isScheduledWithinBootTime) {
	          print "node $node_urn is busy\n";
	          $trigger_reboot = 0; 
	      }
	   }
	   
	   # if no node is busy, reboot!
	   if($trigger_reboot) {
	   	  print "REBOOTING GRIDLAB23: nodes are not busy and no experiment is upcoming\n";
	   	  system("reboot");
	   }
   }
}


sub startSensorNodesOutputHandlerDaemons_TelosB {
	
	foreach my $node_urn (@sensornodes_telosB){ 
	   my $myproc = Proc::Simple->new();  # Create a new process object
	   $myproc->start(\&sensorNodesOutputHandler, $node_urn);  
	}
	
    my $myproc = Proc::Simple->new(); 
    $myproc->start(\&telosBMaintenance, \@sensornodes_telosB);  
}

our $BOOT_TIME_TELOSB = 300;

sub hardResetNode {
    my $urn = $_[0];
    print "WSNService::hardResetNode: hardResetting node $urn\n";
    my $id = int(&UBERNUtils::stripeIDfromURN($urn));
    my $meshnode_hostname = &UBERNUtils::getMeshNodeHostnameOf($id);
    my $URL = "http://$meshnode_hostname.unibe.ch/control/rebootMeshNode.php";
    my $request = HTTP::Request->new(GET => $URL);
    my $ua = LWP::UserAgent->new;
    $ua->timeout(5);
    return 1;   
}


sub telosBMaintenance {
 
   my @nodes = @{$_[0]};
   
   $0 = "telosBMaintenance";
   my $serviceName = "telosBMaintenance";
   my $CURRENT_DIRECTORY = $ENV{'TARWIS_DIRECTORY'} . "UBERN_TESTBED_IMPLEMENTATION/";
   my $outfile = $CURRENT_DIRECTORY . "$serviceName.out";
   my $errfile = $CURRENT_DIRECTORY . "$serviceName.err";
   
   chdir '/'                   or die "Can't chdir to /: $!";
   open STDIN,  "/dev/null"    or die "Can't read /dev/null: $!";
   open STDOUT, ">$outfile"    or die "Can't write to $outfile: $!"; 
   open STDERR, ">$errfile"    or die "Can't write to $errfile: $!";   
            
   print "telosBMaintenance started - number of nodes: " . @nodes . "\n";         
            
   my $time_now = &UBERNUtils::getTimeInMySQLFormat();
   my $secs = &UBERNUtils::getMidnightAsInt()-time();   
   
   # print "seconds until midnight: $secs\n";
   my $isMidnight; 
   if($secs < 30) {
   	  $isMidnight = 1;
   } 
   else {
   	  $isMidnight = 0;
   }         
            
   while(1) {
        
       my $database = new DatabaseInterface();
       
       $time_now = &UBERNUtils::getTimeInMySQLFormat();    
       
       my $isMidnight; 
       
       $secs = &UBERNUtils::getMidnightAsInt()-time();
	   if($secs < 30) {
	      $isMidnight = 1;
	   } 
	   else {
          $isMidnight = 0;
       }
               
       foreach my $node_urn (@nodes) {
       	   # print "node: $node_urn now=$time_now\n";
       	   my $hasJustExpiredExperiment  = int($database->hasExperimentExpiredWithNode_WithinSeconds($node_urn,30));
       	   # print "hasJustExpiredExperiment: $hasJustExpiredExperiment\n";
           my $isScheduledNow            = int($database->isExperimentRunningWithNode($node_urn)); 
           # print "isScheduledNow: $isScheduledNow\n";
           my $isScheduledWithinBootTime = int($database->isExperimentStartingWithNode_WithinSeconds($node_urn, $BOOT_TIME_TELOSB));
           # print "isScheduledWithinBootTime: $isScheduledWithinBootTime\n";
           
           if( ($isMidnight || $hasJustExpiredExperiment) && !$isScheduledNow && !$isScheduledWithinBootTime) {
               print "resetting node $node_urn! isMidnight=$isMidnight timenow=$time_now\n"; 
               hardResetNode($node_urn);  
           }
       }
       $time_now = &UBERNUtils::getTimeInMySQLFormat();
       $secs = &UBERNUtils::getMidnightAsInt()-time();
       # print "seconds until midnight: $secs\n";
       $secs = Utils::min($secs, 10);
       # print "sleeping $secs and going back to loop now=$time_now\n";
       sleep($secs);
   }
}


sub startSensorNodesOutputHandlerDaemons_MSB430 {                      
                               
    my $database = new DatabaseInterface();                                   
                               
    foreach my $node_urn (@sensornodes_msb){ 
       # print "node $node_urn is currently scheduled for experiment: " . $database->isExperimentRunningWithNode($node_urn) . " "; 
       # print "within the next 2000 seconds: "                         . $database->isExperimentStartingWithNode_WithinSeconds($node_urn, 2000) . "\n";
       my $myproc = Proc::Simple->new();  # Create a new process object
       $myproc->start(\&sensorNodesOutputHandler, $node_urn);  
    }
    
    if($START_EEPC_MAINTENANCE_BOOTS) {
        my $myproc = Proc::Simple->new(); 
        $myproc->start(\&eepcMaintenance, \@sensornodes_msb);
    }  
}

our $BOOT_TIME_EEPC = 300;


sub eepcMaintenance {
 
   my @nodes = @{$_[0]};
   
   $0 = "eepcMaintenance";
   my $serviceName = "eepcMaintenance";
   my $CURRENT_DIRECTORY = $ENV{'TARWIS_DIRECTORY'} . "UBERN_TESTBED_IMPLEMENTATION/";
   my $outfile = $CURRENT_DIRECTORY . "$serviceName.out";
   my $errfile = $CURRENT_DIRECTORY . "$serviceName.err";
   
   chdir '/'                   or die "Can't chdir to /: $!";
   open STDIN,  "/dev/null"    or die "Can't read /dev/null: $!";
   open STDOUT, ">$outfile"    or die "Can't write to $outfile: $!"; 
   open STDERR, ">$errfile"    or die "Can't write to $errfile: $!";   
      
   print "eepcMaintenance started - number of nodes: " . @nodes . "\n";         
      
   my $time_now = &UBERNUtils::getTimeInMySQLFormat();
   my $secs = &UBERNUtils::getMidnightAsInt()-time();   
   # print "seconds until midnight: $secs\n";
   
   my $isMidnight; 
   
   $secs = &UBERNUtils::getMidnightAsInt()-time();
   if($secs < 30) {
      $isMidnight = 1;
   }
   else {
      $isMidnight = 0;
   }    
      
   while(1) {
   	    
       my $database = new DatabaseInterface();
       
       $time_now = &UBERNUtils::getTimeInMySQLFormat();    
       
       $secs = &UBERNUtils::getMidnightAsInt()-time();   
       if($secs < 30) {
          $isMidnight = 1;
       } 
       else {
          $isMidnight = 0;
       }         
           
       foreach my $node_urn (@nodes) {
		   my $host      = &UBERNUtils::getEEPCHostnameOf($node_urn);
           # print "node: $node_urn host: $host now=$time_now\n";
           my $hasJustExpiredExperiment  = int($database->hasExperimentExpiredWithNode_WithinSeconds($node_urn,30));
           # print "hasJustExpiredExperiment: $hasJustExpiredExperiment\n";
		   my $isScheduledNow            = int($database->isExperimentRunningWithNode($node_urn)); 
		   # print "isScheduledNow: $isScheduledNow\n";
	       my $isScheduledWithinBootTime = int($database->isExperimentStartingWithNode_WithinSeconds($node_urn, $BOOT_TIME_EEPC));
	      
	       if(($isMidnight || $hasJustExpiredExperiment) && !$isScheduledNow && !$isScheduledWithinBootTime) {
	       	   print "resetting node $node_urn at host: $host timenow=$time_now isMidnight=$isMidnight\n"; 
	           &UBERNUtils::sendData_flush($host, 4002, "RESET");	
	       }
       }
       $time_now = &UBERNUtils::getTimeInMySQLFormat();
       $secs     = &UBERNUtils::getMidnightAsInt()-time();
       # print "seconds until midnight: $secs\n";
       $secs = Utils::min($secs, 10);
       # print "sleeping $secs and going back to loop now=$time_now\n";
       sleep($secs);
   }
}


sub validTime {
   return length($_[0])>0 && $_[0] =~ /-/;
}

sub validUsecs {
   return length($_[0])>0 && &UBERNUtils::isInteger($_[0]);
}

sub validLine {
   return length($_[0])>0;
}

sub getThisMachineIP {
   my $ifconfig="/sbin/ifconfig";
   my @lines=qx|$ifconfig eth0:0| or die("Can't get info from ifconfig: ".$!);
   foreach(@lines){
      if(/inet addr:([\d.]+)/){
      return "$1";
      }
   }
}

# msb machines do not use eth0:0, they use eth0
sub getThisMachineIP_msb {
   my $ifconfig="/sbin/ifconfig";
   my @lines=qx|$ifconfig eth0| or die("Can't get info from ifconfig: ".$!);
   foreach(@lines){
      if(/inet addr:([\d.]+)/){
      return "$1";
      }
   }
}



#  for each sensor node, a process is spawned that listens on a socket for incoming
#  data of the particular sensor node

#  localmode is not used with TARWIS

sub sensorNodesOutputHandler {
 
   my $node_urn  = $_[0];
   my $localmode = $_[1];
   my $port = $PORT_BASE;
   my $id = &UBERNUtils::stripeIDfromURN($node_urn);
   
   my $serviceName = "node_output_handler_";
   
   my $CURRENT_DIRECTORY = $ENV{'TARWIS_DIRECTORY'} . "UBERN_TESTBED_IMPLEMENTATION/";
   my $local_output_file;
   
   if($localmode) {
   	  $CURRENT_DIRECTORY = $CURRENT_DIRECTORY . "LOCAL/";
   	  $local_output_file = $CURRENT_DIRECTORY . "$id.txt";
   	  &UBERNUtils::flushFile($local_output_file);
   }
   
   my $outfile = $CURRENT_DIRECTORY . "$serviceName" . "node_$id.out";
   my $errfile = $CURRENT_DIRECTORY . "$serviceName" . "node_$id.err";
   
   chdir '/'                   or die "Can't chdir to /: $!";
   open STDIN,  "/dev/null"    or die "Can't read /dev/null: $!";
   open STDOUT, ">$outfile"    or die "Can't write to $outfile: $!"; 
   open STDERR, ">$errfile"    or die "Can't write to $errfile: $!";
 
   my $is_msb430_node = 0;
   if($id =~ m/msb/) {
   	  $port = $MSB_PORT_BASE;
      my $p = $id;
      $p =~ s/msb//;
      $port += $p;
      $is_msb430_node = 1;
      # set the process names for the output handlers!
      $0 = "node_output_handler_node_" . $id . "_port_" . "$port";
   }
   else {
   	  $port += int($id);
      $0 = "node_output_handler_node_telosb_" . $id . "_port_" . "$port";
   }
   
      
   my $database = new DatabaseInterface();
   $database->resetSensorNodesOutputBufferTable();
 
   print "starting sensorNodesOutputHandler of id=$id\n";
   
   my $ip = getThisMachineIP();
   if($id =~ m/msb/) {
      $ip = getThisMachineIP_msb();
   }
   
   print "opening socket on ip:   $ip\n";      
   print "opening socket on port: $port\n";      
   
  my $sock = new IO::Socket::INET(
        LocalHost => $ip, 
        LocalPort => $port,
        Type => SOCK_STREAM,
        Listen => 50,
        Reuse => 1,
        Proto => 'tcp') || die "Error creating socket: $!";

   my $truncateCounter = 0;
   my $line_number     = 0;

   while(1) {
      while(my $client = $sock->accept()) {
          my $host=$client->peerhost();
          my $data = "";
          while (defined(my $buf=<$client>)) {
             $data .= $buf;
          }
           
          # print "received data: " . $data . "\n"; 
          
          my @lines = split("\n", $data);
          foreach my $line (@lines) {     
              (my $number, my $puredata) = extractNodeNumberAndMessage($line);    
              if( ! $number eq "" && ! $puredata eq "") {
                 my ($line, $time, $usecs) = extractLineAndTimestamp($puredata);
                 if(validLine($line) && validTime($time) && validUsecs($usecs)) {
                    if($is_msb430_node && $localmode) {
                        my $line_number_str;
                        my $usecs_str;
                        $line_number_str = sprintf("%06d", $line_number);
                        $usecs_str = sprintf("%06d", $usecs);
                        &UBERNUtils::appendStringToFile($local_output_file, $line_number_str . "|" . $time . " " . $usecs_str . "|".$line . "\n");
                        $line_number++; 
                    }
                    else {
                        $database->writeTimestampedSensorNodeOutputToOutputBuffer($node_urn, $line, $time, $usecs); 
                    }   
                 }
                 else {
                    print "invalid data: line=$line time=$time usecs=$usecs time\@gridlab=" . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n\n\n";
                 }
              }
              else {
                print "invalid data: number=$number puredata=$puredata time\@gridlab=" . &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n\n\n";
              }           
          }
          
          if($truncateCounter > ($MAX_LINES/2)) {
              &UBERNUtils::truncateFile($outfile, $MAX_LINES);
              $truncateCounter = 0;
          }
          $truncateCounter++;
          
      }
      print "looping in socket handler $ip $port\n";
   }
   close($sock);
}

sub startOutputProcessor {
    my $myproc = Proc::Simple->new();                           # Create a new process object
    $myproc->start(\&outputProcessor);  
}

sub outputProcessor {
	
    my $serviceName = "output_processor";
    my $outfile     = $ENV{'TARWIS_DIRECTORY'} . "UBERN_TESTBED_IMPLEMENTATION/$serviceName.out";
    my $errfile     = $ENV{'TARWIS_DIRECTORY'} . "UBERN_TESTBED_IMPLEMENTATION/$serviceName.err";
    
    chdir '/'                   or die "Can't chdir to /: $!";
    open STDIN,  "/dev/null"    or die "Can't read /dev/null: $!";
    open STDOUT, ">$outfile"  or die "Can't write to $outfile: $!"; 
    open STDERR, ">$errfile"  or die "Can't write to $errfile: $!";
 
    $0 = "output_processor";
    my $truncateCounter = 0;
    
    my $database = new DatabaseInterface();
    while(1){
        my ($urns, $times, $usecs, $lines) = $database->getBufferedOutput();
        if(@{$urns}>0){
            callReceive($urns, $lines, $times, $usecs);        
            if($truncateCounter > ($MAX_LINES/2)) {
              &UBERNUtils::truncateFile($outfile, $MAX_LINES);
              $truncateCounter = 0;
            }
            $truncateCounter++;
        }
        usleep(500000);
    }	
}

sub extractNodeNumberAndMessage {
    my $data = $_[0];
    $data = &UBERNUtils::trimchomp($data);
    # after the '|' sign, the line begins!
    my $indexOfSign =  index($data, '|');
    my $number = substr($data,0,$indexOfSign);
    $number =~ s/'//g;
    my $puredata = substr($data,$indexOfSign+1);
    $puredata = &UBERNUtils::trimchomp($puredata);     
    return ($number, $puredata);
}


sub extractLineAndTimestamp {
    my $outputAsString = $_[0];
    my $line = $outputAsString;
    
    my $finalline = "";
    my $finaltime = "";
    my $finalusecs = "";
    
    if( $line =~ /CURRENT_TIME/ ) {
        my $value = $line;
        my $time = $line;
        $value =~ s/CURRENT_TIME.*//;
        $time =~ s/.*CURRENT_TIME://;
        $finalline = &UBERNUtils::trimchomp($value);
        print "finalline: $finalline\n";
        $finaltime = $time;
        $finaltime =~ s/[0-9]*$//;
        $finaltime = &UBERNUtils::trimchomp($finaltime);
        $finalusecs = $time;
        $finalusecs =~ s/$finaltime\ *//;
        $finalusecs = &UBERNUtils::trimchomp($finalusecs);
        # print "finaltime: $finaltime\n";
        # print "finalusecs: $finalusecs\n";
        
    }
    return ($finalline, $finaltime, $finalusecs);
}





sub callReceive {
   
   print "callReceive called ". &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
     
   my $node_urns    = $_[0];
   my $lines        = $_[1];
   my $timestamps   = $_[2];
   my $microseconds = $_[3];
   
   if(!$USE_WORKAROUND_WITHOUT_SERVICES) { 
       my @soap_messages =  ();
       
       for(my $i=0; $i<@$node_urns; $i++){  
           my $soap_datetime = generateSOAPDateTimeObject(@$timestamps[$i], @$microseconds[$i]);
           my $soap_message  = generateComplexTypeMessageText(@$node_urns[$i], @$lines[$i], $soap_datetime);
           push(@soap_messages, $soap_message);
       }	   
    	
       eval{
       	   print "callReceive: " . @$node_urns . "\n";
           getControllerServiceDriver()->receive(@soap_messages);
       };
       if ($@){
    	   print "ControllerService not available or error in SOAP request\n";
       };
   } 
   # use the workaround: direct access to the database
   
   else {
       my $database = new DatabaseInterface();
       my @experiment_ids = ();
       foreach my $node (@$node_urns) {
       	  my $experiment_id;
       	  my $temp_index = &Utils::indexOfElement(\@experiment_ids, $node);  
          if($temp_index != -1) {
             $experiment_id = @experiment_ids[$temp_index];
          }
          else {
             $experiment_id = $database->getCurrentExperimentOfNode($node);
          }
          push(@experiment_ids, $experiment_id);
       }
       $database->writeTimestampedSensorNodeOutputToDB_arrays($node_urns, \@experiment_ids, $lines, $timestamps, $microseconds);
   }
   print "callReceive end: ". &UBERNUtils::getTime_Hours_Minutes_Secs_Usecs() . "\n";
}


sub generateSOAPDateTimeObject {
   my $myTime = $_[0];
   my $microseconds = $_[1];
   # print "myTime=" . $myTime . "\n";
   my $date = ParseDate($myTime);
   # print "date=" . $date . "\n";
   my $datestr = UnixDate($date, "%a %b %e %H:%M:%S %z %Y"); 
   # print "datestr=" . $datestr . "\n";
   my $soap_datetime = ConvertDate($datestr);
   my $date_including_microseconds;
   if(int($microseconds)>0){
     $date_including_microseconds = sprintf("%s.%06d", $soap_datetime, $microseconds);
   }
   else {
     $date_including_microseconds = $soap_datetime;
   }
   return $date_including_microseconds;
}

sub generateComplexTypeMessageText {
  
   my $node_urn      = $_[0];
   my $textData      = $_[1];
   my $soap_datetime = $_[2];
   
   # my $encoded = &UBERNUtils::encodeMessage($textData);
   # my $decoded = &Utils::decodeMessage($encoded);
   # print "generateComplexTypeMessageText($node_urn): text=$textData\n";
   # print "generateComplexTypeMessageText($node_urn): encoded=$encoded\n";
   # print "generateComplexTypeMessageText($node_urn): decoded=$decoded\n";
   
   my $soap_message = SOAP::Data->name("msg" => \SOAP::Data->value( 
                                                     SOAP::Data->new(name => "sourceNodeId",  value => $node_urn, type => "string"), 
                                                     SOAP::Data->new(name => "timestamp",     value => $soap_datetime, type => "dateTime"), 
                                                     SOAP::Data->new(name => "binaryData",    value => $textData, type => "base64Binary"), 
                                                     )
                                       ); 
   return $soap_message;                                    
}




1;