package WSNService;

################## UBERN IMPLEMENTATION OF THE MANAGEMENT INTERFACE USING MESH BACKBONE ######################################

use strict;
use POSIX qw(setsid);
use POSIX;
use LWP;
use LWP::UserAgent;
use File::Slurp; 
use HTTP::Request;
use HTTP::Response;
use Env;
use Pod::WSDL;
use Data::Dumper;
use Net::Ping;
use Coro;
use Storable;
use Data::Dumper;
use Time::HiRes qw(gettimeofday usleep);


require "UBERNUtils.pm";
require "CommonTypes.pm";

&UBERNUtils::readConfiguration("/etc/TARWIS.conf");
push(@INC, $ENV{"TARWIS_DIRECTORY"});
push(@INC, $ENV{"TARWIS_DIRECTORY"} . "UBERN_TESTBED_IMPLEMENTATION");


our $CURRENT_DIRECTORY 				= &UBERNUtils::correctPath($ENV{'TARWIS_DIRECTORY'} . "UBERN_TESTBED_IMPLEMENTATION");  
our $MAX_TIMEOUT_SENSORNODES 		= 5;
our $WISEML_FILE 					= $CURRENT_DIRECTORY . "/UBERN_TESTBED.wiseml";
our $ControllerServiceDriver 		= 0;
our $raw_xml_soap_request;
our $experiment_id;
our $outfile;
our $errfile;
our $MAX_LINES 						       = 500;

our $VERSION_STRING                        = "3.0";
    
our $MSB_COMMAND_PORT                      = 4000;    
our $MESHNODES_WITH_TELOS_B_NODES_ATTACHED = 40;
our $EE_PCS_WITH_MSB430_NODES_ATTACHED     = 10;    
    
our $MAX_ATTEMPTS                          = 2;
    
sub setOutfile {	
	$outfile = $_[0];
}

sub setErrfile {    
    $errfile = $_[0];
}

sub setRawXML_SOAPRequest {
	$raw_xml_soap_request = $_[0];
}

sub setExperimentID {
    $experiment_id = $_[0];
}

sub getControllerServiceDriver {
   if($ControllerServiceDriver == 0) {
	   my $client = SOAP::Lite->new();
	   $client->proxy($ENV{'CONTROLLER_SERVICE_URL'}, timeout => $ENV{'SOAP_TIMEOUT'});
       $ControllerServiceDriver = $client->service("file:" . &Utils::correctPath($ENV{'TARWIS_DIRECTORY'}. "/WSDL/ControllerService.wsdl"));
           
   }
   return $ControllerServiceDriver;
}

our $requestIDCounter = 0;

sub getVersion {
	print "WSNService::getVersion called (experiment_id=$experiment_id) " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
	my $version = SOAP::Data->name("return")->type("string")->value($VERSION_STRING);
	print "WSNService::getVersion end " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
	return $version;
}

sub generateRequestID {
    $requestIDCounter++;
    return "request#$experiment_id#$requestIDCounter";
}

sub resetNodes {
    checkSOAPRequest(@_);
    
    print "WSNService::resetNodes called " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    
    my $method_name = &UBERNUtils::getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "resetNodes"); 
    my @urns        = &UBERNUtils::getItemsFromSOAPBody($raw_xml_soap_request, $method_name, "nodes");
    
    my $database = new DatabaseInterface();
    my $requestID = generateRequestID();
    
    foreach my $urn (@urns){
    	print "urn: $urn\n";
        $database->setAsynchronousStateOfNode($requestID, $urn, 0); 
    }
     
    $database->setAsynchronousRequestType($requestID, "resetNodes");
    
    print "requestID: $requestID\n";
   
    my $requestID_soap = SOAP::Data->name("return")->value($requestID);
    
    print "WSNService::resetNodes end " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    
    return $requestID_soap;
}

sub doResetOperations {
    
    print "WSNService::doResetOperations called " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    my $node_urns = $_[0];
    my $requestID = $_[1];
    
  
    my @status = ();
    my @procsarray = ();
    foreach my $urn (@$node_urns){
    	my $procHandle = startResetNodeProcess($urn);    
        print "resetting node $urn...\n";
        push(@procsarray,  $procHandle);
    }
    
    my $waitForProcesses        = 1;
    my $cycles                  = 0;
    my $MAX_WAIT_TIME           = $ENV{'WAIT_PERIOD_AFTER_REBOOT'} * 1000000;
    
    if($MAX_WAIT_TIME <= 0) {
        $MAX_WAIT_TIME = 0;
    }
    my $WAIT_CYCLES             = 20;
    
    while($waitForProcesses==1 && $cycles < $WAIT_CYCLES ) {
       my $done = 1;
       for(my $i=0; $i<@procsarray; $i++) {
           my $myproc = @procsarray[$i];
           my $running = $myproc->poll();
           $done = $done && (!$running); 
       }
       if($done == 1) {
       	   print "all processes are done\n";
           $waitForProcesses = 0;
       }
       else {
           print "not done, continue " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
       }
       $cycles++;
       usleep($MAX_WAIT_TIME/$WAIT_CYCLES);
    }
    

    
    # after flashing is done, collect the exit states of the procs!
    
    print "collecting exit states! cycles=$cycles\n";
    
    my @exit_states = ();
    for(my $i=0; $i<@procsarray; $i++) {
       # check if the exit status equals 0 !
       if(@procsarray[$i]->exit_status()==0) {
          push(@status, 1) ;
       }
       else {
          push(@status, 0) ;	
       }
       print "proc: $i status: " . @status[$i] . "\n";
    }
    
    for(my $i=0; $i<@procsarray; $i++) {
       my $myproc = @procsarray[$i];
       $myproc->kill()
    }
    usleep(500000);
    for(my $i=0; $i<@procsarray; $i++) {
       my $myproc = @procsarray[$i];
       $myproc->wait();
    }
        
    
    deliverMultipleStatus_SingleCall($node_urns, \@status, $requestID);
    
    
    print "WSNService::doResetOperations ended " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    
}

sub startResetNodeProcess {
    my $node_urn  = $_[0];
    my $myproc = Proc::Simple->new();  # Create a new process object
    if( $node_urn =~ m/msb/ ) {   
        my $cmd = "perl " . $CURRENT_DIRECTORY . "/ResetMSBNode.pl $node_urn";
        $myproc->start($cmd);	
    }
    else {
        my $cmd = "perl " . $CURRENT_DIRECTORY . "/HardResetNode.pl $node_urn";
        $myproc->start($cmd);
    }
    
    return $myproc; 
}


     
sub deployImageToNode {
    my $urn    = $_[0];
    my $image  = $_[1];
    my $myproc = Proc::Simple->new();  # Create a new process object
    if( $urn =~ m/msb/ ) {   
    	my $cmd = "perl " . $CURRENT_DIRECTORY . "/DeployMSBImage.pl $urn $image";
        $myproc->start($cmd);
    }
    else {
        my $cmd = "perl " . $CURRENT_DIRECTORY . "/DeployImage.pl $urn $image";
        $myproc->start($cmd);
    }
    return $myproc; 
}

sub fetchParameterAsBinary64 {
    checkSOAPRequest(@_);
    my $param = $_[0];
    my $file = &UBERNUtils::decode_base64($param);
    return "fetchParameterAsBinary64=" . $file . "\n";
}

# ControllerService sends
#
# <?xml version="1.0" encoding="UTF-8"?>
#	<soap:Envelope soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
#	xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" 
#	xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" 
#	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
#	xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
#	xmlns:tns="urn:WSNService">
#	<soap:Body>
#		<tns:flashPrograms>
#			<nodeIds xsi:type="xsd:string">urn:wisebed:node:ubern:10</nodeIds>
#			<nodeIds xsi:type="xsd:string">urn:wisebed:node:ubern:12</nodeIds>
#			<nodeIds xsi:type="xsd:string">urn:wisebed:node:ubern:17</nodeIds>
#			<nodeIds xsi:type="xsd:string">urn:wisebed:node:ubern:18</nodeIds>
#			<nodeIds xsi:type="xsd:string">urn:wisebed:node:ubern:19</nodeIds>
#			<nodeIds xsi:type="xsd:string">urn:wisebed:node:ubern:20</nodeIds>
#			<programIndices xsi:type="xsd:int">0</programIndices>
#			<programIndices xsi:type="xsd:int">0</programIndices>
#			<programIndices xsi:type="xsd:int">0</programIndices>
#			<programIndices xsi:type="xsd:int">0</programIndices>
#			<programIndices xsi:type="xsd:int">0</programIndices>
#			<programIndices xsi:type="xsd:int">0</programIndices>
#			<programs xsi:type="tns:program">
#				<program xsi:type="xsd:base64Binary">BINARYBINARYBINARY</program>
#				<metaData>
#					<version xsi:type="xsd:string" />
#					<name xsi:type="xsd:string">Prog1</name>
#					<platform xsi:type="xsd:string" />
#					<other xsi:type="xsd:string">other</other>
#			    </metaData>
#			</programs>
#           <programs xsi:type="tns:program">
#               <program xsi:type="xsd:base64Binary">BINARYBINARYBINARY</program>
#               <metaData>
#                   <version xsi:type="xsd:string" />
#                   <name xsi:type="xsd:string">Prog1</name>
#                   <platform xsi:type="xsd:string" />
#                   <other xsi:type="xsd:string">other</other>
#               </metaData>
#           </programs>
#		</tns:flashPrograms>
#	</soap:Body>
# </soap:Envelope>



sub flashPrograms {
    checkSOAPRequest(@_);
    
    print "WSNService::flashPrograms called " . &UBERNUtils::getTimeInMySQLFormat() . "\n";  
    # print "SOAP: $raw_xml_soap_request\n"; 
    my $method_name = &UBERNUtils::getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "flashPrograms"); 
    # print "MethodName: $method_name\n";
    
    
    my @binaryPrograms = ();
	my @sn_ids = ();
     
     
    my @urns            = &UBERNUtils::getItemsFromSOAPBody($raw_xml_soap_request,  $method_name, "nodeIds");
    my @program_indices = &UBERNUtils::getItemsFromSOAPBody($raw_xml_soap_request,  $method_name, "programIndices");
    my @programs        = &UBERNUtils::getHashesFromSOAPBody($raw_xml_soap_request, $method_name, "programs");
    
    foreach my $urn (@urns){
    	print "urn: $urn\n";
    }
    
    foreach my $programIndices (@program_indices){
    	print "programIndices: $programIndices\n";
    }
    
    foreach my $program (@programs){
    	if(&UBERNUtils::checkIfPropertyIsDefined($program, "program")) {
	    	my $binaryProgram = &UBERNUtils::getValue($program->{program});
	    	push(@binaryPrograms, $binaryProgram);
	    	print "program found...\n";
	    } 
    	else {
    	   print STDERR "Error in flashPrograms! binary program cannot be found in tag <program>\n";
           print STDOUT "Error in flashPrograms! binary program cannot be found in tag <program>\n";
           return;	
    	}
    }
	
    for(my $i=0; $i<@urns; $i++){
        my $node_urn = @urns[$i];
        my $sn_id = &UBERNUtils::stripeIDfromURN($node_urn);
        push(@sn_ids, $sn_id);
        print "sn_id: $sn_id\n";
    }
    
	my $requestID = generateRequestID();    
    
    store \@urns, 			 &Utils::correctPath($CURRENT_DIRECTORY  . "/node_urns.dat_$requestID");
    store \@program_indices, &Utils::correctPath($CURRENT_DIRECTORY . "/program_indices.dat_$requestID");
    store \@binaryPrograms,  &Utils::correctPath($CURRENT_DIRECTORY  . "/binaryPrograms.dat_$requestID");
    store \@sn_ids, 		 &Utils::correctPath($CURRENT_DIRECTORY  . "/sn_ids.dat_$requestID");
 
    print "WSNService::flashPrograms end " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
 
    my $requestID_soap = SOAP::Data->name("return")->value($requestID);
    
    &UBERNUtils::truncateFile($errfile, $MAX_LINES);
    &UBERNUtils::truncateFile($outfile, $MAX_LINES);
    
    
    return $requestID_soap;
}

my $SECURITY_GAP_FLASHING = 20;

sub doFlashOperations {

    my $requestID = $_[0];
 	
	print "WSNService::doFlashOperations called reqID=$requestID " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    
    my $binaryPrograms   = retrieve(&UBERNUtils::correctPath($CURRENT_DIRECTORY . "/binaryPrograms.dat_$requestID"));
	my $sn_ids           = retrieve(&UBERNUtils::correctPath($CURRENT_DIRECTORY . "/sn_ids.dat_$requestID"));
	my $program_indices  = retrieve(&UBERNUtils::correctPath($CURRENT_DIRECTORY . "/program_indices.dat_$requestID"));
	my $node_urns  		 = retrieve(&UBERNUtils::correctPath($CURRENT_DIRECTORY . "/node_urns.dat_$requestID"));
			
	foreach my $node (@$sn_ids) {
		print "node to be flashed: $node\n";
	}		
			
	unlink(&UBERNUtils::correctPath($CURRENT_DIRECTORY . "/binaryPrograms.dat_$requestID"));
	unlink(&UBERNUtils::correctPath($CURRENT_DIRECTORY . "/sn_ids.dat_$requestID"));
	unlink(&UBERNUtils::correctPath($CURRENT_DIRECTORY . "/program_indices.dat_$requestID"));
	unlink(&UBERNUtils::correctPath($CURRENT_DIRECTORY . "/node_urns.dat_$requestID"));
	
			
    # to hold the pointers to the processes!
    my @procsarray   = ();
    my @sensornodes  = ();
    my %procNodeHash = ();
    
    my @filepaths = ();      
    
    for(my $currentMeshNode=0; $currentMeshNode < $MESHNODES_WITH_TELOS_B_NODES_ATTACHED; $currentMeshNode++){
       my $sensornode= getSensorNodeOfMeshNode($currentMeshNode);
              
       if( &UBERNUtils::containsElement($sn_ids, $sensornode) ){ 
          my $nodeIndex = &UBERNUtils::indexOfElement($sn_ids,$sensornode);
          my $programIndex = @$program_indices[ $nodeIndex ];
        
          print "flash node of mesh node $currentMeshNode with program $programIndex\n";
          my $image = @$binaryPrograms[$programIndex];
          my $filepath = &UBERNUtils::correctPath($CURRENT_DIRECTORY . "/image_" . $programIndex . "_" . $requestID);
          &UBERNUtils::flushFile($filepath);
          &UBERNUtils::decodeAndWriteToFile($image,$filepath);
          
          if( ! &UBERNUtils::containsElement(\@filepaths, $filepath)) {
          	  push(@filepaths, $filepath);
          }
         
          my $procHandle = deployImageToNode($sensornode, $filepath);    
          usleep(200000);
          
          print "image $filepath deployed at node $sensornode...\n";
             
          push(@procsarray,  $procHandle);
          push(@sensornodes, $sensornode);
          $procNodeHash{ $sensornode } = $procHandle;
       }
    }          
    
    for(my $currentEEPC=0; $currentEEPC<$EE_PCS_WITH_MSB430_NODES_ATTACHED; $currentEEPC++){
       my $sensornode = &UBERNUtils::getSensorNodeOfEEPC($currentEEPC);
       if( &UBERNUtils::containsElement($sn_ids, $sensornode) ){ 
       	  my $nodeIndex = &UBERNUtils::indexOfElement($sn_ids,$sensornode);
          my $programIndex = @$program_indices[ $nodeIndex ];
          print "flash node of EEPC $currentEEPC with program $programIndex\n";
          my $image = @$binaryPrograms[$programIndex];
          my $filepath = &UBERNUtils::correctPath($CURRENT_DIRECTORY  . "/image_" . $programIndex . "_" . $requestID);
          &UBERNUtils::flushFile($filepath);
          &UBERNUtils::decodeAndWriteToFile($image,$filepath);
          
          if( ! &UBERNUtils::containsElement(\@filepaths, $filepath)) {
              push(@filepaths, $filepath);
          }
          my $procHandle = deployImageToNode($sensornode, $filepath);    
          print "image $filepath deployed at node $sensornode...\n";
          
          push(@procsarray,  $procHandle);
          push(@sensornodes, $sensornode);
          $procNodeHash{ $sensornode } = $procHandle;
       }	
    }
    
    
    my $waitForProcesses = 1;
    my $cycles = 0;
    my $MAX_WAIT_TIME 		   = int($ENV{'WAIT_PERIOD_BEFORE_FLASH_RETRY'})-$SECURITY_GAP_FLASHING;
    my $CYCLE_WAIT_INTERVAL    = 5;
    
    print "going to wait loop! MAX_WAIT_TIME = $MAX_WAIT_TIME\n";
    
    my $started = time();
    my $end_time = $MAX_WAIT_TIME + $started;
    my $time_elapsed = 0;
    
    while($waitForProcesses==1 && $end_time > time()) {
       my $done = 1;
       sleep(5);
       for(my $i=0; $i<@procsarray; $i++) {
           my $myproc = @procsarray[$i];
           my $running = $myproc->poll();
           if($running == 1) {
              $done = 0;
           } 
       }
       if($done==1) {
           $waitForProcesses = 0;
       }
       else {
       	   $time_elapsed = time() - $started;
           print "not done, elapsed = $time_elapsed continue " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
           sleep(1);
       }
       $cycles++;
    }
    
    # after flashing is done, collect the exit states of the procs!
    
    $time_elapsed = time() - $started;
    print "elapsed: $time_elapsed\n";
    print "collecting exit states! cycles=$cycles\n";
    
    my @exit_states = ();
    for(my $i=0; $i<@$sn_ids; $i++) {
       my $myproc = %procNodeHash->{@$sn_ids[$i]};
       push(@exit_states, $myproc->exit_status());
    }
    
    
    my @return_values = ();
    for(my $i=0; $i<@exit_states; $i++) {
       if(@exit_states[$i]==0) {
       	   print "state at node: " . @$sn_ids[$i] . " is: 100\n";
           push(@return_values, 100);
       } 
       else{
       	   print "state at node: " . @$sn_ids[$i] . " is: -1\n";
           push(@return_values, -1);
       }
    }  
    
    
    deliverMultipleStatus_SingleCall($node_urns, \@return_values, $requestID);
    
    # kill remaining processes
    for(my $i=0; $i<@procsarray; $i++) {
       if(@exit_states[$i]!=0) {
          print "killing proc at $i\n";
          my $myproc = @procsarray[$i];
          $myproc->kill()
       }
    }
    usleep(500000);
    for(my $i=0; $i<@procsarray; $i++) {
       my $myproc = @procsarray[$i];
       $myproc->wait();
    }
    
    
    # cleaning up, remove all temporary files 
    for(my $i=0; $i< @filepaths; $i++) {
    	if( -e @filepaths[$i]) {
    		unlink(@filepaths[$i]);
    	}
    }
    
    print "doFlashOperations End " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
}

sub deliverStatusSOAP {
   my $urn    = $_[0];
   my $status = $_[1];
   my $requestID = $_[2];
   print "deliverStatusSOAP: urn: $urn status: $status requestId: $requestID " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
   my $soap_message = generateComplexStatusMessage($requestID, $urn, $status, "status update");
   
   my $failed = 0;
   
   for(my $i=0; $i<$MAX_ATTEMPTS; $i++) {
       eval{
           getControllerServiceDriver()->receiveStatus($soap_message);
       };
       
       if ($@) {
           print STDERR "WSNService::deliverMultipleStatus_SingleCall failed! " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
           print STDERR "urns: $urn\n";
           if($i<$MAX_ATTEMPTS) {
               print STDERR "retrying...\n";
           }
           $failed = 1;
       };
       
       if(!$failed) {
       	   print "deliver status succeeded!\n";
       	   last;
       }
   }
   print "deliverStatusSOAP end " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
}

sub deliverMultipleStatus_SingleCall {
   my @urns      = @{$_[0]};
   my @status    = @{$_[1]};
   my $requestID = $_[2];
   
   print "deliverMultipleStatus_SingleCall requestId: $requestID " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
   my $soap_message = generateMultipleComplexStatusMessage($requestID, \@urns, \@status, "status update");
   
   my $failed = 0;
      
   for(my $i=0; $i<$MAX_ATTEMPTS; $i++) {
	   eval{
	       getControllerServiceDriver()->receiveStatus($soap_message);
	   };
	   if ($@) {
	       print STDERR "WSNService::deliverMultipleStatus_SingleCall failed! ". &UBERNUtils::getTimeInMySQLFormat() . "\n";
	       print STDERR "urns: @urns\n";
	       if($i<$MAX_ATTEMPTS) {
	       	   print STDERR "retrying...\n";
	       }
	       $failed = 1;
	   };
	   
	   if(!$failed) {
	   	   print "deliver status succeeded!\n";
           last;
       }
   }
   
   print "deliverMultipleStatus_SingleCall end " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
}



sub generateSoapStatus {
	my $nodeId = $_[0];
    my $value  = $_[1];
    my $msg    = $_[2];
    
	my $soap_obj = SOAP::Data->name("status" => \SOAP::Data->value( SOAP::Data->new(name => "nodeId", value => $nodeId, type => "string"),
                                                                    SOAP::Data->new(name => "value", value => $value , type => "int"),
                                                                    SOAP::Data->new(name => "msg", value => $msg , type => "string"))); 
    return $soap_obj;
}


sub generateMultipleComplexStatusMessage {
  
   my $requestId     = $_[0];
   my @urns          = @{$_[1]};
   my @status        = @{$_[2]};
   my $msg           = $_[3];
 
   # first the status' 
   my @soap_objects = ();
    for(my $i=0; $i<@urns; $i++) {
       my $soap_status = generateSoapStatus(@urns[$i],@status[$i], $msg);
       push(@soap_objects, $soap_status);
   }
   # then the requestID
   my $soap_obj_urn = SOAP::Data->new(name => "requestId",   value => $requestId,     type => "string");
   push(@soap_objects, $soap_obj_urn);
  
   # then the resulting final message
   my @result = ();
   for(my $i=0; $i<@soap_objects; $i++) {
     my $soap_obj = SOAP::Data->value(@soap_objects[$i]);
     push(@result, $soap_obj);
   }                                       
   my $finalMessage = SOAP::Data->name("status" =>  \SOAP::Data->value(  @result ) ) ;
   return $finalMessage;
}

sub generateComplexStatusMessage {
  
   my $requestId     = $_[0];
   my $nodeId        = $_[1];
   my $value         = $_[2];
   my $msg           = $_[3];
 
   my $soap_message = SOAP::Data->name("status" => \SOAP::Data->value( 
                                                  SOAP::Data->new(name => "requestId",  value => $requestId), 
                                                  SOAP::Data->name("status" => \SOAP::Data->value( 
                                                                                SOAP::Data->new(name => "nodeId", value => $nodeId, type => "string"),
                                                                                SOAP::Data->new(name => "value", value => $value , type => "int"),
                                                                                SOAP::Data->new(name => "msg", value => $msg , type => "string")
                                                                                                  )
                                                                  ) )
                                       ); 
   return $soap_message;                                    
}

sub flashAllSensorsAttachedToMeshNode {
    my $meshnode = $_[0];
    my $file     = $_[1];
    my $index    = $_[2];
   
    my $ua = LWP::UserAgent->new;
    $ua->timeout(200);
    my $url = "http://$meshnode.unibe.ch/control/flashNodes.php";
    my $field_name = "file";
    
    my $value = read_file( $file , binmode => ':raw' , scalar_ref => 1 );
    my $response = $ua->post( $url,
                            Content_Type => 'form-data',
                            Content => [ $field_name => ["$file"] ]
                             );
    if( $response->content =~ /succeeded/ ) {
        exit(0)
    }
    else {
        exit(1);
    }
}


# returns an associative array (map) with "key" -> "value"
# from the entire output of the sensor (entire outfile!), it seeks the freshest value!

sub extractLatestSensorValueOf {
    my $property = $_[0];
    my $data = $_[1];
    
    my @lines = split(/\n/,$data);
    @lines = &UBERNUtils::purifyData( @lines );
    @lines = reverse(@lines);
    my %propertyValues;
    my $contikiPropertyName = getContikiName($property);
    
    for(my $i=0; $i<@lines; $i++) {
        my $line = @lines[$i];
        if( $line =~ /$contikiPropertyName/ ) {
            # extract property...
            my $tempLine = $line;
            $tempLine =~ s/$contikiPropertyName//;
            $tempLine =~ s/CURRENT_TIME.*//;
            $tempLine = &UBERNUtils::trimchomp($tempLine);
            $propertyValues{"$property"} = $tempLine;
            # and time...
            $tempLine = $line;
            $tempLine =~ s/.*CURRENT_TIME://;
            $tempLine = &UBERNUtils::trimchomp($tempLine);
            my @tokens = split(/\ /, $tempLine);
            $propertyValues{"time"} = @tokens[0]." ".@tokens[1];
            $propertyValues{"usecs"} = @tokens[2];  
            return %propertyValues;
        }
    }
        
    
}

sub getPropertyValueOf {
    checkSOAPRequest(@_);
    
    # <?xml version="1.0" encoding="UTF-8"?>
    # 	<soap:Envelope soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
    # 	xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" 
    # 	xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" 
    #	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
    # 	xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
    #	xmlns:tns="urn:WSNService">
    #		<soap:Body>
    #			<tns:getPropertyValueOf>
    #			<node xsi:type="xsd:string">urn:wisebed:node:ubern:20</node>
    #			<propertyName xsi:type="xsd:string">urn:wisebed:node:capability:light_2</propertyName>
    #			</tns:getPropertyValueOf>
    #		</soap:Body>
    #	</soap:Envelope>
    
    
    print "WSNService::getPropertyValueOf called " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    
    my $method_name = &UBERNUtils::getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "getPropertyValueOf"); 
    
    # print "SOAP: " . $raw_xml_soap_request . "\n";
    # print "MethodName: $method_name\n";
    
    my $node         = &UBERNUtils::getItemFromSOAPBody($raw_xml_soap_request, $method_name,"node");
    my $propertyName = &UBERNUtils::stripePropertyNamefromURN(getItemFromSOAPBody($raw_xml_soap_request, $method_name,"propertyName"));
    
    print "node=$node ";
    print "propertyName=$propertyName\n";
    
    # Assumption in UBERN_IMPLEMENTATION: property value is within 
    # the last 100s
    
    my $PROPERTY_VALUE_INTERVAL = 100;
    
    my @times;
    my @usecs;
    my @lines;
    
   
    my $database = new DatabaseInterface();
    my ($ref1, $ref2, $ref3) = $database->getLastNSecsOutputOfNode($node, $PROPERTY_VALUE_INTERVAL, \@times, \@usecs, \@lines);
     
    @times = @$ref1;
    @usecs = @$ref2;
    @lines = @$ref3;
    
    my $value = 0.0;
    
    for(my $i=0; $i<@lines; $i++){
       my $line = @lines[$i];
       
       my $contikiPropertyName = getContikiName($propertyName);
       
       if($line =~ /$contikiPropertyName/ ) {
          $line =~ s/$contikiPropertyName//;
          $line =~ s/CURRENT_TIME.*//;
          $line = &UBERNUtils::trimchomp($line);
          $value = $line;
       }
    }
    
    print "WSNService::getPropertyValueOf end (value=$value)" . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    
    my $returnValue_soap = SOAP::Data->name("return")->value($value)->type("string");
    
    &UBERNUtils::truncateFile($errfile, $MAX_LINES);
    &UBERNUtils::truncateFile($outfile, $MAX_LINES);
    
    return $returnValue_soap;
}



sub getContikiName {
    my $property = $_[0];
    if($property eq "light_1") {
        return "Light 1";
    }
    if($property eq "light_2") {
        return "Light 2";
    }
    if($property eq "temperature") {
        return "Temperature";
    }
    if($property eq "relative_humidity") {
        return "Relative humidity";
    }
    if($property eq "rssi") {
        return "RSSI";
    }
	return undef;    
}

#  specify the command, the id, and the time the mesh node shall wait until sending back the 
#  serial output
# 
#
#	<?xml version="1.0" encoding="UTF-8"?>
#		<soap:Envelope soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
#		xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" 
#		xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" 
#		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
#		xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
#		xmlns:tns="urn:WSNService">
#			<soap:Body>
#				<tns:send>
#					<nodeIds xsi:type="xsd:string">urn:wisebed:node:ubern:1</nodeIds>
#					<msg>
#						<sourceNodeID xsi:type="xsd:string">user input</sourceNodeID>
#						<timestamp xsi:type="xsd:dateTime">2010-04-16T12:20:35.214691</timestamp>
#						<textMessage>
#							<msg xsi:type="xsd:string">help</msg>
#						</textMessage>
#					</msg>
#				</tns:send>
#			</soap:Body>
#	</soap:Envelope>

sub send {
    checkSOAPRequest(@_);   
    print "WSNService::send called " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    my $method_name = &UBERNUtils::getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "send"); 
    
    my @urns = &UBERNUtils::getItemsFromSOAPBody($raw_xml_soap_request, $method_name, "nodeIds");
    my @messages = &UBERNUtils::getHashesFromSOAPBody($raw_xml_soap_request, $method_name, "message");
    
    @urns = &UBERNUtils::sortURNs(@urns);


    foreach my $urn (@urns){
        print "urn: $urn\n";
    }
    
    foreach my $msg (@messages) {
    	if(&UBERNUtils::checkTagExists($msg, "textMessage") && &UBERNUtils::checkTagExists($msg->{textMessage}, "msg")) {
    		my $reference = $msg->{textMessage}->{msg};
	    	my $textMessage = &UBERNUtils::getValue($reference);
	    	print "extracted text message: $textMessage\n";
		    foreach my $urn (@urns){
		    	my $id = &UBERNUtils::stripeIDfromURN($urn);    
		        if(@urns == 1) {
		          runCommandOnSensorNode($textMessage, $id);
		        }
		        else {
		          print "command sent to node $id" . &UBERNUtils::getTimeInMySQLFormat() . "\n";
		          runCommandOnSensorNode_nowait($textMessage, $id);	
		        }
		    }
    	}
    	else {
    		print "no text message found\n";
    	}
    }
    
    print "WSNService::send ended " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    
    &UBERNUtils::truncateFile($errfile, $MAX_LINES);
    &UBERNUtils::truncateFile($outfile, $MAX_LINES);
    
    return;
}

sub runCommandOnSensorNode {
    my $command = $_[0];
    my $id = $_[1];
    if( $id =~ m/msb/ ) {   
        my $number = $id;
        $number =~ s/msb//;
        $number = int($number)-1;
        # determine the host
        my $hostname = "ee0" . int($number)  . ".unibe.ch";
        print "runCommandOnSensorNode for MSB430, host: $hostname\n";
        return &UBERNUtils::sendData_flush($hostname, $MSB_COMMAND_PORT, $command); 
    }
    else {
	    print "runCommandOnSensorNode command: $command\n";
	    my $meshnode_hostname = getMeshNodeOfSensorNode($id);
	    my $URL = "http://$meshnode_hostname.unibe.ch/control/runCommand.php?nodeid=$id&command=$command";
	    print "URL: $URL\n";
	    
	    my $request = HTTP::Request->new(GET => $URL);
	    my $ua = LWP::UserAgent->new;
	    $ua->timeout($MAX_TIMEOUT_SENSORNODES);
	    my $retVal = 0;
	    eval{
	        my $response = $ua->request($request);
	        $retVal = 1;
	    };
	    if ($@) {
	        $retVal = 0;
	    };
	    return $retVal;
	}
}


sub runCommandOnSensorNode_nowait {
    my $command = $_[0];
    my $id = $_[1];
    if( $id =~ m/msb/ ) {
        my $number = $id;
        $number =~ s/msb//;
        $number = int($number)-1;
        my $hostname = "ee0" . int($number) . ".unibe.ch";
        print "runCommandOnSensorNode for MSB430, host: $hostname\n";
        return &UBERNUtils::sendData_flush($hostname, $MSB_COMMAND_PORT, $command); 
    }
    else {
	    my $meshnode_hostname = getMeshNodeOfSensorNode($id);
	    my $URL = "http://$meshnode_hostname.unibe.ch/control/runCommand.php?nodeid=$id&command=$command&nowait=1";
	    system("cd /opt/TARWISServer/UBERN_TESTBED_IMPLEMENTATION; wget -O /opt/TARWISServer/UBERN_TESTBED_IMPLEMENTATION/out_$id.txt --no-proxy -b -q \"$URL\"");
	    system("rm -rf /opt/TARWISServer/UBERN_TESTBED_IMPLEMENTATION/out_*");
	    system("rm -rf /opt/TARWISServer/UBERN_TESTBED_IMPLEMENTATION/wget-log*");
	}
}





sub resetOutputFileOfNode {
    my $id = $_[0];
    my $meshnode_hostname = getMeshNodeOfSensorNode($id);
    my $request = HTTP::Request->new(GET => "http://$meshnode_hostname.unibe.ch/control/resetOutputFile.php?nodeid=$id");
    my $ua = LWP::UserAgent->new;
    my $response = $ua->request($request);
}



sub getDataFromSensor {
    my $id = $_[0];
    my $meshnode_hostname = getMeshNodeOfSensorNode($id);
    my $request = HTTP::Request->new(GET => "http://$meshnode_hostname.unibe.ch/control/getOutputOfNode.php?nodeid=$id");
    my $ua = LWP::UserAgent->new;
    $ua->timeout($MAX_TIMEOUT_SENSORNODES);
    my $response = $ua->request($request);
    return $response->content;
}


sub getFreshDataFromSensor {
    my $id = $_[0];
    my $meshnode_hostname = getMeshNodeOfSensorNode($id);
    my $request = HTTP::Request->new(GET => "http://$meshnode_hostname.unibe.ch/control/getFreshSensorData.php?nodeid=$id");
    my $ua = LWP::UserAgent->new;
    $ua->timeout($MAX_TIMEOUT_SENSORNODES);
    my $response = $ua->request($request);
    return $response->content;
}

sub getMeshNodeOfSensorNode {
    my $sensorNode_id = $_[0];
    my $mesh_id = $sensorNode_id-1;
    if($mesh_id<10) {
       $mesh_id = "0".$mesh_id;
    }
    return "wb$mesh_id";
}


sub getSensorNodeOfMeshNode() {
    my $mesh_id = $_[0];
    return $mesh_id+1;
}

sub getNeighbourhood {
    
    checkSOAPRequest(@_);
    
    print "WSNService::getNeighbourhood called " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    my $method_name = getTag_IncludingNamespace_FromRawSOAP($raw_xml_soap_request, "getNeighbourhood"); 
    # print "MethodName: $method_name\n";
  
    my $node_urn = getItemFromSOAPBody($raw_xml_soap_request, $method_name,"node");
    
    print "node: $node_urn\n";
     
    # Assumption in UBERN_IMPLEMENTATION: if node has received a packet of 
    # another node within the last 100s, it considers him to be an adjacent neighbor!
    
    my $NEIGHBOR_DISCOVERY_INTERVAL = 100;
    
    my @times;
    my @usecs;
    my @lines;
    
    my $database = new DatabaseInterface();
    my ($ref1, $ref2, $ref3) = $database->getLastNSecsOutputOfNode($node_urn, $NEIGHBOR_DISCOVERY_INTERVAL, \@times, \@usecs, \@lines);
    
    @times = @$ref1;
    @usecs = @$ref2;
    @lines = @$ref3;
    
    my @neighbors = ();
    
    print "getNeighbourhood: checking lines\n";
    
    for(my $i=0; $i<@lines; $i++){
       my $line = @lines[$i];
       # check each line for neighborhood information
       # a neighbor table entry has the following form: 
       # neighbor: Addr 10, RT 48, ETX 16 MAC nullmac CURRENT_TIME: 2009-04-29 15:08:02 006056
       # check if this line is such an entry
       if($line =~ /neighbor:/ && $line =~ /Addr/ && $line =~ /ETX/ ) {
          my ($addr, $rt, $etx) = getAddr_RT_ETX($line);
          my $neighbor_node_urn = &UBERNUtils::generateURN($addr);
          push(@neighbors, $neighbor_node_urn);
       }
    }
    
    
    print "getNeighbourhood: lines checked\n";
    
    my @urns_soap_request = ();
   
    for(my $i=0; $i<@neighbors; $i++) {
   	  print "neighbor of $node_urn = " . @neighbors[$i] . "\n";
      my $soap_obj = SOAP::Data->name("return")->type("string")->value(@neighbors[$i]);
      push(@urns_soap_request, $soap_obj);
    }
    
    print "WSNService::getNeighbourhood end " . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    
    &UBERNUtils::truncateFile($errfile, $MAX_LINES);
    &UBERNUtils::truncateFile($outfile, $MAX_LINES);
    
    return @urns_soap_request;
}


sub getAddr_RT_ETX {
     my $tempLine = $_[0];
     
     my $neighbor_id;
     my $RT;
     my $ETX;
       
     $tempLine =~ s/neighbor://;
     $tempLine =~ s/MAC.*//;
     my @fragments = split(/,/,$tempLine);
     for(my $k=0; $k<@fragments; $k++){
           my $var = @fragments[$k];
           if($var =~ /Addr/) {
                 $var =~ s/Addr//;
                 $var = &UBERNUtils::trimchomp($var);
                 if(&UBERNUtils::isInteger($var)) {
                     $neighbor_id = $var;
                 }
           }
           if($var =~ /RT/) {
                 $var =~ s/RT//;
                 $var = &UBERNUtils::trimchomp($var);
                 if(&UBERNUtils::isInteger($var)) {
                     $RT = $var;     
                 }
           }
           if($var =~ /ETX/) {
                 $var =~ s/ETX//;
                 $var = &UBERNUtils::trimchomp($var);
                 if(&UBERNUtils::isInteger($var)) {
                     $ETX = $var;        
                 }
           }
     }
     return ($neighbor_id, $RT, $ETX);
}     

sub getLinkQuality {
    checkSOAPRequest(@_);
    
    my $link_quality_soap = SOAP::Data->name("linkquality")->value("1")->type("string");
    
    return $link_quality_soap;
}


                     

sub getNetwork {
    checkSOAPRequest(@_);
    # print "SOAP: $raw_xml_soap_request\n";
    
    print "getNetwork called at WSNService" . &UBERNUtils::getTimeInMySQLFormat() . "\n";
    
    my $graphDescription = read_file($WISEML_FILE);
    
    my $network_soap = SOAP::Data->name("return")->value($graphDescription);
    
    &UBERNUtils::truncateFile($errfile, $MAX_LINES);
    &UBERNUtils::truncateFile($outfile, $MAX_LINES);
    
    return $network_soap;
}


sub setChannelPipeline {
    my $null = SOAP::Data->name("return")->value("NULL");
    return $null;	
}

sub getSupportedChannelHandlers {
	my $null = SOAP::Data->name("return")->value("NULL");
	return $null;
}

sub checkSOAPRequest() {
    if($_[0] eq "WSNService") {
       for(my $i=0; $i<(@_-1); $i++) {
          $_[$i] = $_[$i+1];
       }
    }
}



1;
