package Utils;
use strict;
use POSIX qw(setsid);
use DateTime::Format::MySQL;
use SOAP::Transport::HTTP;
use Date::Parse;
use MIME::Base64 qw( encode_base64 decode_base64 );
use Time::HiRes qw(gettimeofday); 
use File::Slurp;
use Data::Dumper;
use File::Basename;
use Archive::Zip;
use Time::Zone;
use CGI;
use File::Copy;
use Fcntl qw/ :flock /;
use XML::Simple;

sub encodeMessage {
   my $text = $_[0];
   my $encoded=encode_base64($text);
   return $encoded;
}

sub decodeMessage {
   my $text = $_[0];
   my $decoded=decode_base64($text);
   return $decoded;
}


sub generateSimpleSOAPElement {
	my $name  = $_[0];
    my $type  = $_[1];
    my $value = $_[2];
    
	my $data = SOAP::Data->name($name)->type($type)->value($value);
	return $data;
}


#   Fork and daemonize a perl subroutine
#   write output and stderr to files

sub daemonize {
    
    my $service = $_[0];
    my $serviceName = $_[1];
    
    # redirect STDOUT and STDERR to outfiles
    
    my $outfile = "$ENV{'TARWIS_DIRECTORY'}"."$serviceName.out";
    my $errfile = "$ENV{'TARWIS_DIRECTORY'}"."$serviceName.err";
    
    # fork process to background!
    my $myproc = Proc::Simple->new();  # Create a new process object
    $myproc->start($service, $outfile, $errfile);  
    return $myproc;      
}

sub daemonize_with_args {
    
    my $service = shift;
    my $serviceName = shift;
    
    # redirect STDOUT and STDERR to outfiles
    
    my $outfile = "$ENV{'TARWIS_DIRECTORY'}"."$serviceName.out";
    my $errfile = "$ENV{'TARWIS_DIRECTORY'}"."$serviceName.err";
    
    # fork process to background!
    my $myproc = Proc::Simple->new();  # Create a new process object
    $myproc->start($service, $outfile, $errfile, @_ );  
    return $myproc;      
}


#   Fork and daemonize a perl subroutine

sub daemonize_no_stderr_stdout_redirection {
    my $service = $_[0];
    my $serviceName = $_[1];
    
    # redirect STDOUT and STDERR to outfiles
    
    # fork process!
    my $myproc = Proc::Simple->new();  # Create a new process object
    $myproc->start($service);  
}

#   gets data lines in the form "line <CURRENT_TIME: ...>"
#   and removes - http fragments (e.g. </pre> or similia)
#               - empty lines
#               - lines with only time values

sub purifyData {
    my @data = @_;
    chomp(@data);
    my @result;
    for(my $i=0; $i<@data; $i++){
       my $line = trimchomp(@data[$i]);
       $line =~ s/<.*>//;
       $line =~ s/<\.*>//;
       $line =~ s/\x00//;
       $line = trimchomp($line);
          
       # check if the line without the CURRENT_TIME part would be empty!
       my $tempLine = @data[$i];
       my @toks = split(/CURRENT_TIME/,$tempLine);
       my $lineValue = trimchomp(@toks[0]);
       
       if( ! ( $line eq "") && !( $lineValue eq "" )) {
          push(@result, $line);
       }
       
    }
    return @result;
}


#   gets only one lines in the form "line <CURRENT_TIME: ...>"
#   and removes - http fragments (e.g. </pre> or similia)
#               - empty lines
#               - lines with only time values

sub purifyLine {
    my $data = $_[0];
    my $line = trimchomp($data);
    $line =~ s/<.*>//;
    $line =~ s/<\.*>//;
    $line =~ s/\x00//;
    $line = trimchomp($line);
          
    # check if the line without the CURRENT_TIME part would be empty!
    my $tempLine = $line;
    my @toks = split(/CURRENT_TIME/,$tempLine);
    my $lineValue = trimchomp(@toks[0]);
    if( ! ( $line eq "") && !( $lineValue eq "" )) {
       return $line;
    }
    return "";
}

#   Trims and chomps tabs/spaces/newlines

sub trimchomp
{
    my $string = $_[0];
    chomp($string);
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    chomp($string);
    return $string;
}


sub stripeIDfromURN {
   my $urn   = $_[0];
   # stripe away the testbed name
   $urn =~ s/.*://;
   return int($urn);
}


sub getURNPrefix {
   my $urn   = $_[0];
   my $id    = stripeIDfromURN($urn);
   # stripe away the id
   $urn =~ s/$id//;
   return $urn;
}


sub hasURNPrefix {
   my $urn   = $_[0];
   # stripe away the id
   if ($urn =~ m/urn:.*node:.*/) {
      return 1;
   }
   else {
      return 0;
   }
}




#   Counts the lines of $data

sub getLineCount {
    my $data = $_[0];
    my @lines = split(/\n/,$data);
    my $linecount = @lines;
    return $linecount;
}

#   Returns the tail difference between two strings

sub getTailDifference {
    my $data1 = $_[0];
    my $data2 = $_[1];
    
    my @lines1 = split(/\n/,$data1);
    my @lines2 = split(/\n/,$data2);
    
    my $diffData;
    for(my $k=@lines1; $k<@lines2; $k++){
        $diffData = $diffData . @lines2[$k] . "\n";
    }
    return $diffData;
}

#   Returns 1 if argument is integer, 0 if not...

sub isInteger {
    if ( $_[0] =~ /^\d+$/ ) { 
       return 1;
    } 
    return 0;
}

sub isString {
    if ( ! ($_[0] eq undef) && length($_[0]) > 0 && ! ($_[0] eq "") ) { 
       return 1;
    } 
    return 0;
}

sub isIP {

	if( $_[0] =~ m/^(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)$/ )
	{
	    if($1 <= 255 && $2 <= 255 && $3 <= 255 && $4 <= 255)
	    {
	        return 1;
	    }
	    else
	    {
	        return 0;
	    }
	}
    else {
            return 0;
    }
}


#   for log files: cut off all lines of <arg1> if the file is bigger than <arg2> lines 


sub readFile {
    my $filepath = $_[0];
    my $string;
    {
	   local $/=undef;
	   open FILE, "$filepath" or die "Couldn't open file: $!";
	   $string = <FILE>;
	   close FILE;
    }
   return $string;
}


sub flushFile {
   my $file = $_[0];
   open (FILE, ">$file") or die "Can't read $file $!";
   close(FILE);
}

sub checkTagExistsInXML {
	my $xml = new XML::Simple;
    my $data = $xml->XMLin($_[0]);
    my $tagName = $_[1];
    my $dump = Dumper($data->{$tagName});
    if($dump =~ m/undef/ && scanDump($dump) == 0){
      return 0;
    }
    else {
      return 1;
    }
}

sub checkTagExists {
    my $record = $_[0];
    my $tagName = $_[1];
    # print "checkTagExists: $tagName\n";
    my $dump = Dumper($record->{$tagName});
    # print "dump=" . $dump . "\n";
    if($dump =~ m/undef/ && scanDump($dump) == 0){
      # print "returning false\n";
      return 0;
    }
    else {
      # print "returning true\n";
      return 1;
    }
}

sub scanDump {
    my $dumpcount = 0; $dumpcount++ while $_[0] =~ /\{/g;
    return $dumpcount;
}

sub getPropertyValue {
    my $reference    = $_[0];
    my $propertyName = $_[1];
    # print "getPropertyValue($propertyName)\n";
    
    $reference = $reference->{$propertyName};

    if( ref($reference) eq "" ){
        my $value = trimchomp($reference);
        return $value;    
    }
    elsif(ref($reference) eq 'HASH' ){
        # print "getPropertyValue: is Hash!\n";
        # print Dumper($reference);
        $reference = $reference->{content};
        return trimchomp($reference);
    }
    elsif(ref($reference) eq 'SCALAR') {
        return trimchomp("$reference");
    }
    else {
        # print "getPropertyValue($propertyName)=ELSE\n";
        return "";
    }
    return "";
}


sub getPropertyValue {
	my $reference    = $_[0];
	my $propertyName = $_[1];
	# print "getPropertyValue($propertyName)\n";
	
	$reference = $reference->{$propertyName};

    if( ref($reference) eq "" ){
        my $value = trimchomp($reference);
        return $value;    
    }
    elsif(ref($reference) eq 'HASH' ){
        # print "getPropertyValue: is Hash!\n";
        # print Dumper($reference);
        $reference = $reference->{content};
        return trimchomp($reference);
    }
    elsif(ref($reference) eq 'SCALAR') {
        return trimchomp("$reference");
    }
    else {
    	# print "getPropertyValue($propertyName)=ELSE\n";
    	return "";
    }
    return "";
}

sub getValue {
    my $reference    = $_[0];
    if( ref($reference) eq "" ){
        return trimchomp($reference);    
    }
    elsif(ref($reference) eq 'HASH' ){
        $reference = $reference->{content};
        return trimchomp($reference);
    }
    elsif(ref($reference) eq 'SCALAR') {
        return trimchomp("$reference");
    }
    die "getValue given wrong argument type!";
}

sub log {
    my $msg  = $_[0];
    my $file = $_[1];
    open my $fh, ">>$file"    or die  "$0 [$$]: open: $!";
	flock $fh, LOCK_EX        or die  "$0 [$$]: flock: $!";
	print $fh "$msg\n"        or die  "$0 [$$]: write: $!";
	close $fh                 or warn "$0 [$$]: close: $!";
}


sub truncateFile {
    my $file = $_[0];
    my $lines = $_[1];

    if(-e $file) {
	    open FILE, "<$file"        or die  "$0 [$$]: open: $!";
	    flock FILE, LOCK_EX        or die  "$0 [$$]: flock: $!";
	    my(@lines) = <FILE>;
	    close(FILE);
	    
	    if(@lines <= $lines) {
	    	return;
	    }
	    else {
	    	unlink($file);
	    	chdir '/'                   or die "Can't chdir to /: $!";
	    	if($file =~ /\.out/) {
	    	   open STDOUT, ">$file"    or die "Can't write to $file: $!"; 
	    	   my $fileLines = @lines;
               for(my $i= $fileLines - ($lines/2); $i<@lines; $i++)
               {
                  print STDOUT @lines[$i];
               }
	    	}
	    	else {
		       open STDERR, ">$file"    or die "Can't write to $file: $!"; 
               my $fileLines = @lines;
               for(my $i= $fileLines - ($lines/2); $i<@lines; $i++)
               {
                  print STDERR @lines[$i];
               }
	    	} 
	    }
    }   
}


sub convertToMySQLTimestamp() {
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime($_[0]);
   my $time = sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900,$mon+1,$mday,$hour,$min,$sec);
   return $time;
}

sub getTimeInMySQLFormat() {
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(get_UTC_time());
   my $time = sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900,$mon+1,$mday,$hour,$min,$sec);
   return $time;
}

sub getTimeISO() {
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(get_UTC_time());
   my $time = sprintf("%4d-%02d-%02dT%02d:%02d:%02d", $year+1900,$mon+1,$mday,$hour,$min,$sec);
   return $time;
}

sub getTime_addSeconds() {
   my $seconds = $_[0];
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(get_UTC_time() + $seconds);
   my $time = sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900,$mon+1,$mday,$hour,$min,$sec);
   return $time;
}



sub getUsecs {
   my ($epochseconds, $microseconds) = gettimeofday; 
   return $microseconds;
}

sub getTime_Hours_Minutes_Secs_Usecs() {
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(get_UTC_time());
   my ($epochseconds, $microseconds) = gettimeofday; 
   my $time = sprintf("%02d:%02d:%02d:%06d",$hour,$min,$sec, $microseconds);
   return $time;
}


sub parseDate {
   my $time = $_[0];
   return str2time($time); 
}

sub containsElement {
   my $arrayRef   = $_[0];
   my $element = $_[1];
   for(my $i=0; $i<@$arrayRef; $i++){
      if($element eq @$arrayRef[$i]) {
         return 1;
      }
   }
   return 0;
}



##########################################################################


sub readConfiguration {
     # read the configuration
     my $conf = $_[0];
     
     if(!-e $conf || !-r $conf) {
        print "Configuration file: $conf does not exist!\n";
        return 0;
     }
     
     open(CONFIG, $conf)  or return 0;   
     while (<CONFIG>) {
         chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
     
     return 1;
}



#   contains non-ASCII characters
sub containsBadEncoding {
    my $data = $_[0];        
    my @chars=split(//,$data);            
    for(my $i=0; $i<@chars; $i++){   
       my $n = ord(@chars[$i]);       
       if( ! ( ( $n >= 0 && $n <= 13) || ($n >= 32 && $n <= 127) ) ) {
           return 1;
       }

    }
    return 0;
}

#   contains non-ASCII characters   
sub removeBadEncoding {
    my $data = $_[0];
    my $puredata = "";
    my @chars=split(//,$data);
    for(my $i=0; $i<@chars; $i++){
       my $n = ord(@chars[$i]);
       if( $n == 0 || $n == 9 || ($n >= 32 && $n <= 126) ) {
         $puredata .=  @chars[$i];
       }
    }
    return $puredata;
}


#   sort the array according to the IDs in the URNs
sub sortURNs {
    my @urns = @_;
    my @urns_sorted = sort{&URN_sort_function}(@urns);
    return @urns_sorted;
}


sub URN_sort_function {
    if (int(stripeIDfromURN($a)) < int(stripeIDfromURN($b))) { -1; }
    elsif (int(stripeIDfromURN($a))==int(stripeIDfromURN($b))) { 0; }
    else { return 1; }
}
    
sub stripeIDfromURN {
    my $urn   = $_[0];
    # stripe away the testbed name
    $urn =~ s/.*://;
    return int($urn);
}

sub stripePrefixFromURN {
    my $urn   = $_[0];
    $urn =~ s/.*://;
    return $urn;
}


sub removeQuotes {
    my $string = $_[0];
    $string =~ tr/"//d;
    $string =~ tr/"//d;
    return $string;
}

sub bubbleSortURNsImages {
    my @urns   = @{$_[0]};
    my @images = @{$_[1]};

    foreach (@urns)
    {
      for (my $i = 0; $i < $#urns; $i++)
      {
        if (int(stripeIDfromURN($urns[$i + 1]))< int(stripeIDfromURN($urns[$i])))
        {
          ($urns[$i], $urns[$i + 1]) = ($urns[$i + 1], $urns[$i]);
          ($images[$i], $images[$i + 1]) = ($images[$i + 1], $images[$i]);
        }
      }
    }
    return (\@urns, \@images);
} 

sub portChecker {
    my $host = $_[0];
    my $port = $_[1];
    my $sock = new IO::Socket::INET(PeerAddr=>$host,PeerPort=>$port,Proto=>'tcp');
    if($sock)
    {
       close $sock;
       return 1;
    }
    return 0;
}

sub verifyPort {
    my $host = $_[0];
    my $port = $_[1];
    my $portServed = &Utils::portChecker($_[0],$_[1]);
    if($portServed){
       # print "port " . $_[1] . " is served at host: " . $_[0] . "\n";
    }
    else{
       print "port " . $_[1] . " is NOT served at host: " . $_[0] . "... exiting\n";
       exit(1);
    }
     
}

sub appendStringToFile {
   my $filename = $_[0];
   my $string   = $_[1];
   
   my $output = new IO::File(">>$filename");
   print $output $string;
   $output->close();
}


sub encodeFile {
   my $filepath = $_[0];
   open INFILE, '<', $filepath;
   binmode INFILE;
   my $buf;
   my $encoded;
   while ( read( INFILE, $buf, 100000000 ) ) {
        $encoded.=encode_base64($buf);
   }
   close INFILE;
   return $encoded;
}

sub decodeFile {
   my $filepath = $_[0];
   open INFILE, '<', $filepath;
   binmode INFILE;
   my $buf;
   my $decoded;
   while ( read( INFILE, $buf, 100000000 ) ) {
        $decoded.=decode_base64($buf);
   }
   close INFILE;
   return $decoded;
}

sub indexOfElement {
   my $arrayRef   = $_[0];
   my $element = $_[1];
   for(my $i=0; $i<@$arrayRef; $i++){
      if($element eq @$arrayRef[$i]) {
         return $i;
      }
   }
   return -1;
}




sub getThisMachineIP {
   my $ifconfig="/sbin/ifconfig";
   my @lines=qx|$ifconfig eth0| or die("Can't get info from ifconfig: ".$!);
   foreach(@lines){
      if(/inet addr:([\d.]+)/){
         return "$1";
      }
   }
}

sub max {
    my ($max, @vars) = @_;
    for (@vars) {
        $max = $_ if $_ > $max;
    }
    return $max;
}

sub min {
    my ($min, @vars) = @_;
    for (@vars) {
        $min = $_ if $_ < $min;
    }
    return $min;
}

sub createZipfile
{
	my $archiveName      = $_[0];
	my @filesToBeZipped  = @{$_[1]};
	unlink($archiveName);
	print "createZipfile $archiveName with number of files: " . @filesToBeZipped . "\n";
	my $zip = Archive::Zip->new();
	foreach my $fileToBeZipped (@filesToBeZipped) {
	   $zip->addFile($fileToBeZipped, basename($fileToBeZipped));
	}
	$zip->writeToFileNamed($archiveName);
}


sub correctPath {
	my $path = $_[0];
	$path =~ s/\/\//\//;
	return $path;	
}


sub get_UTC_time() {
	use Time::Zone;
	my $offset = tz_offset(0);
	return (time() - $offset);
}

sub redirectOutput {
	my $outfile = $_[0]; 
    my $errfile = $_[1];
	
	&Utils::flushFile($outfile);
    &Utils::flushFile($errfile);
    
    chdir '/'                   or die "Can't chdir to /: $!";
    open STDIN,  "/dev/null"    or die "Can't read /dev/null: $!";
    open STDOUT, ">$outfile"    or die "Can't write to $outfile: $!"; 
    open STDERR, ">$errfile"    or die "Can't write to $errfile: $!";
}

sub writeStringToFile {
   my $filename = $_[0];
   my $string   = $_[1];
   
   my $output = new IO::File(">$filename");
   print $output $string;
   $output->close();
}

sub replaceEndpointURLinWSDL {
	my $WSDL = $_[0];
    my $URL  = $_[1]; 
    
    my $file = readFile($WSDL);
    $file =~ s/<soap:address location=.*\/\>/<soap:address location=\"$URL\"\/\>/;
    writeStringToFile($WSDL,$file);
}

sub getHostFromURL {
	my $uri = URI->new($_[0]);
	return $uri->host;
}

sub getPortFromURL {
	my $uri = URI->new($_[0]);
	return $uri->port;
}

sub escapeXML {
    my $to_escape = $_[0]; 
    return CGI::escapeHTML($to_escape);
}



sub getTag_IncludingNamespace_FromRawSOAP {
    my $raw_soap = $_[0];
    my $ns = $_[1];
    
    if($raw_soap =~ m/$ns/ ) {
        my $retVal = $raw_soap;
        $retVal =~ s/\s+//g;

        # print "retVal: " . $retVal . "\n";
        while($retVal =~ m/[\r\n]/) {
            $retVal =~ s/[\r\n]/ /;
            # print "retVal: " . $retVal . "\n";
        }
        while( $retVal =~ m/\>\</ )
        {
            $retVal =~ s/\>\</\>\ \</;
            # print "retVal: " . $retVal . "\n";
        }
        
        # print "retVal: " . $retVal . "\n";
        my @tokens = split(" ", $retVal);
        $retVal = $ns;
        foreach my $token (@tokens){
            if($token =~ m/.*<\/.*:$ns>/){
                # print "token matches: " . $token . "\n";
                $retVal = $token;
                $retVal =~ s/.*\<\///;
                $retVal =~ s/\///;
                $retVal =~ s/\>//;
                $retVal = &Utils::trimchomp($retVal);
                # print "retVal: " . $retVal . "\n";
                return $retVal;
            }
        }
        return $retVal;     
    }
    else {
        return -1;
    }
}

    
sub replaceSomeChars {
    my $name = $_[0];
    $name =~ s/\ /\_/g;
    $name =~ s/\./\_/g;
    $name =~ s/\&/\_/g;
    $name =~ s/\</\_/g;
    $name =~ s/\>/\_/g;
    $name =~ s/\+/\_/g;
    $name =~ s/\"/\_/g;
    $name =~ s/\'/\_/g;
    $name =~ s/\\/\_/g;
    $name =~ s/\//\_/g;
    return $name;
}




sub getItemsFromSOAPBody {
    
    # print "getItemsFromSOAPBody\n";
    
    my $raw_soap = $_[0];
    
    my $xml = new XML::Simple;
    my $data = $xml->XMLin($raw_soap);
    
    my $Envelope_ns = &Utils::getTag_IncludingNamespace_FromRawSOAP($raw_soap, "Envelope");
    my $Body_ns     = &Utils::getTag_IncludingNamespace_FromRawSOAP($raw_soap, "Body");
    
    # print "Envelope: $Envelope_ns\n";
    # print "Body: $Body_ns\n";
    
    my $item_name = @_[@_-1];
    
    my @items = ();
    
    my $reference;
    
    if(&Utils::checkTagExists($data->{$Envelope_ns})){
        $reference = $data->{$Envelope_ns};     
    }
    else {
        $reference = $data;
    }
    
    $reference = $data->{$Body_ns};
    
    # start at 1 because $_[0] is the raw xml
    for(my $i=1; $i<@_-1; $i++) {
        my $tag_name = @_[$i];
        $reference = $reference->{$tag_name};
    }
    
    $reference = $reference->{$item_name};
    
    if( ref($reference) eq "" ){
        # print "adding string: $reference\n";
        if(!&Utils::trimchomp($reference) eq "")
        {
            push(@items, $reference);
        }
    }
    elsif(ref($reference) eq 'HASH' ){
        # print "adding hash: $reference\n";
        # print Dumper($reference);
        my $record = &Utils::getValue($reference);
        push(@items, $record);
    }
    elsif(ref($reference) eq 'SCALAR') {
        # print "adding scalar: $reference\n";
        if(!&Utils::trimchomp("$reference") eq "")
        {
           push(@items, $reference);
        }
    }
    elsif(ref($reference) eq 'ARRAY') {
        # print "adding array: $reference\n";
        # print Dumper($reference);
        foreach my $record (@{$reference})  {
            $record = &Utils::getValue($record);
            push(@items, $record);
        }
    }
    # print "getItemsFromSOAPBody end\n";
    return @items;
}


sub getItemFromSOAPBody {
    my @items = &Utils::getItemsFromSOAPBody(@_);
    return @items[0];
}

sub getHashesFromSOAPBody {
    
    # print "getHashesFromSOAPBody\n";
    
    my $raw_soap = $_[0];
    
    my $xml = new XML::Simple;
    my $data = $xml->XMLin($raw_soap);
    
    my $Envelope_ns = &Utils::getTag_IncludingNamespace_FromRawSOAP($raw_soap, "Envelope");
    my $Body_ns     = &Utils::getTag_IncludingNamespace_FromRawSOAP($raw_soap, "Body");
    
    # print "Envelope: $Envelope_ns\n";
    # print "Body: $Body_ns\n";
    
    my $item_name = @_[@_-1];
    
    my @items = ();
    
    my $reference;
    
    if(&Utils::checkTagExists($data->{$Envelope_ns})){
        $reference = $data->{$Envelope_ns};     
    }
    else {
        $reference = $data;
    }
    
    $reference = $data->{$Body_ns};
    
    # start at 1 because $_[0] is the raw xml
    for(my $i=1; $i<@_-1; $i++) {
        my $tag_name = @_[$i];
        $reference = $reference->{$tag_name};
    }
    
    $reference = $reference->{$item_name};
    
    if(ref($reference) eq 'HASH' ){
        # print "is Hash! " . Dumper($reference) . "\n";
        push(@items, $reference);
    }
    else {
        # print "is array! " . Dumper($reference) . "\n";
        foreach my $record (@{$reference})  {
            push(@items, $record);
        }
    }
    
    # print "getHashesFromSOAPBody end @items\n";
    
    return @items;
}


use Net::Ping;

sub pingNode {
    my $ip_or_hostname = $_[0];
    my $p = Net::Ping->new( "udp", 1, 64 );
    my $alive = $p->ping($ip_or_hostname);
    $p->close();
    return $alive;    
}

sub validEmail {
	my $email_address = $_[0];
	if ($email_address =~ /.*@.*\..*$/)
	{
	    return 1;
	}
	else {
	    return 0;
	}
}


1;
