<?php
require_once("Image/GraphVizNG.php");

class NetworkGraph
{
  public $name;
  public $network;
  public $sensornodes;

  
//*****************************************************************************		

//Constructor:
//  - construct the network graph object
//  - gets the network and take all sensor nodes from it
public function __construct($name, $network)
{
  if($name == "" || $network == null)
  {
    throw new Exception('Name and network required!!');
  }
  $this->name = $name;
  $this->network = $network;
  $this->sensornodes = $network->getAllSensornodes();
}

public function getSensornodes(){
	return $this->sensornodes;
}


//*****************************************************************************		

//drawGraph:
//  - creates a GraphViz Image and add all nodes and connections to the graph
//    according the the given network object
public function drawGraph()
{
  global $config;
  $graphFormat = "svg";
  $dotFile = "./" . $this->name . "Graph.dot";
  $graphFile = "./" . $this->name . "Graph." . $graphFormat;
  
  $fontname = "Arial";
  $fontsize = "14";
  
  $mapDPI = 72;
  $sizeXInch = $config['MAP_IMG_WIDTH'] / $mapDPI;
  $sizeYInch = $config['MAP_IMG_HEIGHT'] / $mapDPI;
  $sizeStr = $sizeXInch . ", " . $sizeYInch;
  
  $graph = new Image_GraphVizNG(false); //creating a GraphVir Image object
			
  //graph attributes
  $graph->addAttributes(array("overlap" => "scale",
                              "ratio" => "fill",
                              "splines" => "true",
                              "model" => "shortpath",
                              "size" => $sizeStr,
                              "mode" => "KK",
                              "sep" => "+50",
                              "dpi" => "72.0",
                              "margin" => "0.0,0.0",
                              "mindist" => "0.0",
                              "pack" => "false",
                              "bgcolor" => "transparent"));
  
  //array for all nodes positions, to check if nodes are too close together (see isNodeTooClose() )
  $tmpCheckArray = array();
  
  //fakepoints on the edges of the image, because of correct scaling of the image
  $fakeNodePos0 = "0,0";
  $tmpCheckArray['f0'] = $fakeNodePos0;
  $graph->addNode("f0", array("shape" => "box", "color" => 'transparent', "fontcolor" => "transparent", "width" => "0.1", "height" => "0.1", "fixedsize" => "true", "pos" => $fakeNodePos0, "pin" => "true"));
  
  $fakeNodePos1 = $config['MAP_IMG_WIDTH'] . ",0";
  $tmpCheckArray['f1'] = $fakeNodePos1;
  $graph->addNode("f1", array("shape" => "box", "color" => 'transparent', "fontcolor" => "transparent", "width" => "0.1", "height" => "0.1", "fixedsize" => "true", "pos" => $fakeNodePos1, "pin" => "true"));
  
  $fakeNodePos2 = $config['MAP_IMG_WIDTH'] . "," . $config['MAP_IMG_HEIGHT'];
  $tmpCheckArray['f2'] = $fakeNodePos2;
  $graph->addNode("f2", array("shape" => "box", "color" => 'transparent', "fontcolor" => "transparent", "width" => "0.1", "height" => "0.1", "fixedsize" => "true", "pos" => $fakeNodePos2, "pin" => "true"));
  
  $fakeNodePos3 = "0," . $config['MAP_IMG_HEIGHT'];
  $tmpCheckArray['f3'] = $fakeNodePos3;
  $graph->addNode("f3", array("shape" => "box", "color" => 'transparent', "fontcolor" => "transparent", "width" => "0.1", "height" => "0.1", "fixedsize" => "true", "pos" => $fakeNodePos3, "pin" => "true"));

  //adding nodes
  if(!empty($this->sensornodes))
  {
    foreach($this->sensornodes as $sensornode)
    {
      //calculating the 2D coordinates accoring to the given 3D coordinates and the soze of the image
      $nodeX2Dpx = round($this->getX2Dpx($sensornode->x3D, $sensornode->y3D, $config['MAP_ANGLE'], $config['MAP_SIZE_X'], $config['MAP_SIZE_Y']));
      $nodeY2Dpx = round($this->getY2Dpx($sensornode->y3D, $sensornode->z3D, $config['MAP_ANGLE'], $config['MAP_SIZE_Y'], $config['MAP_SIZE_Z']));
      $sensornode->add2DCoordinates($nodeX2Dpx, $nodeY2Dpx);    //add the 2D coordinates to the sensor node object
    
      $nodePos = $nodeX2Dpx . "," . $nodeY2Dpx;
      $isTooClose = $this->isNodeTooClose($nodePos, $tmpCheckArray);
      while($isTooClose > -1)
      {
        $pos1 = explode(",", $nodePos);
        $pos2 = explode(",", $tmpCheckArray[$isTooClose]);  //take the position of the already placed node
        $newXPos = $pos2[0] + 4;                            //replace the problem-node according to the x-coordinate
        $nodePos = $newXPos . "," . $pos1[1];               // of the already placed node
        $isTooClose = $this->isNodeTooClose($nodePos, $tmpCheckArray);
      }
      $tmpCheckArray[$sensornode->id] = $nodePos;                          
    
      //generating the node attributes
      $nodearray = array("color" => 'transparent',
                         "shapefile" => "../pics/" . $sensornode->type . ".png",
                         "tooltip" => $sensornode->name . ":" . $sensornode->type,
                         "label" => $sensornode->name,
                         "labelloc" => "b",
                         "width" => "0.45",
                         "height" => "0.45",
                         "fixedsize" => "true",
                         "fontname" => $fontname,
                         "fontsize" => $fontsize,
                         "fontweight" => "bold",
                         "fontcolor" => "#A50021",
                         "pos" => $nodePos,
                         "pin" => "true");
      //generating the node attributes
      
      $graph->addNode($sensornode->id, $nodearray, $sensornode->id);  //adding the node to the GraphViz Image
    }
  }
  
  //edges sensornode -> sensornode
  $wsnConnections = $this->network->getAllSnSnConnections();
  foreach($wsnConnections as $con)
  {
    //check if the nodes of the egde exisiting, otherwise neato will fail
    if(!empty($tmpCheckArray[$con['node_urn']]) && !empty($tmpCheckArray[$con['node_urn_neighbor']]))
    {
      $edge = array($con['node_urn'] => $con['node_urn_neighbor']);
      $edgeAttr = array('label' => $con['ETX'], 'style' => 'dashed', 'color' => 'green', 'fontcolor' => 'orange');
      if(!$graph->containsEdge($edge))      //if the edge is not already addes
      {
        $graph->addEdge($edge, $edgeAttr);  //add edge
      }
    }
  }

  //generating image
  $dotFile = $graph->saveParsedGraph($dotFile);
  $resultat = $graph->renderDotFile($dotFile, $graphFile, $graphFormat, true);
  
}

		
//*****************************************************************************		

//getX2Dpx:
//  - calculates the 2D x-coordinates accoring the given parameters
//  - called by drawGraph
public function getX2Dpx($x3d, $y3d, $alpha, $buildingXpx, $buildingYpx)
{
  $result = ($buildingXpx * $x3d/100) + (tan(deg2rad($alpha)) * ($buildingYpx * $y3d/100));
  return $result;
}


//*****************************************************************************		

//getY2Dpx:
//  - calculates the 2D y-coordinates accoring the given parameters
//  - called by drawGraph
public function getY2Dpx($y3d, $z3d, $alpha, $buildingYpx, $buildingZpx)
{
  $result = ($buildingYpx * $y3d/100) + ($buildingZpx * $z3d/100);
  return $result;
}


//*****************************************************************************		

//isNodeTooClose:
//  - the node graph is generqated using neato, which crashes if two nodes are
//     too close together
//  - this function determines if two nodes are to close together
//  - it returns -1, if not and the index of node in the array which is too
//    close
//  - called by drawGraph
public function isNodeTooClose($posStr, $posArray)
{
  $res = -1;
  $newNodePos = explode(",", $posStr);
  foreach($posArray as $key => $nodePos)
  {
    $pos = explode(",", $nodePos);
    $diffPosX = abs($newNodePos[0] - $pos[0]);
    $diffPosY = abs($newNodePos[1] - $pos[1]);
    if($diffPosX < 4 && $diffPosY < 4)
    {
      $res = $key;
    }
  }
  return $res;
}


//*****************************************************************************		

}
?>
