<?php

	
	
	require_once("include/Subscription.class.inc");
	require_once("include/MySQLConnection.class.inc");
	require_once("include/Logger.class.inc");
	require_once("include/Event.class.inc");
	
	
	class Action {
		
		
		/* *********************************************************************** */
		/*                      member variables                                   */
		/* *********************************************************************** */
		
		var $_fields;    
		
		/* *********************************************************************** */
		/*                      constructor                                        */
		/* *********************************************************************** */

		function Action() {
			$this->_fields = array();
		}
		
		/* *********************************************************************** */
		/*                      static utility functions                           */
		/* *********************************************************************** */
		
		function & getActionsAttributes($search_field= null, $search_value= null, $orderby= null, $offset= null, $count= null) {
			$conn=& MySQLConnection::getConnection();
			
			$query= " SELECT action_id, action_name, action_title, date_created, date_modified "
            . " FROM Action ";

			if (!is_null($search_field) and !is_null($search_value) ) {
				$esc_search_value= $conn->escape_string($search_value);
				$query.= " WHERE $search_field LIKE '$esc_search_value' ";
			}
			
			if (!is_null($orderby)) {
				if (strpos($orderby, 'action_title') !== false) {
					$query.= " ORDER BY action_title, action_name, date_created ";
				}
				elseif (strpos($orderby, 'action_name') !== false) {
					$query.= " ORDER BY action_name, action_title, date_created ";
				}
				elseif (strpos($orderby, 'date_created') !== false) {
					$query.= " ORDER BY date_created, action_name, action_title ";
				}
				elseif (strpos($orderby, 'date_modified') !== false) {
					$query.= " ORDER BY date_modified, date_created, action_title ";
				}
				
			}
			if (!is_null($offset) and !is_null($count)) {
				$query.= " LIMIT $offset,$count ";
			}
			
			
			$conn->query($query);
			$ret = array();
			while ($row = $conn->fetch_array()) {
				$ret[]= $row;
			}
			$conn->freeresult();
			return $ret;
		}
		
		/* *********************************************************************** */
		function getActionCount($search_field= null, $search_value= null) {
			$conn=& MySQLConnection::getConnection();
			$query= " SELECT action_id "
			. " FROM Action ";
			if (!is_null($search_field) and !is_null($search_value) ) {
				$esc_search_value= $conn->escape_string($search_value);
				$query.= " WHERE $search_field LIKE '$esc_search_value' ";
			}
			$conn->query($query);
			$n_actions= $conn->num_rows();
			$conn->freeresult();
			
			return $n_actions;
		}
		
		/**
		 * Reconstitutes a action object from the database  given an internal database ID
		 *
		 */
		function & reconstituteById($aid) {
			$action= null;
			if (! (is_scalar($aid) && $aid > 0)) {
				trigger_error(
							  "Action::reconstituteById: expected \$aid of type 'int' > 0, got '$aid'",
							  E_USER_ERROR
							  ); 
				return $action; // null
			}
			
			
			$conn =& MySQLConnection::getConnection();
			
			$query = " select * from Action "
            . " where action_id = $aid ";
			
			$conn->query($query);
			
			if (! ($row = $conn->fetch_array())) {
				$conn->freeresult();
				return $action; // null
			}
			$action = new Action();
			$action->_setFields($row);
			return $action; 
		}
		
		/** 
		 * Get all Actions  ANWANDER
		 */
		function & getAllActions() {
						
			$conn=& MySQLConnection::getConnection();
			
			$query= " SELECT action_id, action_name, action_title "
            . " FROM Action "
            . " ORDER BY action_name;";
			
			$conn->query($query);
			$ret = array();
			while ($row = $conn->fetch_array()) {
				$ret[]= $row;
			}
			$conn->freeresult();
			return $ret;
		}
		
		/* *********************************************************************** */
		/**
		 * Creates a new action owned by a given user. 
		 *
		 */
		function & create($user_id, $action_name, $action_title, $action_description) {
			
			if (! is_numeric($user_id) || ! ($user_id > 0)) {
				trigger_error(
							  "Action::create: positive integer expected in \$user_id, got '$user_id'",
							  E_USER_ERROR
							  ); 
			}
			
			$remote= '127.0.0.1';
			if (isset($_SERVER['REMOTE_ADDR'])) { $remote= $_SERVER['REMOTE_ADDR']; }
			
			$conn =& MySQLConnection::getConnection();
			
			$query  = " INSERT INTO Action (action_id, role_user, action_name, action_title,action_description, date_created) ";
			$query .= " VALUES ( ";
			$query .= "   NULL, ";
			$query .= "   $user_id, ";
			$query .= "   '" . $conn->escape_string($action_name) . "', ";
			$query .= "   '" . $conn->escape_string($action_title)   . "', ";
			$query .= "   '" . $conn->escape_string($action_description)   . "', ";
			$query .= "   NULL ";
			$query .= " )";
						
			$conn->query($query);
			$aid= $conn->insert_id();
			
			$action=& Action::reconstituteById($aid);     
			
			Event::storeRoleEvent("Action created: $action_name ($aid)", $user_id);
			
			return $action;
			
			
		}
		
		/* *********************************************************************** */
		/**
		 * Replies the resoure ID 
		 *
		 * @return int the action ID; 0, if the action ID is not defined 
		 */
		function getID() {
			if (isset($this->_fields["action_id"])) {
				return $this->_fields["action_id"];
			} else {
				return 0;
			}
		}
		
		

		/* *********************************************************************** */
		/**
		 * Sets the array of attributes. 
		 *
		 */
		function _setFields($row) {
			if (! is_array($row) || is_null($row)) {
				trigger_error(
							  "Action::_setFields: expected non-null array in \$row, got $row",
							  E_USER_ERROR
							  ); 
				return;
			}
			$this->_fields = $row;
		}

		
		/* *********************************************************************** */
		/**
		 * Set/get parameters 
		 *
		 */
		
		function setTitle($title) {   
			$this->_fields["action_title"] = $title;
		}
		
		function getTitle() {
			return $this->_fields["action_title"];
		}
		
		function setName($name) {   
			$this->_fields["action_name"] = $name;
		}
		
		function getName() {
			return $this->_fields["action_name"];
		}
		
		function setDescription($actionDescription) {
			$this->_fields["action_description"] = $actionDescription;    
		}
		
		function getDescription() {
			if (isset($this->_fields["action_description"])) {
				return $this->_fields["action_description"];
			} else {
				return false;  
			}
		}
		
		function getDateCreated() {
			return $this->_fields['date_created'];
		} 
		function getDateModified() {
			return $this->_fields['date_modified'];
		} 
		
		
		/* *********************************************************************** */
		/**
		 * Persists
		 *
		 */
		function persist() {
			
			$conn =& MySQLConnection::getConnection(); 
			
			$remote= '127.0.0.1';
			if (isset($_SERVER['REMOTE_ADDR'])) { $remote= $_SERVER['REMOTE_ADDR']; }
			
			$aid = $this->getID();
			if (! $aid) {
				
				// create the insert statement
				//				
				$query  = " INSERT INTO Action (action_id, role_user, action_name, action_title,action_description, date_created) ";
				$query .= " VALUES ( ";
				$query .= "   NULL, ";
				$query .= "   $user_id, ";
				$query .= "   '" . $conn->escape_string($this->_fields["action_name"]) . "', ";
				$query .= "   '" . $conn->escape_string($this->_fields["action_title"])   . "', ";
				$query .= "   '" . $conn->escape_string($this->_fields["action_description"])   . "', ";
				$query .= "   NULL ";
				$query .= " )";

				
				$conn->query($query);
				$this->_fields["action_id"] = $conn->insert_id();
				
			} else {    
				
				// create the update statement
				//
				$query = " UPDATE Action "
				.  "    SET action_name        = '" . $conn->escape_string($this->_fields["action_name"]) . "',"
				.  "        action_title       = '" . $conn->escape_string($this->_fields["action_title"]) . "',"
				.  "        action_description = '" . $conn->escape_string($this->_fields["action_description"])   . "',"
				.  "        date_modified       =  NOW() "
				.  " WHERE action_id = $aid";
				
				$conn->query($query);
				$numAffectedRows = $conn->affected_rows();
				if ($numAffectedRows != 1) {
					trigger_error(
								  "Action::persist: failed to persist role '$aid'. Expected 1 affected row, got $numAffectedRows ",
								  E_USER_ERROR   
								  );
					return;
				}       
			}
            
		}
		
		/**
		 * Removes this action from the database.
		 */
		function delete($admin_id= 1) {
			
			$conn =& MySQLConnection::getConnection();
			
			$aid = $this->getID();
			if (!$aid) {
				return; 
			}
			
			
			$query = "DELETE FROM Action WHERE action_id = $aid";
			$conn->query($query);
			
			$query = "DELETE FROM ActionManager WHERE action_id = $aid";
			$conn->query($query);
			
			Event::storeRoleEvent("Action deleted: $title ($aid)",$admin_id,EVENT_LEVEL_WARNING);  
			
			// reset internal fields 
			$this->_fields = array();

		}
		
	}  