<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: Administrator.class.inc,v 1.53 2007/10/18 08:48:54 vlry Exp $
 */

require_once("include/constants.inc");
require_once("include/User.class.inc");

/**
 * This class represents an administrator of the SNA-Portal. There
 * are two types of administrators, Portal Administrators and
 * Role Administrators.
 *
 * @author Thomas Spreng <spreng@iam.unibe.ch>
 * @author Valery Tschopp <tschopp@switch.ch>
 * @version $Revision: 1.53 $
 *
 * @package Library 
 */
class Administrator extends User {


 /**
  * Reconstitutes an administrator from the database using the external user identity 
  * as key. Typical examples for external user IDs are "portaladmin"
  * or "demo@aaitest.switch.ch".
  *
  * <b>Typical usage</b>
  * <pre>
  *    
  *    $user = User::reconstituteByUid('portaladmin');
  *    if (is_null($user)) {
  *      echo "ERROR: failed to reconstitute user with database id '5'";
  *    }
  * </pre>
  *
  * @param string $uid  the administrator id
  *
  * @return Administrator an initialized instance of class Administrator, if the respective user
  *     exists; NULL, otherwise
  *
  * @static 
  */
  function & reconstituteByUid($uid) {
    
    if (empty($uid)) {
        $admin= null;
        return $admin;
    }

    $conn = MySQLConnection::getConnection();
    
    $user_uid= $conn->escape_string($uid);
    $query = " SELECT user_id, user_uid, user_privilege FROM User "
           . " WHERE user_uid = '$user_uid' ";

    $conn->query($query);
    if (! ( $row = $conn->fetch_array() ) ) {
      $conn->freeresult();
      $admin= null;
      return $admin;
    }

    $admin = new Administrator();
    $admin->uid= $row["user_uid"];
    $admin->user_id= $row["user_id"];
    $admin->user_privilege= $row["user_privilege"];

    if (! $admin->isAdministrator()) {
      $admin= null; 
    }    
    return $admin;
  }

  /**
  * @param string $id  the administrator user_id
  *
  * @return Administrator an initialized instance of class Administrator, if the respective user
  *     exists; NULL, otherwise
  *
  * @static 
  */  
  function & reconstituteById($id) {
    
    $conn= MySQLConnection::getConnection();
    
    $query = " SELECT user_id, user_uid, user_privilege FROM User "
           . " WHERE user_id = $id "; 

    $conn->query($query);
    if (! ( $row = $conn->fetch_array() ) ) {
      $conn->freeresult();
      $admin= null;
      return $admin;
    }

    $admin= new Administrator();
    $admin->uid= $row["user_uid"];
    $admin->user_id= $row["user_id"];
    $admin->user_privilege= $row["user_privilege"];

    if (! $admin->isAdministrator()) {
      $admin= null;
      return $admin;
    }    
    
    return $admin;
 } 

  
  /**
   * Returns the list of all administrator user_id.
   *
   * @static
   */
  function & getAdministratorIDs() {
    $admin_ids= array();
  
    $query= "SELECT user_id "
          . "FROM User "
          . "WHERE user_privilege <> " . USER_PRIVILEGE_ID_USER . " "
          . "ORDER BY user_privilege";
          
    $conn= MySQLConnection::getConnection();
    $conn->query($query);
    while( $row= $conn->fetch_array() ) {
        $admin_ids[]= $row['user_id'];
    }
    $conn->freeresult();

    return $admin_ids;

  }

  /**
   * Returns the list of all portal administrator user_id.
   *
   * @static
   */
  function & getPortalAdministratorIDs() {

    $resadmin_ids= Administrator::getPrivilegedAdministratorIDs( USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR );
    return $resadmin_ids;

  }

  /**
   * Returns the list of all portal administrator user_id.
   *
   * @static
   */
  function & getRoleAdministratorIDs() {

    $resadmin_ids= Administrator::getPrivilegedAdministratorIDs( USER_PRIVILEGE_ID_RESOURCE_ADMINISTRATOR );
    return $resadmin_ids;

  }

  /**
   * Returns the list of all user_id subscription administrator.
   *
   * @static
   */
  function & getSubscriptionAdministratorIDs() {

    $subadmin_ids= Administrator::getPrivilegedAdministratorIDs( USER_PRIVILEGE_ID_SUBSCRIPTION_ADMINISTRATOR );
    return $subadmin_ids;

  }

  /**
  * Returns a list of admin id given the privilege. 
  *
  * @return array List of admin ids.
  * @static
  */
  function & getPrivilegedAdministratorIDs( $admin_privilege ) {
  
    $admin_ids= array();
    $conn= MySQLConnection::getConnection();
  
     // all admins (unsorted)
    $query= " SELECT user_id "
          . " FROM User "
          . " WHERE user_privilege = " . $admin_privilege;
    $conn->query($query);
    while( $row= $conn->fetch_array() ) {
        $user_id= $row['user_id'];
        $admin_ids[]= $user_id;
    }
    $conn->freeresult();

    return $admin_ids;
  
  }

  /**
   * @param $privilege_ids array of privilege_id to return.
   * @parma $orderby string sort results by.
   * @return array to administrators attributes: user_id, commonName, privilegeName
   * 
   * @static
   */
  function getPrivilegedAdministratorsAttributes( $privilege_ids, $orderby= null ) {
  
    $admins_attrs= array();
    if (empty($privilege_ids)) {
        $privilege_ids[]= USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR;
        $privilege_ids[]= USER_PRIVILEGE_ID_RESOURCE_ADMINISTRATOR;
        $privilege_ids[]= USER_PRIVILEGE_ID_SUBSCRIPTION_ADMINISTRATOR;
    }
    
    $privileges= array();
    foreach($privilege_ids as $privilege) {
        $privileges[]= "user_privilege = $privilege";
    }
    $privileges_clause= implode(' OR ', $privileges);
    
     // all admins
    $query= " SELECT User.user_id, commonName, privilegeName "
          . " FROM User, UserAttributes "
          . " WHERE User.user_id = UserAttributes.user_id "
          . "   AND ($privileges_clause) ";

    if (!is_null($orderby)) {
        if (strpos($orderby, 'privilegeName') !== false) {
            $query.= " ORDER BY privilegeName, commonName ";
        }
        else if (strpos($orderby, 'commonName') !== false) {
            $query.= " ORDER BY commonName, privilegeName ";
        }
        else {
            $query.= " ORDER BY $orderby ";        
        }
    }
    $conn= MySQLConnection::getConnection();
    $conn->query($query);
    while( $row= $conn->fetch_array() ) {
        $admins_attrs[]= $row;
    }
    $conn->freeresult();

    return $admins_attrs;
  
  }


  /* ************************************************************* */
  /*               constructor                                     */
  /* ************************************************************* */

  /**
   * constructor
   *
   * @access private 
   */
  function Administrator() {
  }
  
  /**
   * @return true iff admin is a portal admin
   */
  function isPortalAdministrator() {
    return $this->getPrivilege() == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR;
  }

  /**
   * @return true iff admin is a role admin
   */
  function isRoleAdministrator() {
    return $this->getPrivilege() == USER_PRIVILEGE_ID_RESOURCE_ADMINISTRATOR;
  }

  /**
   * @return true iff admin is a subscription admin
   */
  function isSubscriptionAdministrator() {
    return $this->getPrivilege() == USER_PRIVILEGE_ID_SUBSCRIPTION_ADMINISTRATOR;
  }

  /* ************************************************************* */
  /**
   * Stores the given password for this administrator in the database.
   *
   * @param string  $pass   the password  
   *
   */
   function storePassword($pass) {
     $this->storeAttribute(ATID_LOCALPASSWORD, $pass);
   }


   /**
    * Returns int Number of role where admin is provider or manager.
    *
    * @return int Number of my roles.
    */
   function getMyRolesCount($search_field= null, $search_value= null) {
        $conn= MySQLConnection::getConnection();
        $user_id= $this->getID();
        $query= " SELECT DISTINCT Role.role_id "
              . " FROM Role, RoleManager "
              . " WHERE ( role_user = $user_id "
              . "         OR ( user_id = $user_id AND "
              . "              Role.role_id = RoleManager.role_id ) ) ";
        if (!is_null($search_field) and !is_null($search_value) ) {
            $esc_search_value= $conn->escape_string($search_value);
            $query.= " AND $search_field LIKE '$esc_search_value' ";
        }
        $conn->query($query);
        $n_roles= $conn->num_rows();
        $conn->freeresult();
        return $n_roles;
   }
   

   /**
    * @return array List of role_id where admin is either provider or manager
    */
   function getMyRoleIDs($search_field= null, $search_value= null,
                             $offset= null, $count=null) {
        $conn= MySQLConnection::getConnection();
        $rids= array();
    
        $user_id= $this->getID();
        $query= " SELECT DISTINCT Role.role_id "
              . " FROM Role, RoleManager "
              . " WHERE ( role_user = $user_id "
              . "         OR ( user_id = $user_id AND "
              . "              Role.role_id = RoleManager.role_id ) ) ";
        if (!is_null($search_field) and !is_null($search_value) ) {
            $esc_search_value= $conn->escape_string($search_value);
            $query.= " AND $search_field LIKE '$esc_search_value' ";
        }
        $query.= " ORDER BY role_title ASC ";
        if (!is_null($offset) and !is_null($count)) {
            $query.= " LIMIT $offset,$count ";
        }
              
        $conn->query($query);
        while( $row= $conn->fetch_array() ) {
            $rids[]= $row['role_id'];
        }
        $conn->freeresult();
    
        return $rids;      
   }


    /**
     * @return int  Numbers of users subscribed to any role where the
     * admin is either provider or manager
     */
    function getMyUsersCount($search_field= null, $search_value= null) {
        $conn= MySQLConnection::getConnection();
        $user_id= $this->getID();
        $query= "SELECT DISTINCT User.user_id "
              . "FROM ( User, UserAttributes ) "
              . "LEFT JOIN Subscription ON ( User.user_id = subscription_user ) "
              . "WHERE UserAttributes.user_id = User.user_id ";
        if (!is_null($search_field) and !is_null($search_value) ) {
            if ($search_field == 'user_uid') { $search_field= 'User.user_uid'; }
            $esc_search_value= $conn->escape_string($search_value);
            $query.= " AND $search_field LIKE '$esc_search_value' ";
        }
        $query.="  AND subscription_role IN "
              . "( "
              . "  SELECT Role.role_id "
              . "    FROM Role, RoleManager "
              . "   WHERE role_user = $user_id "
              . "      OR ( user_id = $user_id AND Role.role_id = RoleManager.role_id ) "
              . ")";
        $conn->query($query);
        $n_roles= $conn->num_rows();
        $conn->freeresult();
        return $n_roles;    
    }

    /**
     * @return array List of users attributes subscribed to a role where the
     * admin is either provider or manager
     *
     */
    function getMyUsersAttributes($search_field= null, $search_value= null,
                                  $orderby= null, $offset= null, $count=null) {

        $conn= MySQLConnection::getConnection();
        $users_attrs= array();
        $admin_id= $this->getID();

        $query= "SELECT DISTINCT User.user_id, User.user_uid, surname, givenName, privilegeName, user_last_login "
              . "FROM ( User, UserAttributes ) "
              . "LEFT JOIN Subscription ON ( User.user_id = subscription_user ) "
              . "WHERE UserAttributes.user_id = User.user_id ";
        //search param
        if (!is_null($search_field) and !is_null($search_value) ) {
            if ($search_field == 'user_uid') { $search_field= 'User.user_uid'; }
            $esc_search_value= $conn->escape_string($search_value);
            $query.= " AND $search_field LIKE '$esc_search_value' ";
        }
        $query.="  AND subscription_role IN "
              . "( "
              . "  SELECT Role.role_id "
              . "    FROM Role, RoleManager "
              . "   WHERE role_user = $admin_id "
              . "      OR ( user_id = $admin_id AND Role.role_id = RoleManager.role_id ) "
              . ")";
        if (!is_null($orderby)) {
            if (strpos($orderby, 'surname') !== false) {
                $query.= " ORDER BY surname, givenName, privilegeName, user_uid ";
            }
            elseif (strpos($orderby, 'givenName') !== false) {
                $query.= " ORDER BY givenName, surname, privilegeName, user_uid ";
            }
            elseif (strpos($orderby, 'privilegeName') !== false) {
                $query.= " ORDER BY privilegeName, surname, givenName, user_uid ";
            }
            elseif (strpos($orderby, 'user_last_login') !== false) {
                $query.= " ORDER BY user_last_login DESC, surname, givenName, user_uid ";
            }
            else {
                $query.= " ORDER BY $orderby ";
            }
        }
        if (!is_null($offset) and !is_null($count)) {
            $query.= " LIMIT $offset,$count ";
        }
        $conn->query($query);
        $users_attrs= array();
        while($row = $conn->fetch_array()) {
            $users_attrs[]= $row;
        }
        $conn->freeresult();
        return $users_attrs;
    }



  /* **************************************************************** */
  /**
   * Replies true, if the aministratoruser can edit 
   * privileges of another user
   * 
   * @param User $user   an instance of class User 
   *
   * @return boolean true, if the administrtor can edit 
   * privileges of another user
   *
   */
  function canChangePasswordOf($user) {
    if ($this->getUid() == $user->getUid()) return true;
    if ($this->getPrivilege() == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR) {
      return true; 
    }
    return false; 
  }

  /* ***************************************************************** */

  /**
   * Checks whether the current administator can create a new role
   * 
   * @return boolean true, if the current administator can create a role;
   *    false, otherwise 
   */
  function canCreateRole() {
  
     // a Portal Administrator can create any role
     if ($this->getPrivilege() == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR) {
        return true; 
     }
     // Role Administrator too
     if ($this->getPrivilege() == USER_PRIVILEGE_ID_RESOURCE_ADMINISTRATOR) {
        return true;       
     }
     
     return false;    
  }


  function canDuplicateRole(&$role) {
     return $this->canEditRole($role) and $this->canCreateRole();
  }
  /**
  * Replies true if the current administrator can delete the role 
  *
  * @param Role     $role  an instance of class Role 
  *
  * @return  boolean true, if the current administrator can delete the role;
  *   false, otherwise  
  *
  */
  function canDeleteRole(&$role) {
     // a Portal Administrator can delete any role
     //
     if ($this->getPrivilege() == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR) {
        return true; 
     }
    
     // a Role administrator can only delete his or her own
     // roles 
     //   
     if ($this->getPrivilege() == USER_PRIVILEGE_ID_RESOURCE_ADMINISTRATOR) {
        $is_owner= $role->isRoleProvider( $this );
        $is_manager= $role->isRoleManager( $this );

        return $is_owner or $is_manager;       
     }
     
     return false;    
  }



  /* **************************************************************** */
  /**
   * Checks whether the current administator can edit a given role
   * 
   * @param array &$role   
   * 
   * @return boolean true, if the current administator can edit a given role;
   *    false, otherwise 
   */
  function canEditRole(&$role) {
     // a Portal Administrator can edit any role
     //
     if ($this->getPrivilege() == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR) {
        return true; 
     }
     
     // Role Administrator can only edit its own roles and roles
     // where he is manager.
     //   
     if ($this->getPrivilege() == USER_PRIVILEGE_ID_RESOURCE_ADMINISTRATOR) {
        $is_owner= $role->isRoleProvider( $this );
        $is_manager= $role->isRoleManager( $this );
        return $is_owner or $is_manager;       
     }
     
     return false;    
  }


  /* **************************************************************** */
  /**
   * Replies true, if the current admin user can edit privileges of 
   * another user
   * 
   * @return boolen true, if the current admin user can edit privileges of 
   *     another user; false, otherwise 
   */
  function canEditUserPrivileges() {
    return $this->getPrivilege() == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR;
  }
  
  /* **************************************************************** */
  /**
   * Replies true if an administrator user can see a role 
   *
   * <ul>
   *   <li>a Portal Administrator sees all roles
   *   <li>a Role Administrator sees all its own roles and
   *     all visible roles provided by othe administrators
   * </ul>
   *
   * @param Role  $res    a role (instance of class Role)
   *
   * @return boolean  true, if the admin user can see the role; false otherwise 
   */
  function canViewRole(&$role) {
    // a portal administrator sees every role
    // 
    if ($this->getPrivilege() == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR) {
      return true;
    }
            
    if ($role->isRoleProvider( $this )) {
        return true; 
    } 
    elseif ($role->isRoleManager($this)) {
        return true;
    }
    else {
        return $role->isVisible();
    } 
  }
  
  
  /* **************************************************************** */
  /*
   * Replies true, if the 
   *  administrator can manage subscriptions for
   *  role $role
   *
   *  @param Role  $role an instance of class Role 
   *
   *  @return boolean  true, if the 
   *    administrator can manage subscriptions for
   *    role $role
   */
  
  function canManageSubscriptions($role) {
     // a Portal Administrator can manage any resoruce 
     //
     $privilege= $this->getPrivilege();
     if ($privilege == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR) {
        return true; 
     }
     // a Role Administrator can manage its onw roles and role
     // where he is also manager
     //   
     if ($privilege == USER_PRIVILEGE_ID_RESOURCE_ADMINISTRATOR) {
        $is_owner= $role->isRoleProvider( $this );
        $is_manager= $role->isRoleManager( $this );
        return $is_owner or $is_manager;
     }

     // a Role Manager can manager only role where he is manager
     if ($privilege == USER_PRIVILEGE_ID_SUBSCRIPTION_ADMINISTRATOR) {
        $is_manager= $role->isRoleManager( $this );
        return $is_manager;
     }
     return false;    
  }
  
  /* **************************************************************** */
  /*
   * Replies true, if the current admin user can modify a subscription
   *
   * @param $subscription  The subscription object.
   */
  function canModifySubscription($subscription) {
    $role= $subscription->getRole();
    return $this->canManageSubscriptions($role);
  }
    
  /**
  * Replies true, if the administrator user can edit the
  *   user attributes of user $user
  *   
  * The following rules apply:
  * <ul>
  *   <li>all users are editable by a Portal Administrator
  *   <li>an admin user is editable to himself
  *   <li>no user except himself is editable to a Role Administrator
  * </ul>
  *
  * @param User $user  an instance of class User
  *
  * @return boolean true, if this administrator can edit the attributes of user $user; false
  *   otherwise 
  */  
  function canEditUser($user) {

     if ($this->getPrivilege() == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR) {
        return true;
     }

     if ($this->getPrivilege() == USER_PRIVILEGE_ID_RESOURCE_ADMINISTRATOR) {
        // the role administrator can see details about himself
        if ($this->getUserId() == $user->getUserId()) {
          return true;
        }
     }
    
     return false; 
  }
    
  /**
   * @return boolean true, if the current 
   *   admin user can create another user
   * 
   */
  function canCreateUser() {
    return $this->getPrivilege() == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR;
  }
  
  /**
   * @return boolean true, if the current 
   *   admin user can delete another user
   * 
   */
  function canDeleteUser() {
    return $this->getPrivilege() == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR;
  }
  
  /**
   * @return boolean true, if the current 
   *   admin user can edit another user
   * 
   */
  function canListUsers() {
    return $this->getPrivilege() == USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR;
  }
    
  


}

?>
