<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: ErrorHandling.class.inc,v 1.14 2007/11/28 15:49:14 vlry Exp $
 */

require_once('include/Logger.class.inc');
require_once('include/Event.class.inc');

/**
 * ErrorHandling is an utility class which provides static
 * utility functions only. 
 *
 * @version $Revision: 1.14 $
 * @package Library
 */
class ErrorHandling {

   /**
    * Displays an error message and aborts. The message is also logged and store in the Event DB.
    *
    * @param string $msg  the error message
    * @static
    */
   function fatalErrorAndExit() {
      $logger= Logger::getInstance();
      $args= func_get_args();
      echo "<b>FATAL:</b> ";
      $first= true;
      foreach($args as $msg) { 
          echo $msg;
          echo "<br/>";
          // store only the first message
          if ($first) {
              $errmsg= strip_tags($msg);
              Event::storeSystemEvent($errmsg, EVENT_LEVEL_ERROR);
              $first= false;
          }
          $logger->critical($msg);
      }
      
      // add support contact
      echo "<p>If you still have problems, contact the person who is responsible for the  role ";
      echo "or contact your Support (<a href=\"mailto:" . SUPPORT_EMAIL . "\">" . SUPPORT_EMAIL . "</a>) ";
      echo "or <a href=\"". SUPPORT_URL . "\">" . SUPPORT_URL . "</a><p/>";
      echo "Don't forget to add a copy of this error message, the URL, and the name of the role you try to access in either case."; 
      exit;
  }

    /**
     * Error handler for errors thrown by trigger_error(...,E_USER_ERROR).
     * Must be registered as:
     * set_error_handler("ErrorHandling::userErrorHandler", E_USER_ERROR) 
     * somewhere in application.inc
     */
    function userErrorHandler($errno, $errstr= null, $errfile= null, $errline= null) {
        // only handle E_USER_ERROR
        if ($errno == E_USER_ERROR) {
            $error_msg= "SNAportal Fatal Error [$errno]: $errstr ($errfile on line: $errline)";
            ErrorHandling::fatalErrorAndExit($error_msg);
        }
        else {
            // just log a warning
            $logger= Logger::getInstance();
            $error_msg= "userErrorHandler [$errno]: $errstr ($errfile on line: $errline)";
            $logger->warn($error_msg);
        }
        return true;     
    }
}


?>
