<?php
/* 
 * SNA-Portal mediates between one or more Authentication and Authorisation 
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 * 
 * 
 * $Id: Formatting.class.inc,v 1.9 2006/09/29 11:57:40 vlry Exp $ 
 */

/**
 * Formatting is an utility class which provides static 
 * utility functions for formatting HTML output.   
 *
 *  
 * @version $Revision: 1.9 $ 
 * @package Library 
 * @abstract  
 */ 
class Formatting {

    /**
     * Formats a MySQL timestamp    
     *   
     * @param string $date     a MySQL timestamp ('yyyymmddhhmmss' or 'yyyy-mm-dd hh:mm:ss')   
     * @param boolean $with_hour   true, if the format date shall include a
     *                             hour/minute part. Default is false.
     * @return a formated date/time string in the  format dd.mm.yyyy [hh:mm]
     *
     * @static   
     */
    function formatMySQLDate($date, $with_hour=false, $with_second=false) {
        $date= trim($date);
        // MySQL >= 4.1: 'yyyy-mm-dd hh:mm:ss' format 
        if (ereg("([0-9]{4})-([0-9]{2})-([0-9]{2}) ([0-9]{2}):([0-9]{2}):([0-9]{2})",
                 $date, $elts)) {
            $fdate= "$elts[3].$elts[2].$elts[1]";
            if ($with_hour or $with_second) {
                $fdate.= " [";
                if ($with_hour) {
                    $fdate.= "$elts[4]:$elts[5]";
                }
                if ($with_second) {
                    $fdate.= ":$elts[6]";
                }
                $fdate.= "]";
            }
        }
        // MySQL < 4.1: 'yyyymmddhhmmss' format 
        elseif (ereg("([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})",
                $date, $elts)) {
            $fdate= "$elts[3].$elts[2].$elts[1]";
            if ($with_hour or $with_second) {
                $fdate.= " [";
                if ($with_hour) {
                    $fdate.= "$elts[4]:$elts[5]";
                }
                if ($with_second) {
                    $fdate.= ":$elts[6]";
                }
                $fdate.= "]";
            }
        } 
        elseif (empty($date)) {
            $fdate= "Never";
        }
        else {
            $fdate= $date;
        }
        return $fdate;  
     }
     
    /**   
     * Formats the content of a table cell such that empty content is replaced 
     * with a non-breaking space (&nbsp;)
     * @param string $content   the content to be formatted   
     *    
     * @return string the formatted content 
     * @static   
     */  
    function formatTableCellContent($content) {      
        if ( trim($content) == "") {       
            return "&nbsp;";    
        } 
        else {      
            return $content;
        }  
    }
    
    /**   
     * Replies an SMS gateway URI for sending an SMS to specific   
     * phone number   
     *   
     * @param string  $phone_number   the phone number   
     *   
     * @return string  the URI (i.e. mailto:12345678@sms.switch.ch)   
     */  
    function formatSMSGatewayURI($phone_number) {    
        $uri = "mailto:" . Formatting::formatSMSGatewayEmailAddress($phone_number);
        return $uri;  
    }
    
    /**   
     * Replies the email address of the Email-To-SMS gateway 
     * for a specific phone number    
     *   
     * @param string  $phone_number   the phone number   
     *   
     * @return string  the email address  (i.e. 12345678@sms.switch.ch)   
     */    
    function formatSMSGatewayEmailAddress($phone_number) {    
        $phone_number = str_replace(" ", "", $phone_number);    
        $emailaddress = "$phone_number@sms.switch.ch";    
        return $emailaddress;  
    }
    
    /**   
     * constructor. Will generate an error if invoked.    
     *   
     * @access private    
     */  
    function Formatting() {
        trigger_error(
            "Formatting::_constructor: constructor can't be invoked because Formatting is a static utility class",
            E_USER_ERROR ); 
    }

}

?>
