<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 * Copyright (C) 2004-2005  SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: Gridlab.class.inc,v 1.5 2008/06/11 15:43:04 vlry Exp $
 */

/**
* Gridlab is a class that provides functions for adapters which
* use the "Role Management System" of RVS at University of Bern. 
* "Role Management System" is the successor of the LDAP based system
* for which Vitels.class.inc provided functions to connect to.
*
* @author Thomas Bernoulli <bernoull@iam.unibe.ch>
* @version  $Revision: 1.5 $
* @package Library
*/

require_once("include/Logger.class.inc");
require_once("include/NuSOAP.class.inc");


class Gridlab {
    //
    // member variables
    //
    var $wsdlUrl;
    var $usergroup;
    var $authname;
    var $password;
    // error handling
    var $is_error_= false;
    var $server_message_;

    /**
     * Default constructor
     */
    function Gridlab() {
        $this->usergroup= -1;
        $this->authname= "";
        $this->password= "";
    }
   
    /**
     *
     */
    function setCustomParams($customParams) {
        // get custom Gridlab webservice parameters
        if (isset($customParams["gridlab-wsdl-url"]) &&
            !empty($customParams["gridlab-wsdl-url"])) {
            $this->wsdlUrl= $customParams["gridlab-wsdl-url"];
        }
        if (isset($customParams["gridlab-usergroup"]) &&
            !empty($customParams["gridlab-usergroup"])) {
            $this->usergroup= $customParams["gridlab-usergroup"];
        }
        if (isset($customParams["gridlab-authname"]) &&
            !empty($customParams["gridlab-authname"])) {
            $this->authname= $customParams["gridlab-authname"];
        }
        if (isset($customParams["gridlab-password"]) &&
            !empty($customParams["gridlab-password"])) {
            $this->password= $customParams["gridlab-password"];
        }
    }
   
    /**
     * @return true if the user must be rejected.
     */
    function rejectLocalUser($aai_uid) {
        // do not forward a user with a local uid 
        // to an AAI protected Gridlab page
        if (!preg_match("/@/", $aai_uid)) {
            // seems like we have found a local user -> reject
            return true;
        }
        return false;
    }                        
   
    /**
     *
     */
    function assureRegistration($uid) {

        $soap= new nusoapclient($this->wsdlUrl, 'wsdl');
        
        $userRegParams= array(
            'authname' => $this->authname,
            'password' => $this->password,
            'aaiId' => $uid,
            'userId' => '',
            'isUserRegistered' => '',
            'infoString' => '');
        
        $userRegInfo= $soap->call('registerUser', $userRegParams);
    
        if( !isset($userRegInfo['isUserRegistered']) 
            || ($userRegInfo['isUserRegistered'] != 1)
            || $soap->fault) {
            $this->is_error_= true;
            $this->error_message_= "SOAP call 'registerUser' failed: " . $soap->faultstring . " [" . $soap->faultcode . "]";
            $logger= Logger::getInstance();
            $logger->error("Gridlab::assureRegistration($uid): " . $this->error_message_ );

            return false;
        }
        
        $groupRegParams= array(
            'authname' => $this->authname,
            'password' => $this->password,
            'userId' => $userRegInfo['userId'],
            'groupId' => $this->usergroup,
            'isUserAdded' => '');
        
        $groupRegInfo= $soap->call('registerUserToGroup', $groupRegParams);
        
        if(!isset($groupRegInfo['isUserAdded']) 
            || ($groupRegInfo['isUserAdded'] != 1)
            || $soap->fault) {
            $this->is_error_= true;
            $this->error_message_= "SOAP call 'registerUserToGroup' failed: " . $soap->faultstring . " [" . $soap->faultcode . "]";
            $logger= Logger::getInstance();
            $logger->error("Gridlab::assureRegistration($uid): " . $this->error_message_ );
            return false;
        }

        return true;
    }

    /**
    * @return boolean return true if an error occurs while registering the user to the course.
    */
    function isError() {
        return $this->is_error_;   
    }
    
    /**
    * @return string The Gridlab role management server error message.
    */
    function getErrorMessage() {
        return $this->error_message_;   
    }
    
}   
    
?>
