<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: UserPrivilege.class.inc,v 1.11 2007/10/12 09:26:33 vlry Exp $
 */
/* ****************************************************************** */


/**
*/

/** the user privilege ID for a normal user */
define("USER_PRIVILEGE_ID_USER",                     1);
/** the user privilege ID for a role administrator */
define("USER_PRIVILEGE_ID_RESOURCE_ADMINISTRATOR",   2);
/** the user privilege ID for a portal administrator */
define("USER_PRIVILEGE_ID_PORTAL_ADMINISTRATOR",     3);
/** the user privilege ID for a subscription administrator */
define("USER_PRIVILEGE_ID_SUBSCRIPTION_ADMINISTRATOR",         4);



/**
 * This is a utility class for retrieving meta-data about user
 * privileges from the databases. 
 *
 * 
 * @version $Revision: 1.11 $
 * @package Library
 */
class UserPrivilege {

  /* ********************************************************************** */
  /**
   * Fetches all user privileges from the database
   *
   * Returns an array of arrays. For fields in a user privilege records
   * the same names are used as in the table UserPrivilege.
   *
   * <b>Sample usage</b>
   * <pre>
   *   $privileges = UserPrivileges::getPrivileges();
   *   foreach($privileges as $id => $privilege) {
   *      echo "Privilege ID ($id) : Name is " . $privilege["userprivilege_name"]; 
   *      echo "Privilege ID ($id) : Description is " . $privilege["userprivilege_description"]; 
   *   }
   * </pre>
   *
   * @param MySQLConnection $conn   an open MySQL connection
   *
   * @return array  an array of UserPrivilege records, ordered by userprivilege_id.
   *   The userprivilege_id is the key of the array.
   *
   * @static
   *
   */ 
  function getPrivileges() {
     $query= " SELECT * " 
           . " FROM UserPrivilege "
           . " ORDER BY userprivilege_id ";
   
     $conn= MySQLConnection::getConnection();
     $conn->query($query);

     $ret = array();
     while($row = $conn->fetch_array()) {
      $ret[$row["userprivilege_id"]] = $row;
     }
     $conn->freeresult();
     return $ret;
  }
  
  /**
   * Returns a hash table ( privilege_id => privilege_description )
   * @static
   */
  function getDescriptions() {
    $query= "SELECT userprivilege_id, userprivilege_description " 
          . "FROM UserPrivilege "
          . "ORDER BY userprivilege_id";
           
    $conn= MySQLConnection::getConnection();
    $conn->query($query);
    $descriptions= array();
    while( $row= $conn->fetch_array() ) {
        $descriptions[$row['userprivilege_id']]= $row['userprivilege_description'];
    }
    $conn->freeresult();
    return $descriptions;

  }

  /**
   * Returns privilege_description for the given privilegeid, or null;
   * @static
   */
  function getDescription($privilegeid) {
    $query= "SELECT userprivilege_description " 
          . "FROM UserPrivilege "
          . "WHERE userprivilege_id = $privilegeid";
           
    $conn= MySQLConnection::getConnection();
    $conn->query($query);
    $description= null;
    if ($conn->num_rows() > 0) {
      $row = $conn->fetch_array(); 
      $conn->freeresult();
      $description= $row['userprivilege_description'];
    }
    return $description;

  }


} // end class

?>
