<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 * Copyright (C) 2004  SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: WebCTAPI_Request.class.inc,v 1.8 2006/12/01 13:13:17 vlry Exp $
 */

require_once("include/webct/WebCTAPI.inc");
require_once("include/webct/WebCTIMS_Request.class.inc");
require_once("include/webct/WebCTIMS_SwitchAAIAdapting.class.inc");

/**
 * The list of WebCT API paramters NOT taken into consideration for
 * the MAC computation. 
 *
 * @var array 
 * @access private
 */
$_WEBCTCE_API_PARAMS_NOT_IN_MAC= array(
     WEBCTCE_API_PARAM_AUTH,
     WEBCTCE_API_PARAM_ENCRYPTED,
     WEBCTCE_API_PARAM_USERTYPE,
     WEBCTCE_API_PARAM_CHARSET,
);

/**
* WebCT CE API Request class. 
*
* This class is used to handle requests to the WebCT CE API adapter.
* 
* @version $Revision: 1.8 $
* @author Valery Tschopp - SWITCH
* @package WebCTAPI
*/
class WebCTAPI_Request extends WebCTIMS_Request {

   /* ***************************************************** */
   /*                  static factory methods               */
   /* ***************************************************** */

    /**
     * Returns a Standard API request to create a new user in the global database.
     * 
     * @param string $webctid  The new user WebCT ID.
     * @param array $fileds    The fields name-value of the user.
     *                         Valid name: Last Name, First Name, Courses, Registered Courses
     * @param string $password Optional password. If not defined a new random password
     *                         is generated.
     * @return The WebCTAPI_Request object.
     */
    function createAddUserGlobalDatabaseRequest($webctid, $fields= array(), $password= null) {
        $request= new WebCTAPI_Request();
        $request->setParameter(WEBCTCE_API_PARAM_OPERATION, WEBCTCE_API_OPERATION_ADD);
        $request->setParameter(WEBCTCE_API_PARAM_DB, WEBCTCE_API_DB_GLOBAL);
        $request->setParameter(WEBCTCE_API_PARAM_COURSE, WEBCTCE_API_COURSE_PLACEHOLDER);
        $request->setParameter(WEBCTCE_API_PARAM_WEBCTID, $webctid);
        // set or create password
        if (is_null($password)) {
            $password= WebCTIMS_SwitchAAIAdapting::createPassword();
        }
        if (CRYPT_STD_DES == 1) {
            // send pre-encrypted (Unix DES) password
            $crypted_password= crypt($password);
            $request->setParameter(WEBCTCE_API_PARAM_PASSWORD, $crypted_password);
            $request->setParameter(WEBCTCE_API_PARAM_ENCRYPTED, WEBCTCE_API_ENCRYPTED_ENABLE);
        }
        else {
            // Unix DES not available, send clair-text
            $request->setParameter(WEBCTCE_API_PARAM_PASSWORD, $password);
            $request->setParameter(WEBCTCE_API_PARAM_ENCRYPTED, WEBCTCE_API_ENCRYPTED_DISABLE);
        }

        // add all fields
        foreach( $fields as $name => $value ) {
            $request->setParameter($name, $value);
        }

        return $request;
    }

    function createUpdateUserGlobalDatabaseRequest($webctid, $fields= array()) {
        $request= new WebCTAPI_Request();
        $request->setParameter(WEBCTCE_API_PARAM_OPERATION, WEBCTCE_API_OPERATION_UPDATE);
        $request->setParameter(WEBCTCE_API_PARAM_DB, WEBCTCE_API_DB_GLOBAL);
        $request->setParameter(WEBCTCE_API_PARAM_COURSE, WEBCTCE_API_COURSE_PLACEHOLDER);
        $request->setParameter(WEBCTCE_API_PARAM_WEBCTID, $webctid);
        // add all fields
        foreach( $fields as $name => $value ) {
            $request->setParameter($name, $value);
        }

        return $request;
    }


    function createFindUserGlobalDatabaseRequest($webctid) {
        $request= new WebCTAPI_Request();
        $request->setParameter(WEBCTCE_API_PARAM_OPERATION, WEBCTCE_API_OPERATION_FIND);
        $request->setParameter(WEBCTCE_API_PARAM_DB, WEBCTCE_API_DB_GLOBAL);
        $request->setParameter(WEBCTCE_API_PARAM_WEBCTID, $webctid);
        $request->setParameter(WEBCTCE_API_PARAM_COURSE, WEBCTCE_API_COURSE_PLACEHOLDER);
        $request->setParameter(WEBCTCE_API_PARAM_USERTYPE, WEBCTCE_API_USERTYPE_ENABLE);
    
        return $request;
    }

    function createAddUserStudentDatabaseRequest($userid, $courseid, $fields= array(), $password= null ) {
        $request= new WebCTAPI_Request();
        $request->setParameter(WEBCTCE_API_PARAM_OPERATION, WEBCTCE_API_OPERATION_ADD);
        $request->setParameter(WEBCTCE_API_PARAM_DB, WEBCTCE_API_DB_STUDENT);
        $request->setParameter(WEBCTCE_API_PARAM_COURSE, $courseid);
        $request->setParameter(WEBCTCE_API_PARAM_USERID, $userid);
        if (is_null($password)) {
            $password= WebCTIMS_SwitchAAIAdapting::createPassword();
        }
        if (CRYPT_STD_DES == 1) {
            // send pre-encrypted (Unix DES) password
            $crypted_password= crypt($password);
            $request->setParameter(WEBCTCE_API_PARAM_PASSWORD, $crypted_password);
            $request->setParameter(WEBCTCE_API_PARAM_ENCRYPTED, WEBCTCE_API_ENCRYPTED_ENABLE);
        }
        else {
            // Unix DES not available, send clair-text
            $request->setParameter(WEBCTCE_API_PARAM_PASSWORD, $password);
            $request->setParameter(WEBCTCE_API_PARAM_ENCRYPTED, WEBCTCE_API_ENCRYPTED_DISABLE);
        }
        // add all fields
        foreach( $fields as $name => $value ) {
            $request->setParameter($name, $value);
        }

        return $request;
    }

    function createUpdateUserStudentDatabaseRequest($userid, $courseid, $fields= array()) {
        $request= new WebCTAPI_Request();
        $request->setParameter(WEBCTCE_API_PARAM_OPERATION, WEBCTCE_API_OPERATION_UPDATE);
        $request->setParameter(WEBCTCE_API_PARAM_DB, WEBCTCE_API_DB_STUDENT);
        $request->setParameter(WEBCTCE_API_PARAM_COURSE, $courseid);
        $request->setParameter(WEBCTCE_API_PARAM_USERID, $userid);
        // add all fields
        foreach( $fields as $name => $value ) {
            $request->setParameter($name, $value);
        }

        return $request;
    }


   /* ***************************************************** */
   /*                  private members                      */
   /* ***************************************************** */
   
   
   /* ***************************************************** */
   /*                  constructor                          */
   /* ***************************************************** */
   
   /**
   * Constructor  
   *
   * $apiParameters is an array with  initialization values for the request URL
   * and the shared secret. The request URL is initialized according to the
   * following rules:
   * <ol>
   *   <li>to the value of the parameter "WEBCT_AS_REQUEST_URL" if this
   *     parameter is set in $apiParameters</li>
   *   <li>to the value of the constant WEBCT_AS_REQUEST_URL, if this
   *     constant is defined</li>
   *   <li>to false</li>
   * </ol>
   *
   * The shared secret is initialized according to the following
   * rules:
   * <ol>
   *   <li>to the parameter "WEBCT_AS_SHARED_SECRET", if this parameter is
   *     set in the array $apiParameters</li>
   *   <li>to the value of the constant WEBCT_AS_SHARED_SECRET, if this
   *     constant is defined</li>
   *   <li>to the content of the file whose name is given by the parameter
   *      "WEBCT_AS_SHARED_SECRET_FILE", if this parameter is set in the
   *      array $apiParameters</li>
   *   <li>to the content of the file whose name is given by the constant
   *      WEBCT_AS_SHARED_SECRET_FILE, if this constant is defined 
   *      </li>
   *   <li>to false</li>
   * </ol>
   *
   * @param  array $parameters  an array of request parameters 
   */
   function WebCEAPI_Request($parameters = NULL,$apiParameters = NULL) {     

     $this->_parameters= array();

     if (! is_null($parameters)) {
        $this->setParameters($parameters);    
     }
    
     // init the request URL
     //
     if (! is_null($apiParameters) && isset($apiParameters["WEBCTCE_API_REQUEST_URL"])) {
       $this->setRequestUrl($apiParameters["WEBCTCE_API_REQUEST_URL"]);  
     } elseif (defined("WEBCTCE_API_REQUEST_URL")) {
       $this->setRequestUrl(WEBCTCE_API_REQUEST_URL);  
     }

     // init the shared secret 
     //
     if (! is_null($apiParameters) && isset($apiParameters["WEBCTCE_API_SHARED_SECRET"])) {
       $this->setSharedSecret($apiParameters["WEBCTCE_API_SHARED_SECRET"]);  
     } elseif (defined("WEBCTCE_API_SHARED_SECRET")) {
       $this->setSharedSecret(WEBCTCE_API_SHARED_SECRET);  
     }    
   }
    
   /* ***************************************************** */
   /*                  setters and getters                  */
   /* ***************************************************** */
 
   /**
   * sets the value of a  request parameter
   *
   * @param string $name  the name of a request parameter
   * @param  string  $value the value of the request parameter
   */
   function setParameter($name, $value) {
      $this->_parameters[$name] = $value; 
   }
 
   /**
   * Computes the Message
   * Authentication Code (MAC) for this request and sets
   * the respective request parameters.
   *
   * The MAC is saved in the parameter
   * {@link WEBCTCE_API_PARAM_AUTH}. It can be retrieved
   * with the method {@link getParameter() }. 
   *
   * @global $_WEBCTCE_API_PARAMS_NOT_IN_MAC
   */   
    function refreshChecksumAndMAC() {
        global $_WEBCTCE_API_PARAMS_NOT_IN_MAC;
      
        // build up the MAC data string
        //
        $mac_data = "";
        foreach( $this->_parameters as $parameter => $value ) {
            if (! in_array($parameter, $_WEBCTCE_API_PARAMS_NOT_IN_MAC) ) {
                $mac_data.= $value;
            } 
        }

        //XXX echo "XXX: refreshChecksumAndMAC:mac_data: $mac_data\n";

        // compute the checksum over the MAC data string
        //
        $checksum= 0;
        $mac_data_l= strlen($mac_data);
        for ($i= 0; $i < $mac_data_l; $i++) {
             $checksum += ord($mac_data[$i]);
        }
      
        //XXX echo "XXX: refreshChecksumAndMAC:checksum: $checksum\n";

        // compute the MAC
        //
        $secret = $this->getSharedSecret();
        if ($secret == false) {
            trigger_error(
                "WebCTAPI_Request::refreshMAC: potential configuration error - shared secret is not set",
                E_USER_ERROR
            );
        }
        $mac = strtoupper(md5("$checksum$secret"));

        //XXX echo "XXX: refreshChecksumAndMAC:mac: $mac\n";

     
        // save the MAC as parameter 
        //
        $this->setParameter(WEBCTCE_API_PARAM_AUTH, $mac);
    }      

    /**
    * REFACTORING: Returns the request parameters as application/x-www-form-urlencoded.
    *
    * @param HANDLE $handle   The file handle to write.
    */
    function writeAsMimeMultipart($handle) {

        // refresh checksum and MAC
        //
        $this->refreshChecksumAndMAC();

        // urlencode POST parameters
        //
        $form= "";
        foreach ($this->_parameters as $parameter => $value) {
            $parameter= rawurlencode($parameter);
            $value= rawurlencode($value);
            //XXX print "XXX: $parameter => $value\n";
            $form.= "$parameter=$value&";
        }
        // remove last &
        $form= substr( $form, 0, -1 );
        $form_length= strlen( $form );
        $form.= "\r\n";

        $headers= "";
        $headers.= "Content-Length: $form_length\r\n";
        $headers.= "Content-Type: application/x-www-form-urlencoded\r\n";
        $headers.= "\r\n";

        $logger= Logger::getInstance();
        if ($logger->isDebugEnable()) {
            $logger->debug("WebCTAPI_Request::writeAsMimeMultipart: " . $headers);
            $logger->debug("WebCTAPI_Request::writeAsMimeMultipart: " . $form);
        }

        //XXX
        //print "---API_Request---HEADER BEGIN---\n";
        //print $headers;
        //print "---API_Request---HEADER END---\n";
         //XXX
        //print "---API_Request---FORM BEGIN---\n";
        //print $form;
        //print "---API_Request---FORM END---\n";
 
        fwrite($handle, $headers); 
        fwrite($handle, $form); 
    }

}


?>
