<?PHP
/*
* SNAportal mediates between one or more Authentication and Authorisation
* Infrastructures and non-AAI-enabled roles.
*
* Copyright (C) 2003 Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
* Copyright (C) 2004-2006 SWITCH (http://www.switch.ch)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
* $Id: WebCTCE6RegisterAdapter.inc,v 1.8 2006/03/03 11:37:46 vlry Exp $
*/

require_once("include/webct/WebCTVistaRegisterAdapter.inc");

/**
 * WebCT CE 6 Constants
 */ 
define('WEBCTCE6_SECTION_URL','/webct/startFrameSet.dowebct?forward=manageCourse.dowebct&lcid=');

/**
* The WebCTCE6RegisterAdapter process the enrollement + autosignon request.
*
* OOP Inheritance:
*   WebCTVistaLoginAdapter -<- WebCTVistaRegisterAdapter -<- WebCTCE6RegisterAdapter
*
* @version $Revision: 1.8 $
* @author Valery Tschopp - SWITCH
*
* @package WebCTAPI
*/
class WebCTCE6RegisterAdapter extends WebCTVistaRegisterAdapter {

    /**
    * Constructor:
    * Creates a new WebCT CE6 Register Adapter. This adapter will create/update the user,
    * enroll him to the given course and proceed to an autosignon.
    *
    * @param string $server_url The WebCT Vista server basename URL.
    * @param string $glcid The target institution glcId (IMS ID).
    * @param string $ims_secret The IMS API shared secret.
    * @param string $standard_secret The Standard API shared secret.
    */
    function WebCTCE6RegisterAdapter($server_url, $glcid, 
                                     $ims_secret, $standard_secret) {
        // super init
        parent::WebCTVistaRegisterAdapter($server_url, $glcid, 
                                          $ims_secret, $standard_secret);
    }

    /**
     * Returns the XML encoding used in IMS data
     * @return string The XML encoding: UTF-8
     * @protected
     * @see WebCTVistaRegisterAdapter:get_xml_encoding()
     */
    function get_xml_encoding() {
        return "UTF-8";
    }
    
    /**
    * Returns the AutoSigon URL for the section (course) given as parameter.
    *
    * @param string $section_lcid WebCT CE6 Section Learning Context ID.
    * @return string The studentCourseView URL.
    *
    * @protected
    * @see WebCTVistaLoginAdapter:get_autosignon_url($consortia_id,$courseid)
    */
    function get_course_url($section_lcid) {
        //TODO: find lc_id from Course IMS ID 
        // Section IMS ID -> Section Learning Contect ID
        // $lcid= get_lc_id( ims_id )
        return $this->server_url_ . WEBCTCE6_SECTION_URL . $section_lcid;      
    }


}

?>
