<?PHP
/*
 * Copyright (C) 2003 Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 * Copyright (C) 2004 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: WebCTCELoginAdapter.inc,v 1.6 2006/11/28 12:19:24 vlry Exp $
 */

require_once("include/Logger.class.inc");
require_once("include/webct/WebCTCEAdapter.inc");
require_once("include/webct/WebCTAS.inc");

/**
 * Default url paths for WebCT CE courses and MyWebCT pages.
 */
define('WEBCTCE_DEFAULT_MYWEBCT_URL', 'homearea');
define('WEBCTCE_DEFAULT_COURSE_PATH', 'courseid://');


/**
* The WebCTCELoginAdapter process the AutoSignon.
*
* @version $Revision: 1.6 $
* @author Valery Tschopp - SWITCH
*
* @package WebCTAPI
*/
class WebCTCELoginAdapter extends WebCTCEAdapter {
    
    /** The IMS Source identifier */
    var $ims_source_= null;
        
    /**
    * Constructor:
    * Creates a new WebCT Vista Login (AutoSignon) Adapter.
    *
    * @param string $server_url The WebCT Vista server basename URL.
    * @param string $as_secret The AS shared secret.
    */
    function WebCTCELoginAdapter( $server_url, $as_secret ) {
        parent::WebCTCEAdapter( $server_url, $as_secret );
        
        // set default IMS source identifier
        $this->ims_source_= WEBCTCE_DEFAULT_IMS_SOURCE;
    }

    /**
     * Sets the IMS source identifier.
     */
    function setIMSSource( $ims_source ) {
        $this->ims_source_= $ims_source;
    }
    
    /**
    * Sends a redirect to client browser pointing to WebCT Vista AutoSignon handler.
    * After sending a HTTP redirect the function <code>exit</code>.
    *
    * @param string $uniqueid WISEBEDaai Shib-WISEBEDPerson-personUniqueID.
    * @param string $courseid The optional courseID. If not specified, MyWebCT homepage is used as default.
    */
    function autosignon( $uniqueid, $courseid= null ) {
        // redirect browser to autosignon url
        $location= $this->get_autosignon_url($uniqueid, $courseid);
        header( "Location: $location" );
        exit;
    }
    
    /** 
     * Builds the AutoSignon URL. 
     *
     * The URL will be either the direct course entry point or MyWebCT entry point if no
     * course id is given.
     *
     * @param string $uniqueid WISEBEDaai Shib-WISEBEDPerson-personUniqueID.
     * @param string $courseid WebCT CE course ID. If null the function returns MyWebCT url.
     * @return string The AutoSignon URL.
     * @private
     */
    function get_autosignon_url( $uniqueid, $courseid= null ) {
        $logger= Logger::getInstance();
        if ($logger->isDebugEnable()) {
            $logger->debug("get_autosignon_url($uniqueid,$courseid)");
        }
        // create new autosignon request
        $as_req= new WebCTAS_Request();

        $as_req_url= $this->getASRequestURL();
        $as_req->setRequestUrl( $as_req_url );
        $as_req->setSharedSecret( $this->secret_ );

        // set timestamp        
        $as_req->setParameter(WEBCT_AS_PARAM_TIMESTAMP, mktime());

        // set IMS id 
        $ims_id= $uniqueid;
        $as_req->setParameter( WEBCT_AS_PARAM_IMS_ID, $ims_id );

        // set IMS source
        $as_req->setParameter( WEBCT_AS_PARAM_IMS_SOURCE, $this->ims_source_ );

        // set URL
        $url= null;
        if (is_null($courseid)) {
            // read a default somewhere
            $url= $this->get_mywebct_url();
        }
        else {
            $url= $this->get_course_url( $courseid );
        }
        $as_req->setParameter( WEBCT_AS_PARAM_URL, $url );

        // build the autosignon url
        $location= $as_req->buildAutosignonUrl();
        if ($logger->isDebugEnable()) {
            $logger->debug("get_autosignon_url=$location");
        }
        
        return $location;
    }
    
    /**
    * @return string The default MyWebCT URL.
    * @private
    */
    function get_mywebct_url() {
        return WEBCTCE_DEFAULT_MYWEBCT_URL;
    }

    /**
    * Returns the URL for a particular given course.
    * Format: courseid://<courseid>
    *
    * @param string $courseid The course ID
    * @return string The course URL.
    * @private
    */
    function get_course_url( $courseid ) {
        return WEBCTCE_DEFAULT_COURSE_PATH . $courseid;
    }
    
    
}

?>
