<?php
/*
* SNA-Portal mediates between one or more Authentication and Authorisation
* Infrastructures and non-AAI-enabled roles.
* Copyright (C) 2003 Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
* Copyright (C) 2004 SWITCH (http://www.switch.ch)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
* $Id: WebCTVistaIMS_Request.inc,v 1.7 2006/09/07 15:06:30 vlry Exp $
*/

require_once("include/webct/WebCTVista.inc");
require_once("include/webct/WebCTVistaIMS_Constants.inc");
require_once("include/webct/WebCTIMS_Request.class.inc");

/**
* The list of WebCT Vista IMS paramters taken into consideration for
* the MAC computation.
*
* @var array
* @access private
*/
$_WEBCTVISTA_IMS_MAC_PARAM_NAMES = array(
WEBCTVISTA_IMS_PARAM_ACTION,
WEBCTVISTA_IMS_PARAM_OPTION,
WEBCTVISTA_IMS_PARAM_WEBCTID,
WEBCTVISTA_IMS_PARAM_TIMESTAMP,
WEBCTVISTA_IMS_PARAM_GLCID,
WEBCTVISTA_IMS_PARAM_CHECKSUM,
WEBCTVISTA_IMS_PARAM_IMS_ID,
WEBCTVISTA_IMS_PARAM_IMS_SOURCE
);

$_WEBCTVISTA_IMS_REQUEST_PARAM_NAMES = array(
WEBCTVISTA_IMS_PARAM_ADAPTOR,
WEBCTVISTA_IMS_PARAM_ACTION,
WEBCTVISTA_IMS_PARAM_OPTION,
WEBCTVISTA_IMS_PARAM_WEBCTID,
WEBCTVISTA_IMS_PARAM_TIMESTAMP,
WEBCTVISTA_IMS_PARAM_GLCID,
WEBCTVISTA_IMS_PARAM_MAC,
WEBCTVISTA_IMS_PARAM_IMS_ID,
WEBCTVISTA_IMS_PARAM_IMS_SOURCE
);


/**
* WebCTVistaIMS_Request represents a request to the WebCT Vista IMS Adapter.
*
* @author Valery Tschopp - SWITCH
* @version $Revision: 1.7 $
*
* @package WebCTAPI
*/
class WebCTVistaIMS_Request extends WebCTIMS_Request {
    
    /* ***************************************************** */
    /*                  static factory methods               */
    /* ***************************************************** */
    
    /**
    * Creates a WebCT IMS request of type "import".
    *
    * The request is initialized as follows:
    * <ul>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_ACTION} is set to {@link WEBCTVISTA_IMS_ACTION_IMPORT}</li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_OPTION} is set to {@link WEBCTVISTA_IMS_OPTION_RESTRICT}</li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_TIMESTAMP} is set to the current EPOCH: <code>time()</code></li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_ADAPTOR} is set to {@link WEBCTVISTA_IMS_ADAPTOR_IMS}</li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_GLCID} is set to param $glcid</li>
    * </ul>
    *
    * @param string $glcId The glcId of target WEbCT Vista institution.
    * @param WebCTIMS_IMSDataObject $ims_data_obj The IMS Data Object for the import request.
    * @return WebCTVistaIMS_Request the initialized "import" request.
    *
    * @static
    */
    function createImportRequest($glcid, $ims_data_obj) {
        
        // create the request
        //
        $request = new WebCTVistaIMS_Request();
        
        // init the request parameters with default values
        //
        $request->setParameter(WEBCTVISTA_IMS_PARAM_ADAPTOR, WEBCTVISTA_IMS_ADAPTOR_IMS);
        $request->setParameter(WEBCTVISTA_IMS_PARAM_ACTION, WEBCTVISTA_IMS_ACTION_IMPORT);
        $request->setParameter(WEBCTVISTA_IMS_PARAM_OPTION, WEBCTVISTA_IMS_OPTION_RESTRICT);
        $request->setParameter(WEBCTVISTA_IMS_PARAM_TIMESTAMP, time());
        $request->setParameter(WEBCTVISTA_IMS_PARAM_GLCID, $glcid);
        
        $request->setIMSDataObject($ims_data_obj);
        
        return $request;
    }

    /**
    * Creates a WebCT Standard request of type "get".
    *
    * The request is initialized as follows:
    * <ul>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_ACTION} is set to {@link WEBCTVISTA_IMS_ACTION_GET}</li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_OPTION} is set to {@link WEBCTVISTA_IMS_OPTION_CONSORTIAID}</li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_TIMESTAMP} is set to the current EPOCH: <code>time()</code></li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_ADAPTOR} is set to {@link WEBCTVISTA_IMS_ADAPTOR_STANDARD}</li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_GLCID} is set to param $glcid</li>
    * </ul>
    *
    * @param string $glcId The glcId of target WEbCT Vista institution.
    * @return WebCTVistaIMS_Request the initialized "get consortia_id" request.
    *
    * @static
    */
    function createGetConsortiaIdRequest($glcid,$webctid) {
        
        // create the request
        //
        $request = new WebCTVistaIMS_Request();
        
        // init the request parameters with default values
        //
        $request->setParameter(WEBCTVISTA_IMS_PARAM_ADAPTOR, WEBCTVISTA_IMS_ADAPTOR_STANDARD);
        $request->setParameter(WEBCTVISTA_IMS_PARAM_ACTION, WEBCTVISTA_IMS_ACTION_GET);
        $request->setParameter(WEBCTVISTA_IMS_PARAM_OPTION, WEBCTVISTA_IMS_OPTION_CONSORTIAID);
        $request->setParameter(WEBCTVISTA_IMS_PARAM_TIMESTAMP, time());
        $request->setParameter(WEBCTVISTA_IMS_PARAM_GLCID, $glcid);
        $request->setParameter(WEBCTVISTA_IMS_PARAM_WEBCTID, $webctid);

        return $request;
    }
    
   /**
    * Creates a WebCT IMS request of type "export".
    *
    * The request is initialized as follows:
    * <ul>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_ADAPTOR} is set to {@link WEBCTVISTA_IMS_ADAPTOR_IMS}</li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_ACTION} is set to {@link WEBCTVISTA_IMS_ACTION_EXPORT}</li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_OPTION} is set to {@link WEBCTVISTA_IMS_OPTION_PERSONRECORD}</li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_TIMESTAMP} is set to the current EPOCH: <code>time()</code></li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_GLCID} is set to param $glcid</li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_IMS_ID} is set to param $person_ims_id</li>
    *  <li>{@link WEBCTVISTA_IMS_PARAM_IMS_SOURCE} is set to param $person_ims_source</li>
    * </ul>
    *
    * @param string $glcid The glcId of target WEbCT Vista institution.
    * @param string $person_ims_id The person.sourcedid.id (IMS ID).
    * @param string $person_ims_source The person.sourcedid.source (IMS Source).
    * @return WebCTVistaIMS_Request the initialized "export person_record" request.
    *
    * @static
    */
    function createExportPersonRequest($glcid,$person_ims_id,$person_ims_source) {
        
        // create the request
        //
        $request = new WebCTVistaIMS_Request();
        
        // init the request parameters with default values
        //
        $request->setParameter(WEBCTVISTA_IMS_PARAM_ADAPTOR, WEBCTVISTA_IMS_ADAPTOR_IMS);
        $request->setParameter(WEBCTVISTA_IMS_PARAM_ACTION, WEBCTVISTA_IMS_ACTION_EXPORT);
        $request->setParameter(WEBCTVISTA_IMS_PARAM_OPTION, WEBCTVISTA_IMS_OPTION_PERSONRECORD);
        $request->setParameter(WEBCTVISTA_IMS_PARAM_TIMESTAMP, time());
        $request->setParameter(WEBCTVISTA_IMS_PARAM_GLCID, $glcid);
        $request->setParameter(WEBCTVISTA_IMS_PARAM_IMS_ID, $person_ims_id);
        $request->setParameter(WEBCTVISTA_IMS_PARAM_IMS_SOURCE, $person_ims_source);

        return $request;
    }
    
    /**
    * constructor
    *
    * @param  array $parameters  an array of request parameters
    */
    function WebCTVistaIMS_Request($parameters = NULL) {
        parent::WebCTIMS_Request($parameters);
    }
    
    /* ***************************************************** */
    /*                  setters and getters                  */
    /* ***************************************************** */
    
    /**
    * sets the value of a  request parameter
    *
    * @param string $name  the name of a request parameter
    * @param  string  $value the value of the request parameter
    */
    function setParameter($name, $value) {
        $this->_parameters[$name] = $value;
    }
    
    /**
    * Replies the value of a request parameter.
    *
    * @param string $name  the name of a request parameter
    * @return  string the value of the request parameter
    */
    function getParameter($name) {
        if (! isset($this->_parameters, $name)) {
            $value= null;
            return $value; // null
        }
        return $this->_parameters[$name];
    }
    
    /**
    * Sets the request parameters.
    *
    * @param  array $params the request parameters
    */
    function setParameters($params) {
        if (! $params || ! is_array($params)) {
            trigger_error(
            "WebCTVistaIMS_Request::setParameters: expected non-null array in parameter \$params, got '$params'",
            E_USER_WARNING
            );
            return;
        }
        $this->_parameters = array();
        foreach($params as $name => $value) {
            $this->_parameters[$name] = $value;
        }
    }
    
    /* ***************************************************** */
    /*                 checksum and mac calculation          */
    /* ***************************************************** */
    
    /**
    * Computes the IMS data object checksum and the Message
    * Authentication Code (MAC) for this request and sets
    * the respective request parameters.
    *
    * The IMS data object checksum is saved in the parameter
    * {@link WEBCTVISTA_IMS_PARAM_CHECKSUM}. It can be retrieved
    * with the method {@link getParameter() }.
    *
    * The MAC is saved in the parameter
    * {@link WEBCTVISTA_IMS_PARAM_MAC}. It can be retrieved
    * with the method {@link getParameter() }.
    *
    * @see getParameter()
    * @see WebCTIMS_IMSDataObject::getChecksumAsString()
    *
    * @global $_WEBCTVISTA_IMS_MAC_PARAM_NAMES
    *
    */
    function refreshChecksumAndMAC() {
        global $_WEBCTVISTA_IMS_MAC_PARAM_NAMES;
        
        // get file checksum
        $checksum= "";
        if (! is_null($this->getIMSDataObject())) {
            $ims_data_obj = $this->getIMSDataObject();
            $checksum = $ims_data_obj->getChecksumAsString();
            $this->setParameter(WEBCTVISTA_IMS_PARAM_CHECKSUM, $checksum);
        } else {
            unset($this->_parameters[WEBCTVISTA_IMS_PARAM_CHECKSUM]);
        }
        
        $mac_data= "";
        foreach ($_WEBCTVISTA_IMS_MAC_PARAM_NAMES as $param_name) {
            if (isset($this->_parameters[$param_name])) {
                $value= $this->_parameters[$param_name];
                //XXX
                //print "XXX: $param_name => $value\n";
                $mac_data.= $value;
            }
        }
        
        // generate MAC
        $secret= $this->getSharedSecret();
        if ($secret == false) {
            trigger_error(
            "WebCTVistaIMS_Request::refreshChecksumAndMAC: potential configuration error - shared secret is not set",
            E_USER_ERROR
            );
        }
        $mac= WebCTVista::generate_MAC($mac_data, $secret);
        $this->setParameter(WEBCTVISTA_IMS_PARAM_MAC, $mac);
        
        //XXX
        //print "XXX: mac: $mac\n";
    }
    
    /**
    * REFACTORING: Returns the request parameters as MIME multipart.
    *
    * @param string $mime_boundary The MIME boundary tag.
    * @return string The MIME multipart content.
    */
    function getParametersAsMimeMultipart( $mime_boundary ) {
        global $_WEBCTVISTA_IMS_REQUEST_PARAM_NAMES;
        $mime= "";
        foreach ($_WEBCTVISTA_IMS_REQUEST_PARAM_NAMES as $name) {
            if (isset($this->_parameters[$name])) {
                $value= $this->_parameters[$name];
                //XXX
                //print "XXX: $name => $value\n";
                $mime.= "--$mime_boundary\r\n";
                $mime.= "Content-Disposition: form-data; name=\"$name\"";
                $mime.= "\r\n\r\n";
                $mime.= "$value\r\n";
            }
        }
        
        return $mime;
        
    }
    
    /**
    * REFACTORING: Returns the IMS XML data object as a file MIME multipart.
    * <b>XML encoding UTF-8</b>
    *
    * @param string $mime_boundary The MIME boundary tag.
    * @return string The MIME multipart content.
    */
    function getImsXmlAsMimeMultipart( $mime_boundary ) {
        
        $mime= "";
        if (! is_null($this->getIMSDataObject())) {
            $ims_data_obj= $this->getIMSDataObject();
            // write the IMS data object
            $mime.= "--$mime_boundary\r\n";
            $mime.= "Content-Disposition: form-data; name=\"FILENAME\"; filename=\"snaportal-utf8-request.xml\"\r\n";
            $mime.= "Content-Type: text/xml; charset=utf-8\r\n";
            $mime.= "\r\n";
            //XML is utf-8 encoded
            $utf8_xml= $ims_data_obj->getData();
            $mime.= $utf8_xml;
            $mime.= "\r\n";
        }
        
        return $mime;
    }
    
    
}


?>
