<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005  SWITCH (http://www.switch.ch) 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: copy-role.php,v 1.7 2006/06/30 08:10:46 vlry Exp $
 */

/**
* This page duplicate an existing role.
*
* <b>Request Parameters</b>
* <ul>
*   <li>rid=<rid>: MANDATORY. The internal role ID. <rid> must be
*    a valid integer > 0 and the current administrator must be allowed
*    to edit the role with ID <rid>.
*    </li>
*  </ul>
*
* 
* @version $Revision: 1.7 $
* @package AdminPortal
*/

/*
 * Validates the input parameters 
 *
 * @param array &$input_errors  an output parameter with the error messages. The 
 *  following error messages are replies:
 *  <ul>
 *    <li>$input_errors["role_title"]: errors related to the role title
 *        (title missing)
 *    </li>
 *    <li>$input_errors["role_url"]: errors related to the role URL
 *        (URL missing)
 *    </li>
 *  </ul>
 * 
 * @return boolean  true, if the input is valid; false, otherwise.
 *   The out-parameter $input_errors contains specific error messages
 *   for each parameter.
 * 
 * @access private
 */
function validate_input( &$input_errors) {
    $valid= true;
    $title= $_REQUEST["role_title"];
    $title= trim($title);
    if ( empty($title) ) {
        $valid = false;
        $input_errors["role_title"] = "Please enter a non-empty Title.";    
    }
    
    $url= $_REQUEST["role_url"];    
    $url= trim($url);
    if ( empty($url) ) {
        $valid= false;
        $input_errors["role_url"] = "Please enter a non-empty URL.";    
    }
  
    $adapter= $_REQUEST["role_adapter"];
    $adapter= trim($adapter);
    if ( empty($adapter) ) {
        $valid= false;
        $input_errors["role_adapter"]= "Please choose a Role type.";    
    }
  
    return $valid;
}

/**
 * Creates a new role from the values provided
 *   in the $_REQUEST object 
 *
 * @param Administrator $admin       the current administrator user (an instance
 *                         of class User)
 *
 * @return  Role  an initialized role already written to the database
 * 
 * @access private
 */
function & createRole(&$admin) {  

   $provider_id= $admin->getID();
   $url= $_REQUEST["role_url"];
   $title= $_REQUEST["role_title"];

   $res=& Role::create($provider_id, $url, $title );

   $res->setDescription($_REQUEST["role_description"]);
   $res->setRoleAdapter($_REQUEST["role_adapter"]);
   $res->setVisible((boolean)$_REQUEST["role_is_visible"]);
   $res->setRoleAccessState((int)$_REQUEST["role_state"]);
	
	// === ANWANDER ===
	$all_actions= Role::getAllActions();
	$i = 0;
	foreach($all_actions as $all_actions):
	$action_id= $all_actions['action_id'];
	$res->setRoleActionState($action_id,(int)trim($_REQUEST["action_id_$action_id"]));
	endforeach;
	// ================	

   $res->persist();
   return $res;
}

/**
 * Updates the adapter parameters   
 *
 * @param Role  &$res  the role object whose role adapter
 *    parameters are updated.  
 *
 * @access private
 */
function updateRoleAdapterParameters(&$res) {
  $res->deleteAdapterParameters();
  foreach($_REQUEST as $name => $value) {
    if (ereg("^ap-(.*)$", $name, $reg)) {
      $name= $reg[1];
      $value= trim( $value );
      if (!empty($value)) {
        $res->insertAdapterParameter($name, $value);
      }
    }
  }
}

/**
 * Copies the managers from a role to another.
 * @access private
 */
function copyRoleAdministatorsFromTo(&$from_res, &$to_res) {
    $provider_id= $to_res->getRoleProviderID();
    $admin_ids= $from_res->getRoleManagerIDs();
    foreach($admin_ids as $admin_id) {
        //XXX print "XXX:addRoleManagerID($admin_id)<br>";
        // don't set provider as admin
        if ($admin_id != $provider_id) {
            $to_res->addRoleManagerID($admin_id);
        }
    }
}

/**
 *
 * @access private
 */
function copyRolePolicyAttributesFromTo(&$from_res, &$to_res) {
    $policy_atids= $from_res->getPolicyAttributesIDs();
    foreach($policy_atids as $policy_atid) {
        //XXX print "XXX:addToPolicy($policy_atid)<br>";
        $to_res->addToPolicy((int)$policy_atid);
    }
}



// bootstrap application
//
require_once("include/application.inc");

// check mandartory parameter
if (! isset($_REQUEST["rid"])) {
    ErrorHandling::fatalErrorAndExit("Request parameter 'rid' is missing"); 
}

//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("Current user is not priviledged to enter this page"); 
}
Session::init($admin);

$rid= $_REQUEST["rid"];
$role=& Role::reconstituteById($rid);
if (is_null($role)) {
  ErrorHandling::fatalErrorAndExit("Illegal role id ($rid) submitted");
}

if (! $admin->canDuplicateRole($role)) {
   ErrorHandling::fatalErrorAndExit("You do not have enough privileges to duplicate role ($rid)");   
}

//
// process duplicate operation
//
$input_errors = array();
if (isset($_REQUEST['do_duplicate'])) {
  if (validate_input(&$input_errors)) {
     $new_role=& createRole($admin);
     copyRoleAdministatorsFromTo($role,$new_role);
     copyRolePolicyAttributesFromTo($role,$new_role);
     updateRoleAdapterParameters($new_role);
     
     header("Location: " . PORTAL_BASE_URL . "/admin/edit-role.php?rid=" . $new_role->getID());
     exit;
  }
}

// duplicate the role
if (isset($_REQUEST['role_title'])) {
    $role_title= $_REQUEST['role_title'];
}
else {
    $role_title= "Copy of " . $role->getTitle();
}
if (isset($_REQUEST['role_url'])) {
    $role_url= $_REQUEST['role_url'];
}
else {
    $role_url= $role->getURL();
}
if (isset($_REQUEST['role_description'])) {
    $role_description= $_REQUEST['role_description'];
}
else {
    $role_description= $role->getDescription();
}
if (isset($_REQUEST['role_is_visible'])) {
    $role_is_visible= $_REQUEST['role_is_visible'];
}
else {
    $role_is_visible= $role->isVisible();
}
if (isset($_REQUEST['role_state'])) {
    $role_state= $_REQUEST['role_state'];
}
else {
    $role_state= $role->getRoleAccessState();
}
// handle role adapter switch
$role_adapter= $role->getRoleAdapter();
if (isset($_REQUEST['action']) and $_REQUEST['action'] == 'change_adapter' and
    isset($_REQUEST['role_adapter']) ) {
    $role_adapter= $_REQUEST['role_adapter'];
}
if ($role->getRoleAdapter() == $role_adapter ) {
    $role_adapter_params=& $role->getAdapterParameters();
}
else {
    $role_adapter_params= array();
}

	// === ANWANDER ==================
	$checkedActions = array();
	$all_actions= Role::getAllActions();
	foreach($all_actions as $all_actions){
		$action_id= $all_actions['action_id'];
		$checkedActions[$action_id] = isset($_REQUEST["action_id_$action_id"]);
	}
	// ===============================


/* ****************************************************************** */
/*                     page content                                   */
/* ****************************************************************** */
include("include/adminHeader.inc");

?>
<script>
<!--
function change_adapter() {
  document.copy_role.action.value= "change_adapter";
  document.copy_role.submit();
}
// -->
</script>

<h1>Duplicate Role</h1>

  <p>Edit/modify the parameters and click on <span class="button-reference">Save</span> to validate.
  Fields marked with
  <img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">
  are mandatory.<br>
  The <b>Role Administrators</b>, <b>Standard Attributes</b> and <b>Custom Attributes</b> of the original role are also copied.
  <p/>
  
  
  <form name="copy_role" action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
  
  <input type="hidden" name="rid" value="<?= $rid ?>" />
  <input type="hidden" name="action" value="" />

  <table width="90%" border="0" cellspacing="0" class="form-table">
    
    <tr>
      <td width="30%" valign="top" class="form-label form-first">
      Role Title
      </td>
      <td width="70%" valign="top" class="form-entry form-first">
      <input type="text" name="role_title" value="<?= $role_title ?>" size="40"/>
      <img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">
      <?php if ( isset($input_errors["role_title"]) ): ?>
        <br/><span class="error-message"><?= $input_errors["role_title"] ?></span>
      <?php endif; ?>
      </td>
    </tr>

    <tr>
      <td width="30%" valign="top" class="form-label">
      Role URL
      </td>
      <td width="70%" valign="top" class="form-entry">
      <input type="text" name="role_url" value="<?= $role_url ?>" size="40" />
      <img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">
      <?php if ( isset( $input_errors["role_url"] ) ): ?>
      <br/><span class="error-message"><?= $input_errors["role_url"] ?></span>
      <?php endif; ?>

      </td>
    </tr>

    <tr>
      <td width="30%" valign="top" class="form-label">
      Role Description
      </td>
      <td width="70%" valign="top" class="form-entry">
      <textarea name="role_description" cols="40" rows="5"><?= $role_description ?></textarea>
      </td>
    </tr>

    <tr>
      <td width="30%" valign="top" class="form-label">
      Role Visibility
      </td>
      <td width="70%" valign="top" class="form-entry">
      <input type="radio" name="role_is_visible" value="1" <?= $role_is_visible ? "CHECKED" : "" ?> /> Visible
      <input type="radio" name="role_is_visible" value="0" <?= $role_is_visible ? "" : "CHECKED"?> /> Invisible
      </td>
    </tr>
<!-- ************* Role Access State   ********************** -->
    <tr>
      <td width="30%" valign="top" class="form-label">
      Role Subscription State
      </td>
      <td width="70%" valign="top" class="form-entry">
      <input type="radio" name="role_state" value="3" <?= $role_state == 3 ? "CHECKED" : "" ?> /> Free
      <input type="radio" name="role_state" value="2" <?= $role_state == 2 ? "CHECKED" : "" ?> /> Managed
      <input type="radio" name="role_state" value="1" <?= $role_state == 1 ? "CHECKED" : "" ?> /> Suspended
      <input type="radio" name="role_state" value="0" <?= $role_state == 0 ? "CHECKED" : "" ?> /> Closed           
      </td>
    </tr>
<!-- ************* Action  ************************** -->

<tr>
<td width="30%" valign="top" class="form-label">
Allowed Actions 
</td>
<td width="100%" class="form-entry">


<table>
<tr>
<th class="data last">State</th>
<th class="data last">Action</th>
<th class="data last">Title</th>
</tr>
<?

//   ANWANDER
$all_actions= Role::getAllActions();
foreach($all_actions as $all_actions):
$action_name= $all_actions['action_name'];
$action_title= $all_actions['action_title'];
$action_id= $all_actions['action_id'];
$style= ($i++ % 2 == 0) ? "data-even" : "data-odd";
$is_selected = $checkedActions[$action_id];
?>

<tr>
<td valign="top" class="<?= $style ?>">
<input type="checkbox" name="action_id_<?= $action_id  ?>" value="1" <?= $is_selected ? "CHECKED" : "" ?>/>
</td>
<td valign="top" class="<?= $style ?>">
<?= $action_name ?> 
</td>
<td valign="top" class="<?= $style ?>">
<?= $action_title ?>
</td>
</tr>

<?php endforeach; ?>

</table>

</td>	 
</tr>

<!-- ************* Role Adapter  ************************** -->
<tr>
      <td width="30%" valign="top" class="form-label">
      Role Adapter
      </td>
      <td width="70%" valign="top" class="form-entry">
      <select name="role_adapter" onChange="change_adapter()">
      <?php
      $roleAdapters= Adapting::listRoleAdaptors();
      ?>
      <?php foreach ($roleAdapters as $adapterConfig): ?>
         <?php $selected = ($adapterConfig['name'] == $role_adapter) ? " selected " : ""; ?>
         <option value="<?= $adapterConfig['name'] ?>" <?= $selected ?> ><?= $adapterConfig['display-name'] ?></option>
      <?php endforeach; ?>
      </select> <img src="<?= PORTAL_BASE_URL ?>/lookandfeel/current/mandatory.gif">

      <?php if (isset($input_errors["role_adapter"])): ?>
      <br/><span class="error-message"><?= $input_errors["role_adapter"] ?></span>
      <?php endif; ?>
      <?php 
      $help= Adapting::getAdapterHelp($role_adapter);
      if ($help) {
        echo "<p>$help</p>";
      }

      $parameters=& Adapting::getAdapterParameterDescriptors($role_adapter);
      // adapter parameters form
      if (is_null($parameters)) {
         // no parameters
         $parameters= array();
      }
      else {
        echo "<p style='font-weight: bold;'>This Role requires you to provide additional parameters. ";
        echo "Please fill in the following fields.<p/>";
      }
      ?>  
      <table width="100%">
        <?php foreach($parameters as $pardesc_id => $pardesc): ?>
        <tr>
          <td valign="top" class="form-entry form-first" style="font-weight: bold;">
          <?= $pardesc["display-name"] ?>
          </td>
          <?php 
              $param_value= '';
              if (isset($role_adapter_params[$pardesc_id])) {
                  $param_value= $role_adapter_params[$pardesc_id];                
              }
              // default value?
              if (isset($pardesc['default-value']) and empty($param_value)) {
                  $param_value= trim($pardesc['default-value']);
              }
              
              $input_type= "text";
              $checked= '';
              if( eregi('SECRET',$pardesc_id) or eregi('PASSWORD',$pardesc_id) ) {
                  $input_type= "password"; 
              }
              elseif ( eregi('ENABLE', $pardesc_id) ) {
                  $input_type= "checkbox";
                  if (!empty($param_value)) {
                      $checked= 'checked';
                  }
                  $param_value="1";
              }

           ?>
          <td valign="top" class="form-entry form-first">
          <input type="<?= $input_type ?>" <?= $checked ?> name="ap-<?= $pardesc_id ?>" size=40 value="<?= $param_value ?>">
          <br/><?= $pardesc["description"]?>
          </td>
        </tr>
        <?php endforeach; ?>
      </table> 

      </td>
    </tr>
    
    <tr>
      <td width="100%" valign="top" align="center" class="form-control" colspan="2">
      <input type="submit" name="do_duplicate" value="Save" />
      </td>    
    </tr>
  
  </table>
  </form>

  

<?php
// include common page footer
//
include("include/adminFooter.inc");
?>
