<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: delete-role.php,v 1.20 2005/10/17 10:17:23 vlry Exp $
 */
/* ****************************************************************** */


/**
 * delete-role.php  - removes a list of roles from the database 
 *
 * <b>Request Parameters</b>
 * <ul>
 *  <li>rid: OPTIONAL. The role ID of the role to delete
 *  <li>rid-<id>: OPTIONAL   indicates that the role with ID <id>
 *                       is to be deleted
 *  <li>url: OPTIONAL. the follow-up URL. If missing, HTTP_REFERER
 *                       is used as follow-up URL.
 * </ul>
 *
 * <b>Sample Get Request</b>
 * <pre>
 *   delete-role.php?rid=5  
 *   
 *   delete-role.php?rid-5=on
 *
 *   delete-role.php?rid-5=on&rid-22=on&url=list.php
 *
 * </pre>
 *
 * @version $Revision: 1.20 $
 * @package AdminPortal
 */

/**
*/


// bootstrap application
//
require_once("include/application.inc");

define('DEFAULT_FOLLOW_UP_URL', PORTAL_BASE_URL . "/admin/list-my-roles.php");


/* ***************************************************************** */
/*                        functions                                  */
/* ***************************************************************** */





//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);

// check if the administrator object is valid
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("current user is not priviledged to enter this page"); 
}
Session::init($admin);


// delete role identified by request parameter rid
//
if (isset($_REQUEST["rid"])) {
  $rid= $_REQUEST['rid'];
  $role =& Role::reconstituteById($rid);
  if (! is_null($role)) {
     if ($admin->canDeleteRole($role)) {
        $role->delete();
     }
  }
}

// delete role identified by request parameter(s)
// res-<id>
//
foreach($_REQUEST as $key=>$value) {
  if (ereg("^res-([0-9]+)$", $key, $regs)) { 
      $rid= $regs[1];
      $role =& Role::reconstituteById($rid);
      if (! is_null($role)) {
         if ($admin->canDeleteRole($role)) {
            $role->delete();
         }
      }       
  } 
}   

// redirect to followup URL 
//
if (isset($_REQUEST["url"])) {
  $url= $_REQUEST["url"];
} else {
  $url= $_SERVER["HTTP_REFERER"];   
}

if (! isset($url) || empty($url)) {
  $url= DEFAULT_FOLLOW_UP_URL;  
}

header("Location: $url");

?>
