<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: edit-role-aap.php,v 1.21 2005/10/11 07:55:02 vlry Exp $
 */
/* ****************************************************************** */

/**
* This page edits a role.
*
* <b>Request Parameters</b>
* <ul>
*   <li>rid=<rid>: MANDATORY. The internal role ID. <rid> must be
*    a valid integer > 0 and the current administrator must be allowed
*    to edit the role with ID <rid>.
*    </li>
*   <li>sattr-<atid>=On: OPTIONAL. If present adds the standard attribute
*      typ <atid> to the list of required attributes for this role.
*   </li>
*   <li>do_save=On: OPTIONAL. If present saves the data submitted in 
*      the HTTP request. If abstent, just loads the data from the
*      database and displays an input form.
*   </li>
*
*  </ul>
*
* 
* @version $Revision: 1.21 $
* @package AdminPortal
*/

/**
*/



/* ***************************************************************** */
/*                        functions                                  */
/* ***************************************************************** */


/* **************************************************************** */
/**
 * Ensures request parameters be valid.
 *
 * @access private
 */
function ensure_request_parameters_valid() {
  if (! isset($_REQUEST["rid"])) {
    ErrorHandling::fatalErrorAndExit("request parameter 'rid' is missing");	
  }
  $rid = $_REQUEST["rid"];
  if (! is_numeric($rid) || $rid < 0) {
    ErrorHandling::fatalErrorAndExit("illegal value '$rid' for request parameter 'rid'. Aborting.");	
  }
}

/* **************************************************************** */
/**
 * Updates the AAP for a role from the current
 *   request parameters in $_REQUEST
 * 
 * @param array $attribute_type_ids an array of attribute type IDS
 * @param Role  $role    the role whose AAP is to
 *                                   be updated 
 *
 * @access private
 */
function update_aap($attribute_type_ids, $role) {
 
        foreach ($attribute_type_ids as $atid) {
                $role->removeFromPolicy((int)$atid);
        }
 
        foreach ($_REQUEST as $key=>$value) {        	
                if (substr($key, 0, 6) == "sattr-" && $value) {                	
                	$atid=substr($key, 6);                	
                        $role->addToPolicy((int)$atid);
                }
        }
}


// bootstrap application
//
require_once("include/application.inc");

ensure_request_parameters_valid();

//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("Current user is not priviledged to enter this page"); 
}
Session::init($admin);

// make sure 'rid' identifies an existing role
$rid= $_REQUEST["rid"];
$role =& Role::reconstituteById($rid);
if (is_null($role)) {
  ErrorHandling::fatalErrorAndExit("Illegal role id '$rid' submitted");
}

// make sure the current administrator can edit this role
//
if (! $admin->canEditRole($role)) {
   ErrorHandling::fatalErrorAndExit("You do not have enough privileges to edit role ($rid)");  	
}	

// process save operation
//
if (isset($_REQUEST["do_save"])) {
  $attribute_type_ids = AttributeType::getStandardAttributeTypeIds();
  update_aap($attribute_type_ids, $role);
}


/* ****************************************************************** */
/*                     page content                                   */
/* ****************************************************************** */
include("include/adminHeader.inc");
?>

<!-- navigation menu -->
<p align="center">
  <span class="button-reference"><a href="edit-role.php?rid=<?= $rid ?>">Edit Role</a></span>    
| <span class="button-reference"><a href="edit-role-managers.php?rid=<?= $rid ?>">Edit Role Administrators</a></span>
| <span class="button-reference"><a href="edit-custom-role-aap.php?rid=<?= $rid ?>">Edit Custom Attributes</a></span>
</p>


<h1>Role Authorization Policy</h1>

  This form allows you to specify which attributes a user must
  provide when he or she wants to access the role
  <a href="edit-role.php?rid=<?= $rid ?>">
  <?= $role->getTitle() ?>
  </a>.
  <p/>
  Please select the chechbox for every attribute a user <strong>must</strong>
  provide and click on <span class="button-reference">Save</span>.
    
  <form action="<?= $_SERVER['PHP_SELF'] ?>" method="POST">
  <input type="hidden" name="do_save" value="1" />
  <input type="hidden" name="rid" value="<?= $rid ?>" />

  <table width="90%" border="0" cellspacing="0" class="form-table">
     
     <tr>
       <td colspan="2" class="form-head">
       Standard Attributes
       </td>
     </tr>
     <?php
     $attribute_type_ids = AttributeType::getStandardAttributeTypeIds();
     ?>
     <?php foreach ($attribute_type_ids as $atid): ?>
     <?php 
        $atid = (int)$atid;
        $attribute_type_name = AttributeType::getAttributeTypeName($atid);
        $checked = ($role->isFromPolicy($atid)) ? "checked" : "";
     ?>
    <tr>
      <td width="30%" valign="top" class="form-label ">
      <?= $attribute_type_name ?>
      </td>
      <td width="70%" valign="top" class="form-entry">
      <input type="checkbox" name="sattr-<?= $atid ?>" <?= $checked ?> />
      </td>
    </tr>    
    <?php endforeach; ?>

    <tr>
      <td width="100%" valign="top" align="center" class="form-control" colspan="2">
      <input type="submit" value="Save" />
      </td>    
    </tr>
    
  </table>
  </form>

  

<?php
// include common page footer
//
include("include/adminFooter.inc");
?>
