<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: show-role.php,v 1.27 2006/03/28 14:19:21 vlry Exp $
 */

/**
* This page displays detail information about a role.
*
* <b>Request Parameters</b>
* <ul>
*   <li>rid=<rid>: MANDATORY. The internal role ID. <rid> must be
*    a valid integer > 0 and the current administrator must be allowed
*    to edit the role with ID <rid>.
*    </li>
*  </ul>
*
* 
* @version $Revision: 1.27 $
* @package AdminPortal
*/

/**
 * Ensures that request parameters
 *   are valid 
 *
 * @access private  
 */
function ensure_request_parameters_valid() {
  if (! isset($_REQUEST["rid"])) {
    ErrorHandling::fatalErrorAndExit("Request parameter 'rid' is missing"); 
  }
  $rid = $_REQUEST["rid"];
  if (! is_numeric($rid) || $rid < 0) {
    ErrorHandling::fatalErrorAndExit("Illegal parameter rid ($rid).");    
  }
}


// bootstrap application
//
require_once("include/application.inc");

ensure_request_parameters_valid();

//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
   ErrorHandling::fatalErrorAndExit("You have don't have enough priviledge to enter this page."); 
}
Session::init($admin);

$rid= $_REQUEST['rid'];
$role=& Role::reconstituteById($rid);
if (is_null($role)) {
  ErrorHandling::fatalErrorAndExit("Illegal role id ($rid) submitted.");
}

if (! $admin->canViewRole($role)) {
   ErrorHandling::fatalErrorAndExit("You have don't have enough priviledge to view this role ($rid).");   
}   


/* ****************************************************************** */
/*                     page content                                   */
/* ****************************************************************** */
include("include/adminHeader.inc");
?>

<h1>Role Details</h1>
  
  <table width="90%" border="0" cellspacing="0" class="form-table">

    <tr>
      <td width="30%" valign="top" class="form-label form-first">
      Role Key
      </td>
      <td width="70%" valign="top" class="form-entry form-first">
      <?= $role->getRoleKey() ?>
      </td>
    </tr>
    
    <tr>
      <td width="30%" valign="top" class="form-label">
      Role Title
      </td>
      <td width="70%" valign="top" class="form-entry">
      <?= $role->getTitle() ?>
      </td>
    </tr>

    <tr>
      <td width="30%" valign="top" class="form-label">
      Role URL
      </td>
      <td width="70%" valign="top" class="form-entry">
      <?= $role->getUrl() ?>
      </td>
    </tr>

    <tr>
      <td width="30%" valign="top" class="form-label">
      Role Description
      </td>
      <td width="70%" valign="top" class="form-entry">
      <?= $role->getDescription() ?>
      </td>
    </tr>

    <tr>
      <td width="30%" valign="top" class="form-label">
      Role Visible ?
      </td>
      <td width="70%" valign="top" class="form-entry">
      <?php 
        if ($role->isVisible()) {
            echo "Visible";
          } else {
            echo "Invisible";
          }      
      ?>
      </td>
    </tr>

    <tr>
      <td width="30%" valign="top" class="form-label">
      Role open for Subscription ?
      </td>
      <td width="70%" valign="top" class="form-entry">
      <?php
        if ($role->isOpen()) {
            echo "Managed";
        } 
        else if ($role->isFree()) {
            echo "Free";
        }
        else {
            echo "No";
        }      
      ?>
      </td>
    </tr>

    <tr>
      <td width="30%" valign="top" class="form-label">
      Role Adapter 
      </td>
      <td width="70%" valign="top" class="form-entry">
      <?php
        $adapter= $role->getRoleAdapter();
        $adapter_name= Adapting::getAdapterDisplayName($adapter); ?>
      <?= $adapter_name ?>      
      </td>
    </tr>
  
  
  <!-- PROVIDER -->
   <tr>
      <td width="30%" valign="top" class="form-label form-first">
      Role Provider 
      </td>
      <td width="70%" valign="top" class="form-entry form-first">
      <?php
        $provider= $role->getRoleProvider();
        $p_attrs= $provider->getUserAttributes(ATTR_COMMONNAME, ATTR_PRIVILEGENAME, ATTR_EMAIL, ATTR_PHONE);
      ?>
      <?= $p_attrs[ATTR_COMMONNAME] ?> (<?= $p_attrs[ATTR_PRIVILEGENAME] ?>)
      </td>
    </tr>
   <tr>
      <td width="30%" valign="top" class="form-label">
      Provider Contact 
      </td>
      <td width="70%" valign="top" class="form-entry">
      <?php if (!empty($p_attrs[ATTR_PHONE])): ?>
      Phone: <?= $p_attrs[ATTR_PHONE] ?> 
      <?php endif; ?>
      <?php if (!empty($p_attrs[ATTR_EMAIL])):
         $email= $p_attrs[ATTR_EMAIL]; ?>
      Email: <a href="mailto:<?= $email ?>">&lt;<?= $email ?>&gt;</a>
      <?php endif; ?>
      </td>
    </tr>

    <!-- ADMINISTRATORS -->    
    <?php
    $admin_ids= $role->getRoleManagerIDs();
    if ( !empty($admin_ids) ):
        foreach($admin_ids as $admin_id):
            $admin= Administrator::reconstituteById($admin_id);
            $admin_attrs= $admin->getUserAttributes(ATTR_COMMONNAME, ATTR_PRIVILEGENAME, ATTR_EMAIL, ATTR_PHONE);
            ?>
            <tr>
              <td width="30%" valign="top" class="form-label form-first">
              Role Administrator 
              </td>
              <td width="70%" valign="top" class="form-entry form-first">
              <?= $admin_attrs[ATTR_COMMONNAME] ?> (<?= $admin_attrs[ATTR_PRIVILEGENAME] ?>)
              </td>
            </tr>
           <tr>
              <td width="30%" valign="top" class="form-label">
              Administrator Contact 
              </td>
              <td width="70%" valign="top" class="form-entry">
              <?php if (!empty($admin_attrs[ATTR_PHONE])): ?>
              Phone: <?= $admin_attrs[ATTR_PHONE] ?> 
              <?php endif; ?>
              <?php if (!empty($admin_attrs[ATTR_EMAIL])):
                 $email= $admin_attrs[ATTR_EMAIL]; ?>
              Email: <a href="mailto:<?= $email ?>">&lt;<?= $email ?>&gt;</a>
              <?php endif; ?>
              </td>
            </tr>
        <?php endforeach; ?>
    <?php endif; ?>
  </table>
  

<?php
// include common page footer
//
include("include/adminFooter.inc");
?>
