<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: start-page.php,v 1.22 2005/12/07 16:35:31 vlry Exp $
 */
/* ****************************************************************** */

/**
* This is the start page for the administration part of the SNA-Portal.
*
* 
* @version $Revision: 1.22 $
* @package AdminPortal
*/


/* **************************************************************** */
/*                          constants                               */
/* **************************************************************** */
/**
 * the max. number of roles with pending subscription requests shown
 * on this start page
 */
define(MAX_NUM_PENDING_RESOURCES,  10);

/* **************************************************************** */
/*                          functions                               */
/* **************************************************************** */


/* **************************************************************** */
/**
 * Replies the number of pending subscriptions for roles owned by user $uid 
 *
 * @param MySQLConnection $conn  an open MySQL connection
 * @param int  $uid   a user id 
 *
 * @return array  an array of Subscription records. 
 *
 * @access private
 *
 */
function get_num_pending_subscriptions($conn, $uid) {
  $query = " SELECT distinct count(*)  "
         . " FROM Subscription, Role  "
         . " WHERE Role.role_user  = $uid "
         . "   AND Role.role_id  = Subscription.subscription_role "         
         . "   AND Subscription.subscription_state =  " . SUBSCRIPTION_STATE_PENDING . " ";
         

  $conn->query($query);
  $row = $conn->fetch_array();
  return $row[0];
}

/* **************************************************************** */
/**
 * Replies a list of of roles with pending subscriptions 
 *
 * @param MySQLConnection $conn  an open MySQL connection
 * @param int  $uid  a user id 
 *
 * @return array an array of Role records
 *
 * @access private
 *
 */
function get_roles_with_pending_subscriptions($conn, $uid) {

  $query = " SELECT Role.*, "
         . "        count(*) as num_subscriptions "
         . " FROM Role, Subscription "
         . " WHERE Role.role_user = $uid "
         . "   AND Role.role_id = Subscription.subscription_role "         
         . "   AND Subscription.subscription_state =  " . SUBSCRIPTION_STATE_PENDING . " "
         . " GROUP BY Role.role_id ";
  
  $conn->query($query);
  $ret = array();
  $i = 0; 
  while($row = $conn->fetch_array()) {
    $ret[$i] = $row;
    $i++;	
  } 	
  $conn->freeresult();
  return $ret;
}


// bootstrap application
//
require_once("include/application.inc");


//
// resume the current session and make sure we have a valid session
//
Session::ensureAdminSessionValid();

$uid= Session::getUID();
$admin=& Administrator::reconstituteByUid($uid);
if (is_null($admin)) {
  ErrorHandling::fatalErrorAndExit("Current user is not priviledged to enter this page"); 
}
Session::init($admin);

//TODO: replace embedded SQL!
$conn=& MySQLConnection::getConnection();

/* **************************************************************** */
/*                     HTML output                                  */
/* **************************************************************** */
include("include/adminHeader.inc");
?>

  <p class="welcome-title">Welcome to the SNAportal</p>

  This is your personal start page for administering the SNA-Portal.
  
  <p/>
  <?php
    $num_pending = get_num_pending_subscriptions($conn, $admin->getUserId());
  ?>  
  <?php if ($num_pending == 0): ?>
  There are no pending subscription requests for your roles.
  <p/>
  
  Use the menu on the left to manage your roles.
  
  <?php else: ?>
  There
  are <strong><?= $num_pending ?></strong> pending subscription requests
  to process.
  <p/>
 
  
  <table width="90%" border="0" cellspacing="0" class="data-table">

    <tr>
      <th class="data" width="40%" valign="center">Role Title</th>
      <th class="data" width="10%"># Subs. Requests</th>
      <th class="data last" width="50%">Actions</th>
    </tr>
    
    <?php
      $roles = get_roles_with_pending_subscriptions($conn, $admin->getUserId());
      $i = 0;
      $c= 0;
      foreach ($roles as $res_data):
      	if ($i++ >= MAX_NUM_PENDING_RESOURCES) break;
        $style = ($c++ % 2 == 0) ? "data-even" : "data-odd";
    ?>  
    <tr>
      <td width="40%" valign="top" class="<?= $style ?>">
        <?= Formatting::formatTableCellContent($res_data["role_title"]) ?>
      </td>
      <td width="10%" valign="top" class="<?= $style ?>"> 
      <?= Formatting::formatTableCellContent($res_data["num_subscriptions"]) ?>
      </td>
      <td width="50%" valign="top" class="<?= $style ?> last">
      <a href="edit-role.php?rid=<?= $res_data["role_id"] ?>">Edit</a>
      | <a href="list-subscriptions.php?rid=<?= $res_data["role_id"] ?>">Process Subscription Requests</a>
      </td>
    </tr>
    <?php endforeach; ?>
    <?php if (count($roles) > MAX_NUM_PENDING_RESOURCES): ?> 
    <tr>
      <td colspan=3 valign="top" class="data-more">
       <a href="list-my-roles.php">More roles with pending subscription requests ...</a>
      </td>
    </tr>
    <?php endif; ?>
  
  </table>
  
  <?php endif;  ?>

<?php
// include page footer 
include("include/adminFooter.inc");
?>
