<?php

/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch) 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: invitation.php,v 1.13 2006/09/29 08:37:57 vlry Exp $
 */


/**
* This page is the Shibboleth protected start page for the user part of
* the SNA-Portal. After a user authenticates at its Home Organization (HO)
* and after the local Shibboleth Attribute Requester (SHAR) retrieved
* its attributes from the Home Organization, this page is invoked.
*
* The Invitations are handled by this script. First the invitation token
* is check. If valid, the user is reconstituted or created, the session initialized
* and the user redirected to the role.
*
* @version $Revision: 1.13 $
* @package UserPortal
* @subpackage AAI
*/

require_once("include/application.inc");

//
// get unique id (AAI)
//
if (! isset($_SERVER['Shib-WISEBEDPerson-personUniqueID']) or empty($_SERVER['Shib-WISEBEDPerson-personUniqueID'])) {
    // no unique id from shibboleth
    ErrorHandling::fatalErrorAndExit("AAI HTTP header Shib-WISEBEDPerson-personUniqueID missing or empty.");
}
$aai_uniqueid= $_SERVER['Shib-WISEBEDPerson-personUniqueID'];

//
// check request parameters: rid and invitation token 
//
if (! isset($_REQUEST['rid']) or !isset($_REQUEST['token']) ) {
    // no rid, or token in request
    ErrorHandling::fatalErrorAndExit("Request parameter 'rid' and/or invitation 'token' missing.");
}

$token= $_REQUEST['token'];
$rkey= $_REQUEST['rid'];


//
// reconstitute the role.
//
$role= Role::reconstituteByKey($rkey);
if (is_null($role)) {
    Event::storeSystemEvent(
        "Invitation contains invalid role key: $rkey user: $aai_uniqueid",
        EVENT_LEVEL_ERROR);
    ErrorHandling::fatalErrorAndExit(
        "Invalid Role ($rkey).",
        "<p>The System was not able to give you access to the role  because:"
       ."<ul><li>the URL is invalid or wrong (e.g. truncated, contains blanks,...).<br>"
       ."Try to correct the URL (remove space %20, edit the bookmark, copy/paste the original, ...)</li>"
       ."<li>If you received an Invitation by email, try to access the role by clicking on the Login URL annotated in the Invitation."
       ."</li></ul></p>"
   );
}


//
// check if invitation is a valid one
//
$rid= $role->getID();
$valid= Invitation::isValid($token,$rid);
if (! $valid ) {
    Event::storeSystemEvent(
        "Invitation: $token is not valid for role: $rkey user: $aai_uniqueid",
        EVENT_LEVEL_ERROR);
    ErrorHandling::fatalErrorAndExit(
      "The Invitation ($token) is not valid for Role ($rkey).",
      "<p>The System was not able to register you to the role because:"
     ."<ul><li>the Invitation URL was used before and is not valid anymore (only valid once). "
     ."In this case try to access the role with the Login URL annotated in your invitation  email.</li>"
    ."<li>the URL is invalid or wrong (e.g. truncated, contains blanks,...).<br>"
    ."Try to correct the URL (remove space %20, edit the bookmark, copy/paste the original from the Invitation email, ...)."
    ."</li></ul></p>"
    );
}


//
// reconstitute an existing user or create a new one.
// 
$user= User::reconstituteByUid($aai_uniqueid);
// if there is no such user yet, create one
if (is_null($user)) {
  $user=& User::create($aai_uniqueid);
  if (is_null($user)) {
    Event::storeSystemEvent(
        "Failed to create AAI user: $aai_uniqueid",
        EVENT_LEVEL_ERROR); 
    ErrorHandling::fatalErrorAndExit("Failed to create new AAI user ($aai_uniqueid).");
  }
}

//
// reconstitute or create a new subscription for this role and accept it.
//
$subscription=& Subscription::reconstitute($user,$role);
if (is_null($subscription)) {
    $subscription=& Subscription::create($user,$role);
    if (is_null($subscription)) {
        $user_id= $user->getUserID();
        Event::storeSystemEvent(
            "Failed to create new subscription: user: $aai_uniqueid role: " . $role->getTitle() . " ($rid)",
            EVENT_LEVEL_ERROR); 
        ErrorHandling::fatalErrorAndExit("Failed to create new Subscription ($user_id, $rid).");
    }
}
$subscription->accept();

//
// delete the invitation from DB
//
$deleted= Invitation::delete($token,$rid);
if (!$deleted) {
    ErrorHandling::fatalErrorAndExit("Failed to delete Invitation ($token,$rid).");
}


//
// start and init the user session
//
Session::create();
Session::init($user);

//
// redirect to the role.
//
$location= PORTAL_BASE_URL . "/user/go.php?rid=$rid";

// optional req param courseid for WebCT adapters
if ( isset($_REQUEST['courseid']) ) {
   $courseid= $_REQUEST['courseid'];
   $location= "$location&courseid=$courseid";
}

header("Location: $location");

?>
