<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: go.php,v 1.34 2007/03/06 17:22:02 vlry Exp $
 */
/* ****************************************************************** */

/**
 * @package UserPortal
 */


// bootstrap application
//
require_once("include/application.inc");

// start session
//
Session::ensureUserSessionValid();

if (! isset($_REQUEST["rid"])) {
  	ErrorHandling::fatalErrorAndExit("Request parameter 'rid' is missing.");
} 
$rid= $_REQUEST["rid"];
if (! is_numeric($rid) || $rid < 0) {
  ErrorHandling::fatalErrorAndExit("Illegal request parameter rid=$rid.");
}

//
// make sure the current user exists
//
$uid= Session::getUID();
$user=& User::reconstituteByUid($uid);
if (is_null($user)) {
  ErrorHandling::fatalErrorAndExit("Can not reconstitute User ($uid).");
}

// create a role object 
$res=& Role::reconstituteById($rid);
if (is_null($res)) {
  ErrorHandling::fatalErrorAndExit("Can not reconstitute Role ($rid).");
}

// lookup subscription object
$subscription=& Subscription::reconstitute($user, $res);
if (is_null($subscription)) {
    if ($res->isFree()) {
        // free role -> create new subs and accept
        Event::storeSubscriptionEvent("Free subscription created: $uid -> $rid", $user->getID());
        $subscription=& Subscription::create($user, $res);
        $subscription->accept();
    }
    else {
        Event::storeAccessEvent("User: $uid not subscribed to role: ". 
                                $res->getTitle() . " ($rid)", 
                                $user->getID(), EVENT_LEVEL_ERROR);
        ErrorHandling::fatalErrorAndExit(
            "Subscription ($uid,$rid) does not exist.",
            "<p>The System was not able to give you access to the role because:"
           ."<ul>"
           ."<li>you are not subscribed to the role.</li>"
           ."<li>you did not used the Invitation URL you received in a separate email. In this case try to "
           ."access the ressource with the Invitation URL annotated in your invitation email</li>"
           ."<li>the URL used is invalid or wrong (e.g. truncated). Try once again using a valid URL.</li>"
           ."</ul></p>"    
        );
    }
}

// check Role and Subscription state
if ( ! $subscription->isRoleOpen() ) {
  ErrorHandling::fatalErrorAndExit("Role ($rid) is not open.");
}
if ( ! $subscription->isAccessGranted() ) {
  Event::storeAccessEvent(
    "User: $uid subscription still pending, rejected or supsended for role: ". 
    $res->getTitle() . " ($rid)", 
    $user->getID(), EVENT_LEVEL_ERROR );

  ErrorHandling::fatalErrorAndExit(
    "Subscription ($uid,$rid) is still pending, rejected or supsended.",
	"<p>The System was not able to give you access to the role  because:"
   ."<ul>"
   ."<li>your subscription was rejected or suspended by the person who is responsible for the role</li>"
   ."<li>you did not used the Invitation URL you received in a separate email. In this case try to "
   ."access the ressource with the Invitation URL annotated in your invitation email</li>"
   ."<li>the URL used is invalid or wrong (e.g. truncated). Try once again using a valid URL.</li>"
   ."</ul></p>"    
 );
}


// get the url for the redirection php script
$adapter= $res->getRoleAdapter();
$adapterFile= Adapting::getAdapterFileName($adapter);
if ($adapterFile) {
  if ($subscription->isRoleOpen() && $subscription->isAccessGranted()) {
			
    // update the login counter
    //
    $subscription->updateLogin();
    $subscription->persist();

    // redirect the user to the adapter file
    // 
    $location= PORTAL_BASE_URL . "/$adapterFile?rid=$rid";
    
    // optional req param courseid for WebCT adapters
    if ( isset($_REQUEST['courseid']) ) {
        $courseid= $_REQUEST['courseid'];
        $location= "$location&courseid=$courseid";
    }
    
    Event::storeAccessEvent("User: $uid access role: ".
                            $res->getTitle() . " ($rid)",
                            $user->getID());
    
    header("Location: $location");
    exit;
  } 
}


include("include/header.inc");
?>

XXX: isRoleOpen: <?= $subscription->isRoleOpen() ?> <br>

Redirection to a role is not possible because the role is closed or
because your subscription has not yet been accepted. 

<?php
include("include/footer.inc");
?>


