<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: unsubscribe.php,v 1.26 2006/04/06 16:18:54 vlry Exp $
 */

/**
* This page unsubscribes a user from a role.
*
* <b>Request Parameters</b>
* <ul>
*  <li>rid=<rid>: MANDATORY. An internal role ID. <rid> must be a valid
*     integer > 0. The role ID <rid> must exist and the current user
*     must be subscribed to it.
*   </li>
*   <li>url=<url>: OPTIONAL. If present, this is the URL the user is
*      redirected to after the page is executed. If absent, the
*      user is redirected to {@link UNSUBSCRIBE_POST_UNSUBSCRIPTION_URL}.
*   </li>
*
* </ul>
*
* <b>Configuration Parameters</b>
* The following configuration paramters can be used in the global configuration file
* {@link constants.inc}:
* <ul>
*
*   <li>UNSUBSCRIBE_POST_UNSUBSCRIPTION_URL: the URL a user is redirected to after having 
*        unsubscribed from the role
*   </li>
* </ul>
*
* @version $Revision: 1.26 $
* @package UserPortal
*/

// bootstrap application
// 
require_once("include/application.inc");

// start session and ensure access restrictions
Session::ensureUserSessionValid();


// the default follow up URL
// 
if (! defined("UNSUBSCRIBE_POST_UNSUBSCRIPTION_URL")) {
  define('UNSUBSCRIBE_POST_UNSUBSCRIPTION_URL',  PORTAL_BASE_URL . "/user/my-roles.php");
}

//
// handle do_cancel
if (isset($_REQUEST['do_cancel'])) {
    $location= UNSUBSCRIBE_POST_UNSUBSCRIPTION_URL;
    header("Location: $location");
    exit;
}

//
// make sure the current user exists
//
$uid= Session::getUID();
$user=& User::reconstituteByUid($uid);
if (is_null($user)) {
    ErrorHandling::fatalErrorAndExit("No user with UID($uid).");
}

// check request parameters
//
if (!isset($_REQUEST["rid"])) 
{
   ErrorHandling::fatalErrorAndExit("Request parameter 'rid=' is missing");
}
$rid= $_REQUEST["rid"];    // the role ID 
if ( ! is_numeric($rid) || $rid < 0) {
   ErrorHandling::fatalErrorAndExit("Request parameter has an illegal value: 'rid=$rid'");
}

$role =& Role::reconstituteById($rid);
if (is_null($role)) {
  ErrorHandling::fatalErrorAndExit("No role with ID($rid).");
}

// check whether the user is subscribed to this role
//
if (! $user->isSubscribedTo($rid))
{
  ErrorHandling::fatalErrorAndExit("You are not subscribed to role ID($rid).");
}

//TODO
// Confirmation
// handling confirmation token

$confirm_token= md5("unsubscribe:$uid:$rid:" . MYSQL_PASS);

if ( isset($_REQUEST['confirm']) and $_REQUEST['confirm'] == $confirm_token) {

    // unsubscribes from the role 
    $user->unsubscribeFromRole($rid);
    
    // send out notification message to role provider and administrators.
    $message= new Messaging();
    if ($message->is_enabled()) {
        /* get email of role and user provider */
        $user_attrs= $user->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
        $owner=& $role->getRoleProvider();
        $owner_attrs= $owner->getUserAttributes(ATTR_EMAIL, ATTR_COMMONNAME);
        $admins_attrs= $role->getRoleAdministratorsAttributes();
        $to_emails= array(); // ( email => name, ... )
        if (isset($owner_attrs[ATTR_EMAIL])) {
            $to_emails[$owner_attrs[ATTR_EMAIL]]= $owner_attrs[ATTR_COMMONNAME];
        }
        foreach($admins_attrs as $admin_attrs) {
            if (isset($admin_attrs[ATTR_EMAIL])) {
                $to_emails[$admin_attrs[ATTR_EMAIL]]= $admin_attrs[ATTR_COMMONNAME];
            }
        }
        if (isset($user_attrs[ATTR_EMAIL]) and !empty($to_emails)) {
            $from= $user_attrs[ATTR_EMAIL];
            // to: CSV of emails
            $to= implode(array_keys($to_emails),",");
            // parameters for notification
            $data= array();
            $data['from_name']= $user_attrs[ATTR_COMMONNAME];
            $data['from_uid']= $uid;
            $data['role_name']= $role->getTitle();
            $data['role_key']= $role->getRoleKey();
            $data['to_name']= implode(array_values($to_emails),", ");
            $message->stateChanged($from, $to, CLOSE_SUBSCRIPTION, false, $data);
        }
    }
    
    if (! empty($_REQUEST["url"])) {
        $url = $_REQUEST["url"];    
    }else {
        $url = UNSUBSCRIBE_POST_UNSUBSCRIPTION_URL;
    }
    
    header("Location: $url");
    exit;
}
else {
    // no unsubscribe confirm token
    // show HTML page with confirmation form
    I18N::initLocale();
    I18N::loadMessageBundle("user/unsubscribe");

    include("include/header.inc");
}
?>

  <h1><?= I18N::getMessage("unsubscribe.pagetitle") ?></h1>

  <p><?= I18N::getMessage("unsubscribe.warning") ?></p>

  <p><?= I18N::getMessage("unsubscribe.confirmation") ?></p>

  <form name="unsubscribe" action="unsubscribe.php" method="POST">
  <table width="50%" border="0" cellspacing="0" class="form-table">
    <tr>
      <td colspan="2" align="center" class="form-head">
        <?= I18N::getMessage("unsubscribe.form.title") ?>
      </td>    
    </tr>
    <tr>
      <td class="form-label">
        <?= I18N::getMessage("unsubscribe.form.role_title") ?>
      </td>    
      <td class="form-entry">
        <?= $role->getTitle() ?>
      </td>    
    </tr>
    <tr>
      <td class="form-label">
        <?= I18N::getMessage("unsubscribe.form.role_description") ?>
      </td>    
      <td class="form-entry">
        <?= $role->getDescription() ?>
      </td>    
    </tr>
    <tr>
      <td colspan="2" align="center" class="form-control">
        <input type="hidden" name="confirm" value="<?= $confirm_token ?>">
        <input type="hidden" name="rid" value="<?= $rid ?>">
        <?php
            if (! empty($_REQUEST['url'])) {
                $url= $_REQUEST['url'];    
            }else {
                $url= UNSUBSCRIBE_POST_UNSUBSCRIPTION_URL;
            }
        ?>
        <input type="hidden" name="url" value="<?= $url ?>">
        <input type="submit" name="do_cancel" value="<?= I18N::getMessage('unsubscribe.form.cancel') ?>" class="button">
        <input type="submit" name="do_unsubscribe" value="<?= I18N::getMessage('unsubscribe.form.unsubscribe') ?>" class="button">
      </td>    
    </tr>
  </table>
  </form>

<?php
// include common footer
//
include("include/footer.inc");
?>
